@@ -1,10 +1,12 @@
+// © 2016 and later: Unicode, Inc. and others.
+// License & terms of use: http://www.unicode.org/copyright.html
 /*
 *******************************************************************************
-*   Copyright (C) 2010-2011, International Business Machines
+*   Copyright (C) 2010-2012, International Business Machines
 *   Corporation and others.  All Rights Reserved.
 *******************************************************************************
 *   file name:  bytestrie.h
-*   encoding:   US-ASCII
+*   encoding:   UTF-8
 *   tab size:   8 (not used)
 *   indentation:4
 *
@@ -21,10 +23,15 @@
  */
 
 #include "unicode/utypes.h"
+
+#if U_SHOW_CPLUSPLUS_API
+
 #include "unicode/stringpiece.h"
 #include "unicode/uobject.h"
 #include "unicode/ustringtrie.h"
 
+class BytesTrieTest;
+
 U_NAMESPACE_BEGIN
 
 class ByteSink;
@@ -43,7 +50,7 @@ class UVector32;
  * There is no assignment operator.
  *
  * This class is not intended for public subclassing.
- * @draft ICU 4.8
+ * @stable ICU 4.8
  */
 class U_COMMON_API BytesTrie : public UMemory {
 public:
@@ -59,15 +66,15 @@ public:
      * the BytesTrie object is in use.
      *
      * @param trieBytes The byte array that contains the serialized trie.
-     * @draft ICU 4.8
+     * @stable ICU 4.8
      */
     BytesTrie(const void *trieBytes)
-            : ownedArray_(NULL), bytes_(reinterpret_cast<const uint8_t *>(trieBytes)),
+            : ownedArray_(nullptr), bytes_(static_cast<const uint8_t *>(trieBytes)),
               pos_(bytes_), remainingMatchLength_(-1) {}
 
     /**
      * Destructor.
-     * @draft ICU 4.8
+     * @stable ICU 4.8
      */
     ~BytesTrie();
 
@@ -75,16 +82,16 @@ public:
      * Copy constructor, copies the other trie reader object and its state,
      * but not the byte array which will be shared. (Shallow copy.)
      * @param other Another BytesTrie object.
-     * @draft ICU 4.8
+     * @stable ICU 4.8
      */
     BytesTrie(const BytesTrie &other)
-            : ownedArray_(NULL), bytes_(other.bytes_),
+            : ownedArray_(nullptr), bytes_(other.bytes_),
               pos_(other.pos_), remainingMatchLength_(other.remainingMatchLength_) {}
 
     /**
      * Resets this trie to its initial state.
      * @return *this
-     * @draft ICU 4.8
+     * @stable ICU 4.8
      */
     BytesTrie &reset() {
         pos_=bytes_;
@@ -92,18 +99,51 @@ public:
         return *this;
     }
 
+    /**
+     * Returns the state of this trie as a 64-bit integer.
+     * The state value is never 0.
+     *
+     * @return opaque state value
+     * @see resetToState64
+     * @stable ICU 65
+     */
+    uint64_t getState64() const {
+        return (static_cast<uint64_t>(remainingMatchLength_ + 2) << kState64RemainingShift) |
+            static_cast<uint64_t>(pos_ - bytes_);
+    }
+
+    /**
+     * Resets this trie to the saved state.
+     * Unlike resetToState(State), the 64-bit state value
+     * must be from getState64() from the same trie object or
+     * from one initialized the exact same way.
+     * Because of no validation, this method is faster.
+     *
+     * @param state The opaque trie state value from getState64().
+     * @return *this
+     * @see getState64
+     * @see resetToState
+     * @see reset
+     * @stable ICU 65
+     */
+    BytesTrie &resetToState64(uint64_t state) {
+        remainingMatchLength_ = static_cast<int32_t>(state >> kState64RemainingShift) - 2;
+        pos_ = bytes_ + (state & kState64PosMask);
+        return *this;
+    }
+
     /**
      * BytesTrie state object, for saving a trie's current state
      * and resetting the trie back to this state later.
-     * @draft ICU 4.8
+     * @stable ICU 4.8
      */
     class State : public UMemory {
     public:
         /**
          * Constructs an empty State.
-         * @draft ICU 4.8
+         * @stable ICU 4.8
          */
-        State() { bytes=NULL; }
+        State() { bytes=nullptr; }
     private:
         friend class BytesTrie;
 
@@ -117,7 +157,7 @@ public:
      * @param state The State object to hold the trie's state.
      * @return *this
      * @see resetToState
-     * @draft ICU 4.8
+     * @stable ICU 4.8
      */
     const BytesTrie &saveState(State &state) const {
         state.bytes=bytes_;
@@ -134,10 +174,10 @@ public:
      * @return *this
      * @see saveState
      * @see reset
-     * @draft ICU 4.8
+     * @stable ICU 4.8
      */
     BytesTrie &resetToState(const State &state) {
-        if(bytes_==state.bytes && bytes_!=NULL) {
+        if(bytes_==state.bytes && bytes_!=nullptr) {
             pos_=state.pos;
             remainingMatchLength_=state.remainingMatchLength;
         }
@@ -148,7 +188,7 @@ public:
      * Determines whether the byte sequence so far matches, whether it has a value,
      * and whether another input byte can continue a matching byte sequence.
      * @return The match/value Result.
-     * @draft ICU 4.8
+     * @stable ICU 4.8
      */
     UStringTrieResult current() const;
 
@@ -158,7 +198,7 @@ public:
      * @param inByte Input byte value. Values -0x100..-1 are treated like 0..0xff.
      *               Values below -0x100 and above 0xff will never match.
      * @return The match/value Result.
-     * @draft ICU 4.8
+     * @stable ICU 4.8
      */
     inline UStringTrieResult first(int32_t inByte) {
         remainingMatchLength_=-1;
@@ -173,7 +213,7 @@ public:
      * @param inByte Input byte value. Values -0x100..-1 are treated like 0..0xff.
      *               Values below -0x100 and above 0xff will never match.
      * @return The match/value Result.
-     * @draft ICU 4.8
+     * @stable ICU 4.8
      */
     UStringTrieResult next(int32_t inByte);
 
@@ -187,10 +227,10 @@ public:
      *   result=next(c);
      * return result;
      * \endcode
-     * @param s A string or byte sequence. Can be NULL if length is 0.
+     * @param s A string or byte sequence. Can be nullptr if length is 0.
      * @param length The length of the byte sequence. Can be -1 if NUL-terminated.
      * @return The match/value Result.
-     * @draft ICU 4.8
+     * @stable ICU 4.8
      */
     UStringTrieResult next(const char *s, int32_t length);
 
@@ -201,7 +241,7 @@ public:
      *
      * Do not call getValue() after USTRINGTRIE_NO_MATCH or USTRINGTRIE_NO_VALUE!
      * @return The value for the byte sequence so far.
-     * @draft ICU 4.8
+     * @stable ICU 4.8
      */
     inline int32_t getValue() const {
         const uint8_t *pos=pos_;
@@ -213,16 +253,16 @@ public:
     /**
      * Determines whether all byte sequences reachable from the current state
      * map to the same value.
-     * @param uniqueValue Receives the unique value, if this function returns TRUE.
+     * @param uniqueValue Receives the unique value, if this function returns true.
      *                    (output-only)
-     * @return TRUE if all byte sequences reachable from the current state
+     * @return true if all byte sequences reachable from the current state
      *         map to the same value.
-     * @draft ICU 4.8
+     * @stable ICU 4.8
      */
     inline UBool hasUniqueValue(int32_t &uniqueValue) const {
         const uint8_t *pos=pos_;
         // Skip the rest of a pending linear-match node.
-        return pos!=NULL && findUniqueValue(pos+remainingMatchLength_+1, FALSE, uniqueValue);
+        return pos!=nullptr && findUniqueValue(pos+remainingMatchLength_+1, false, uniqueValue);
     }
 
     /**
@@ -231,13 +271,13 @@ public:
      * @param out Each next byte is appended to this object.
      *            (Only uses the out.Append(s, length) method.)
      * @return the number of bytes which continue the byte sequence from here
-     * @draft ICU 4.8
+     * @stable ICU 4.8
      */
     int32_t getNextBytes(ByteSink &out) const;
 
     /**
      * Iterator for all of the (byte sequence, value) pairs in a BytesTrie.
-     * @draft ICU 4.8
+     * @stable ICU 4.8
      */
     class U_COMMON_API Iterator : public UMemory {
     public:
@@ -250,7 +290,7 @@ public:
          *                  pass the U_SUCCESS() test, or else the function returns
          *                  immediately. Check for U_FAILURE() on output or use with
          *                  function chaining. (See User Guide for details.)
-         * @draft ICU 4.8
+         * @stable ICU 4.8
          */
         Iterator(const void *trieBytes, int32_t maxStringLength, UErrorCode &errorCode);
 
@@ -263,26 +303,26 @@ public:
          *                  pass the U_SUCCESS() test, or else the function returns
          *                  immediately. Check for U_FAILURE() on output or use with
          *                  function chaining. (See User Guide for details.)
-         * @draft ICU 4.8
+         * @stable ICU 4.8
          */
         Iterator(const BytesTrie &trie, int32_t maxStringLength, UErrorCode &errorCode);
 
         /**
          * Destructor.
-         * @draft ICU 4.8
+         * @stable ICU 4.8
          */
         ~Iterator();
 
         /**
          * Resets this iterator to its initial state.
          * @return *this
-         * @draft ICU 4.8
+         * @stable ICU 4.8
          */
         Iterator &reset();
 
         /**
-         * @return TRUE if there are more elements.
-         * @draft ICU 4.8
+         * @return true if there are more elements.
+         * @stable ICU 4.8
          */
         UBool hasNext() const;
 
@@ -297,19 +337,19 @@ public:
          *                  pass the U_SUCCESS() test, or else the function returns
          *                  immediately. Check for U_FAILURE() on output or use with
          *                  function chaining. (See User Guide for details.)
-         * @return TRUE if there is another element.
-         * @draft ICU 4.8
+         * @return true if there is another element.
+         * @stable ICU 4.8
          */
         UBool next(UErrorCode &errorCode);
 
         /**
          * @return The NUL-terminated byte sequence for the last successful next().
-         * @draft ICU 4.8
+         * @stable ICU 4.8
          */
-        const StringPiece &getString() const { return sp_; }
+        StringPiece getString() const;
         /**
          * @return The value for the last successful next().
-         * @draft ICU 4.8
+         * @stable ICU 4.8
          */
         int32_t getValue() const { return value_; }
 
@@ -325,7 +365,6 @@ public:
         int32_t initialRemainingMatchLength_;
 
         CharString *str_;
-        StringPiece sp_;
         int32_t maxLength_;
         int32_t value_;
 
@@ -341,6 +380,7 @@ public:
 
 private:
     friend class BytesTrieBuilder;
+    friend class ::BytesTrieTest;
 
     /**
      * Constructs a BytesTrie reader instance.
@@ -349,15 +389,15 @@ private:
      * This constructor is only called by the builder.
      */
     BytesTrie(void *adoptBytes, const void *trieBytes)
-            : ownedArray_(reinterpret_cast<uint8_t *>(adoptBytes)),
-              bytes_(reinterpret_cast<const uint8_t *>(trieBytes)),
+            : ownedArray_(static_cast<uint8_t *>(adoptBytes)),
+              bytes_(static_cast<const uint8_t *>(trieBytes)),
               pos_(bytes_), remainingMatchLength_(-1) {}
 
     // No assignment operator.
-    BytesTrie &operator=(const BytesTrie &other);
+    BytesTrie &operator=(const BytesTrie &other) = delete;
 
     inline void stop() {
-        pos_=NULL;
+        pos_=nullptr;
     }
 
     // Reads a compact 32-bit integer.
@@ -399,7 +439,7 @@ private:
     }
 
     static inline UStringTrieResult valueResult(int32_t node) {
-        return (UStringTrieResult)(USTRINGTRIE_INTERMEDIATE_VALUE-(node&kValueIsFinal));
+        return static_cast<UStringTrieResult>(USTRINGTRIE_INTERMEDIATE_VALUE - (node & kValueIsFinal));
     }
 
     // Handles a branch node for both next(byte) and next(string).
@@ -501,6 +541,13 @@ private:
     static const int32_t kMaxTwoByteDelta=((kMinThreeByteDeltaLead-kMinTwoByteDeltaLead)<<8)-1;  // 0x2fff
     static const int32_t kMaxThreeByteDelta=((kFourByteDeltaLead-kMinThreeByteDeltaLead)<<16)-1;  // 0xdffff
 
+    // For getState64():
+    // The remainingMatchLength_ is -1..14=(kMaxLinearMatchLength=0x10)-2
+    // so we need at least 5 bits for that.
+    // We add 2 to store it as a positive value 1..16=kMaxLinearMatchLength.
+    static constexpr int32_t kState64RemainingShift = 59;
+    static constexpr uint64_t kState64PosMask = (UINT64_C(1) << kState64RemainingShift) - 1;
+
     uint8_t *ownedArray_;
 
     // Fixed value referencing the BytesTrie bytes.
@@ -508,7 +555,7 @@ private:
 
     // Iterator variables.
 
-    // Pointer to next trie byte to read. NULL if no more matches.
+    // Pointer to next trie byte to read. nullptr if no more matches.
     const uint8_t *pos_;
     // Remaining length of a linear-match node, minus 1. Negative if not in such a node.
     int32_t remainingMatchLength_;
@@ -516,4 +563,6 @@ private:
 
 U_NAMESPACE_END
 
+#endif /* U_SHOW_CPLUSPLUS_API */
+
 #endif  // __BYTESTRIE_H__
