@@ -44,11 +44,10 @@ U_CDECL_BEGIN
 /*
 Static cache for already opened StringPrep profiles
 */
-static UHashtable *SHARED_DATA_HASHTABLE = NULL;
-static icu::UInitOnce gSharedDataInitOnce;
-
-static UMutex usprepMutex = U_MUTEX_INITIALIZER;
+static UHashtable *SHARED_DATA_HASHTABLE = nullptr;
+static icu::UInitOnce gSharedDataInitOnce {};
 
+static UMutex usprepMutex;
 /* format version of spp file */
 //static uint8_t formatVersion[4]={ 0, 0, 0, 0 };
 
@@ -92,9 +91,9 @@ isSPrepAcceptable(void * /* context */,
     ) {
         //uprv_memcpy(formatVersion, pInfo->formatVersion, 4);
         uprv_memcpy(dataVersion, pInfo->dataVersion, 4);
-        return TRUE;
+        return true;
     } else {
-        return FALSE;
+        return false;
     }
 }
 
@@ -112,7 +111,9 @@ hashEntry(const UHashTok parm) {
     UHashTok namekey, pathkey;
     namekey.pointer = b->name;
     pathkey.pointer = b->path;
-    return uhash_hashChars(namekey)+37*uhash_hashChars(pathkey);
+    uint32_t unsignedHash = static_cast<uint32_t>(uhash_hashChars(namekey)) +
+            37u * static_cast<uint32_t>(uhash_hashChars(pathkey));
+    return static_cast<int32_t>(unsignedHash);
 }
 
 /* compares two entries */
@@ -125,8 +126,7 @@ compareEntries(const UHashTok p1, const UHashTok p2) {
     name2.pointer = b2->name;
     path1.pointer = b1->path;
     path2.pointer = b2->path;
-    return ((UBool)(uhash_compareChars(name1, name2) & 
-        uhash_compareChars(path1, path2)));
+    return uhash_compareChars(name1, name2) & uhash_compareChars(path1, path2);
 }
 
 static void 
@@ -136,8 +136,8 @@ usprep_unload(UStringPrepProfile* data){
 
 static int32_t 
 usprep_internal_flushCache(UBool noRefCount){
-    UStringPrepProfile *profile = NULL;
-    UStringPrepKey  *key  = NULL;
+    UStringPrepProfile *profile = nullptr;
+    UStringPrepKey  *key  = nullptr;
     int32_t pos = UHASH_FIRST;
     int32_t deletedNum = 0;
     const UHashElement *e;
@@ -147,32 +147,32 @@ usprep_internal_flushCache(UBool noRefCount){
      * return 0
      */
     umtx_lock(&usprepMutex);
-    if (SHARED_DATA_HASHTABLE == NULL) {
+    if (SHARED_DATA_HASHTABLE == nullptr) {
         umtx_unlock(&usprepMutex);
         return 0;
     }
 
     /*creates an enumeration to iterate through every element in the table */
-    while ((e = uhash_nextElement(SHARED_DATA_HASHTABLE, &pos)) != NULL)
+    while ((e = uhash_nextElement(SHARED_DATA_HASHTABLE, &pos)) != nullptr)
     {
         profile = (UStringPrepProfile *) e->value.pointer;
         key  = (UStringPrepKey *) e->key.pointer;
 
-        if ((noRefCount== FALSE && profile->refCount == 0) || 
-             noRefCount== TRUE) {
+        if ((noRefCount== false && profile->refCount == 0) || 
+             noRefCount) {
             deletedNum++;
             uhash_removeElement(SHARED_DATA_HASHTABLE, e);
 
             /* unload the data */
             usprep_unload(profile);
 
-            if(key->name != NULL) {
+            if(key->name != nullptr) {
                 uprv_free(key->name);
-                key->name=NULL;
+                key->name=nullptr;
             }
-            if(key->path != NULL) {
+            if(key->path != nullptr) {
                 uprv_free(key->path);
-                key->path=NULL;
+                key->path=nullptr;
             }
             uprv_free(profile);
             uprv_free(key);
@@ -187,20 +187,20 @@ usprep_internal_flushCache(UBool noRefCount){
 /* Works just like ucnv_flushCache() 
 static int32_t 
 usprep_flushCache(){
-    return usprep_internal_flushCache(FALSE);
+    return usprep_internal_flushCache(false);
 }
 */
 
-static UBool U_CALLCONV usprep_cleanup(void){
-    if (SHARED_DATA_HASHTABLE != NULL) {
-        usprep_internal_flushCache(TRUE);
-        if (SHARED_DATA_HASHTABLE != NULL && uhash_count(SHARED_DATA_HASHTABLE) == 0) {
+static UBool U_CALLCONV usprep_cleanup(){
+    if (SHARED_DATA_HASHTABLE != nullptr) {
+        usprep_internal_flushCache(true);
+        if (SHARED_DATA_HASHTABLE != nullptr && uhash_count(SHARED_DATA_HASHTABLE) == 0) {
             uhash_close(SHARED_DATA_HASHTABLE);
-            SHARED_DATA_HASHTABLE = NULL;
+            SHARED_DATA_HASHTABLE = nullptr;
         }
     }
     gSharedDataInitOnce.reset();
-    return (SHARED_DATA_HASHTABLE == NULL);
+    return (SHARED_DATA_HASHTABLE == nullptr);
 }
 U_CDECL_END
 
@@ -208,9 +208,9 @@ U_CDECL_END
 /** Initializes the cache for resources */
 static void U_CALLCONV
 createCache(UErrorCode &status) {
-    SHARED_DATA_HASHTABLE = uhash_open(hashEntry, compareEntries, NULL, &status);
+    SHARED_DATA_HASHTABLE = uhash_open(hashEntry, compareEntries, nullptr, &status);
     if (U_FAILURE(status)) {
-        SHARED_DATA_HASHTABLE = NULL;
+        SHARED_DATA_HASHTABLE = nullptr;
     }
     ucln_common_registerCleanup(UCLN_COMMON_USPREP, usprep_cleanup);
 }
@@ -227,48 +227,48 @@ loadData(UStringPrepProfile* profile,
          const char* type, 
          UErrorCode* errorCode) {
     /* load Unicode SPREP data from file */    
-    UTrie _sprepTrie={ 0,0,0,0,0,0,0 };
+    UTrie _sprepTrie = {nullptr, nullptr, nullptr, 0, 0, 0, 0};
     UDataMemory *dataMemory;
-    const int32_t *p=NULL;
+    const int32_t *p=nullptr;
     const uint8_t *pb;
     UVersionInfo normUnicodeVersion;
     int32_t normUniVer, sprepUniVer, normCorrVer;
 
-    if(errorCode==NULL || U_FAILURE(*errorCode)) {
+    if(errorCode==nullptr || U_FAILURE(*errorCode)) {
         return 0;
     }
 
     /* open the data outside the mutex block */
     //TODO: change the path
-    dataMemory=udata_openChoice(path, type, name, isSPrepAcceptable, NULL, errorCode);
+    dataMemory=udata_openChoice(path, type, name, isSPrepAcceptable, nullptr, errorCode);
     if(U_FAILURE(*errorCode)) {
-        return FALSE;
+        return false;
     }
 
-    p=(const int32_t *)udata_getMemory(dataMemory);
-    pb=(const uint8_t *)(p+_SPREP_INDEX_TOP);
+    p = static_cast<const int32_t*>(udata_getMemory(dataMemory));
+    pb = reinterpret_cast<const uint8_t*>(p + _SPREP_INDEX_TOP);
     utrie_unserialize(&_sprepTrie, pb, p[_SPREP_INDEX_TRIE_SIZE], errorCode);
     _sprepTrie.getFoldingOffset=getSPrepFoldingOffset;
 
 
     if(U_FAILURE(*errorCode)) {
         udata_close(dataMemory);
-        return FALSE;
+        return false;
     }
 
     /* in the mutex block, set the data for this process */
     umtx_lock(&usprepMutex);
-    if(profile->sprepData==NULL) {
+    if(profile->sprepData==nullptr) {
         profile->sprepData=dataMemory;
-        dataMemory=NULL;
+        dataMemory=nullptr;
         uprv_memcpy(&profile->indexes, p, sizeof(profile->indexes));
         uprv_memcpy(&profile->sprepTrie, &_sprepTrie, sizeof(UTrie));
     } else {
-        p=(const int32_t *)udata_getMemory(profile->sprepData);
+        p = static_cast<const int32_t*>(udata_getMemory(profile->sprepData));
     }
     umtx_unlock(&usprepMutex);
     /* initialize some variables */
-    profile->mappingData=(uint16_t *)((uint8_t *)(p+_SPREP_INDEX_TOP)+profile->indexes[_SPREP_INDEX_TRIE_SIZE]);
+    profile->mappingData = reinterpret_cast<const uint16_t*>(reinterpret_cast<const uint8_t*>(p + _SPREP_INDEX_TOP) + profile->indexes[_SPREP_INDEX_TRIE_SIZE]);
     
     u_getUnicodeVersion(normUnicodeVersion);
     normUniVer = (normUnicodeVersion[0] << 24) + (normUnicodeVersion[1] << 16) + 
@@ -279,21 +279,21 @@ loadData(UStringPrepProfile* profile,
     
     if(U_FAILURE(*errorCode)){
         udata_close(dataMemory);
-        return FALSE;
+        return false;
     }
-    if( normUniVer < sprepUniVer && /* the Unicode version of SPREP file must be less than the Unicode Vesion of the normalization data */
-        normUniVer < normCorrVer && /* the Unicode version of the NormalizationCorrections.txt file should be less than the Unicode Vesion of the normalization data */
+    if( normUniVer < sprepUniVer && /* the Unicode version of SPREP file must be less than the Unicode Version of the normalization data */
+        normUniVer < normCorrVer && /* the Unicode version of the NormalizationCorrections.txt file should be less than the Unicode Version of the normalization data */
         ((profile->indexes[_SPREP_OPTIONS] & _SPREP_NORMALIZATION_ON) > 0) /* normalization turned on*/
       ){
         *errorCode = U_INVALID_FORMAT_ERROR;
         udata_close(dataMemory);
-        return FALSE;
+        return false;
     }
-    profile->isDataLoaded = TRUE;
+    profile->isDataLoaded = true;
 
     /* if a different thread set it first, then close the extra data */
-    if(dataMemory!=NULL) {
-        udata_close(dataMemory); /* NULL if it was set correctly */
+    if(dataMemory!=nullptr) {
+        udata_close(dataMemory); /* nullptr if it was set correctly */
     }
 
 
@@ -305,12 +305,12 @@ usprep_getProfile(const char* path,
                   const char* name,
                   UErrorCode *status){
 
-    UStringPrepProfile* profile = NULL;
+    UStringPrepProfile* profile = nullptr;
 
     initCache(status);
 
     if(U_FAILURE(*status)){
-        return NULL;
+        return nullptr;
     }
 
     UStringPrepKey stackKey;
@@ -319,55 +319,51 @@ usprep_getProfile(const char* path,
      * we use the passed in pointers for fetching the data from the 
      * hash table which is safe
      */
-    stackKey.name = (char*) name;
-    stackKey.path = (char*) path;
+    stackKey.name = const_cast<char*>(name);
+    stackKey.path = const_cast<char*>(path);
 
     /* fetch the data from the cache */
     umtx_lock(&usprepMutex);
-    profile = (UStringPrepProfile*) (uhash_get(SHARED_DATA_HASHTABLE,&stackKey));
-    if(profile != NULL) {
+    profile = static_cast<UStringPrepProfile*>(uhash_get(SHARED_DATA_HASHTABLE, &stackKey));
+    if(profile != nullptr) {
         profile->refCount++;
     }
     umtx_unlock(&usprepMutex);
     
-    if(profile == NULL) {
+    if(profile == nullptr) {
         /* else load the data and put the data in the cache */
         LocalMemory<UStringPrepProfile> newProfile;
-        if(newProfile.allocateInsteadAndReset() == NULL) {
+        if(newProfile.allocateInsteadAndReset() == nullptr) {
             *status = U_MEMORY_ALLOCATION_ERROR;
-            return NULL;
+            return nullptr;
         }
 
         /* load the data */
         if(!loadData(newProfile.getAlias(), path, name, _SPREP_DATA_TYPE, status) || U_FAILURE(*status) ){
-            return NULL;
+            return nullptr;
         }
 
         /* get the options */
-        newProfile->doNFKC = (UBool)((newProfile->indexes[_SPREP_OPTIONS] & _SPREP_NORMALIZATION_ON) > 0);
-        newProfile->checkBiDi = (UBool)((newProfile->indexes[_SPREP_OPTIONS] & _SPREP_CHECK_BIDI_ON) > 0);
-
-        if(newProfile->checkBiDi) {
-            newProfile->bdp = ubidi_getSingleton();
-        }
+        newProfile->doNFKC = static_cast<UBool>((newProfile->indexes[_SPREP_OPTIONS] & _SPREP_NORMALIZATION_ON) > 0);
+        newProfile->checkBiDi = static_cast<UBool>((newProfile->indexes[_SPREP_OPTIONS] & _SPREP_CHECK_BIDI_ON) > 0);
 
         LocalMemory<UStringPrepKey> key;
         LocalMemory<char> keyName;
         LocalMemory<char> keyPath;
-        if( key.allocateInsteadAndReset() == NULL ||
-            keyName.allocateInsteadAndCopy(uprv_strlen(name)+1) == NULL ||
-            (path != NULL &&
-             keyPath.allocateInsteadAndCopy(uprv_strlen(path)+1) == NULL)
+        if( key.allocateInsteadAndReset() == nullptr ||
+            keyName.allocateInsteadAndCopy(static_cast<int32_t>(uprv_strlen(name)+1)) == nullptr ||
+            (path != nullptr &&
+             keyPath.allocateInsteadAndCopy(static_cast<int32_t>(uprv_strlen(path)+1)) == nullptr)
          ) {
             *status = U_MEMORY_ALLOCATION_ERROR;
             usprep_unload(newProfile.getAlias());
-            return NULL;
+            return nullptr;
         }
 
         umtx_lock(&usprepMutex);
         // If another thread already inserted the same key/value, refcount and cleanup our thread data
-        profile = (UStringPrepProfile*) (uhash_get(SHARED_DATA_HASHTABLE,&stackKey));
-        if(profile != NULL) {
+        profile = static_cast<UStringPrepProfile*>(uhash_get(SHARED_DATA_HASHTABLE, &stackKey));
+        if(profile != nullptr) {
             profile->refCount++;
             usprep_unload(newProfile.getAlias());
         }
@@ -375,7 +371,7 @@ usprep_getProfile(const char* path,
             /* initialize the key members */
             key->name = keyName.orphan();
             uprv_strcpy(key->name, name);
-            if(path != NULL){
+            if(path != nullptr){
                 key->path = keyPath.orphan();
                 uprv_strcpy(key->path, path);
             }        
@@ -396,8 +392,8 @@ usprep_open(const char* path,
             const char* name,
             UErrorCode* status){
 
-    if(status == NULL || U_FAILURE(*status)){
-        return NULL;
+    if(status == nullptr || U_FAILURE(*status)){
+        return nullptr;
     }
        
     /* initialize the profile struct members */
@@ -407,20 +403,20 @@ usprep_open(const char* path,
 U_CAPI UStringPrepProfile* U_EXPORT2
 usprep_openByType(UStringPrepProfileType type,
 				  UErrorCode* status) {
-    if(status == NULL || U_FAILURE(*status)){
-        return NULL;
+    if(status == nullptr || U_FAILURE(*status)){
+        return nullptr;
     }
     int32_t index = (int32_t)type;
     if (index < 0 || index >= UPRV_LENGTHOF(PROFILE_NAMES)) {
         *status = U_ILLEGAL_ARGUMENT_ERROR;
-        return NULL;
+        return nullptr;
     }
-    return usprep_open(NULL, PROFILE_NAMES[index], status);
+    return usprep_open(nullptr, PROFILE_NAMES[index], status);
 }
 
 U_CAPI void U_EXPORT2
 usprep_close(UStringPrepProfile* profile){
-    if(profile==NULL){
+    if(profile==nullptr){
         return;
     }
 
@@ -434,11 +430,11 @@ usprep_close(UStringPrepProfile* profile){
 }
 
 U_CFUNC void 
-uprv_syntaxError(const UChar* rules, 
+uprv_syntaxError(const char16_t* rules,
                  int32_t pos,
                  int32_t rulesLen,
                  UParseError* parseError){
-    if(parseError == NULL){
+    if(parseError == nullptr){
         return;
     }
     parseError->offset = pos;
@@ -477,39 +473,39 @@ getValues(uint16_t trieWord, int16_t& value, UBool& isIndex){
          * the source codepoint is copied to the destination
          */
         type = USPREP_TYPE_LIMIT;
-        isIndex =FALSE;
+        isIndex =false;
         value = 0;
     }else if(trieWord >= _SPREP_TYPE_THRESHOLD){
-        type = (UStringPrepType) (trieWord - _SPREP_TYPE_THRESHOLD);
-        isIndex =FALSE;
+        type = static_cast<UStringPrepType>(trieWord - _SPREP_TYPE_THRESHOLD);
+        isIndex =false;
         value = 0;
     }else{
         /* get the type */
         type = USPREP_MAP;
         /* ascertain if the value is index or delta */
         if(trieWord & 0x02){
-            isIndex = TRUE;
+            isIndex = true;
             value = trieWord  >> 2; //mask off the lower 2 bits and shift
         }else{
-            isIndex = FALSE;
-            value = (int16_t)trieWord;
+            isIndex = false;
+            value = static_cast<int16_t>(trieWord);
             value =  (value >> 2);
         }
  
         if((trieWord>>2) == _SPREP_MAX_INDEX_VALUE){
             type = USPREP_DELETE;
-            isIndex =FALSE;
+            isIndex =false;
             value = 0;
         }
     }
     return type;
 }
 
-// TODO: change to writing to UnicodeString not UChar *
+// TODO: change to writing to UnicodeString not char16_t *
 static int32_t 
 usprep_map(  const UStringPrepProfile* profile, 
-             const UChar* src, int32_t srcLength, 
-             UChar* dest, int32_t destCapacity,
+             const char16_t* src, int32_t srcLength,
+             char16_t* dest, int32_t destCapacity,
              int32_t options,
              UParseError* parseError,
              UErrorCode* status ){
@@ -517,7 +513,7 @@ usprep_map(  const UStringPrepProfile* profile,
     uint16_t result;
     int32_t destIndex=0;
     int32_t srcIndex;
-    UBool allowUnassigned = (UBool) ((options & USPREP_ALLOW_UNASSIGNED)>0);
+    UBool allowUnassigned = static_cast<UBool>((options & USPREP_ALLOW_UNASSIGNED) > 0);
     UStringPrepType type;
     int16_t value;
     UBool isIndex;
@@ -538,7 +534,7 @@ usprep_map(  const UStringPrepProfile* profile,
         type = getValues(result, value, isIndex);
 
         // check if the source codepoint is unassigned
-        if(type == USPREP_UNASSIGNED && allowUnassigned == FALSE){
+        if(type == USPREP_UNASSIGNED && allowUnassigned == false){
 
             uprv_syntaxError(src,srcIndex-U16_LENGTH(ch), srcLength,parseError);
             *status = U_STRINGPREP_UNASSIGNED_ERROR;
@@ -578,13 +574,13 @@ usprep_map(  const UStringPrepProfile* profile,
             }
 
         }else if(type==USPREP_DELETE){
-             // just consume the codepoint and contine
+             // just consume the codepoint and continue
             continue;
         }
         //copy the code point into destination
         if(ch <= 0xFFFF){
             if(destIndex < destCapacity ){
-                dest[destIndex] = (UChar)ch;
+                dest[destIndex] = static_cast<char16_t>(ch);
             }
             destIndex++;
         }else{
@@ -640,8 +636,8 @@ usprep_map(  const UStringPrepProfile* profile,
 */
 U_CAPI int32_t U_EXPORT2
 usprep_prepare(   const UStringPrepProfile* profile,
-                  const UChar* src, int32_t srcLength, 
-                  UChar* dest, int32_t destCapacity,
+                  const char16_t* src, int32_t srcLength,
+                  char16_t* dest, int32_t destCapacity,
                   int32_t options,
                   UParseError* parseError,
                   UErrorCode* status ){
@@ -652,9 +648,9 @@ usprep_prepare(   const UStringPrepProfile* profile,
     }
 
     //check arguments
-    if(profile==NULL ||
-            (src==NULL ? srcLength!=0 : srcLength<-1) ||
-            (dest==NULL ? destCapacity!=0 : destCapacity<0)) {
+    if(profile==nullptr ||
+            (src==nullptr ? srcLength!=0 : srcLength<-1) ||
+            (dest==nullptr ? destCapacity!=0 : destCapacity<0)) {
         *status=U_ILLEGAL_ARGUMENT_ERROR;
         return 0;
     }
@@ -665,8 +661,8 @@ usprep_prepare(   const UStringPrepProfile* profile,
     }
     // map
     UnicodeString s1;
-    UChar *b1 = s1.getBuffer(srcLength);
-    if(b1==NULL){
+    char16_t *b1 = s1.getBuffer(srcLength);
+    if(b1==nullptr){
         *status = U_MEMORY_ALLOCATION_ERROR;
         return 0;
     }
@@ -678,7 +674,7 @@ usprep_prepare(   const UStringPrepProfile* profile,
         // redo processing of string
         /* we do not have enough room so grow the buffer*/
         b1 = s1.getBuffer(b1Len);
-        if(b1==NULL){
+        if(b1==nullptr){
             *status = U_MEMORY_ALLOCATION_ERROR;
             return 0;
         }
@@ -709,10 +705,10 @@ usprep_prepare(   const UStringPrepProfile* profile,
     }
 
     // Prohibit and checkBiDi in one pass
-    const UChar *b2 = s2.getBuffer();
+    const char16_t *b2 = s2.getBuffer();
     int32_t b2Len = s2.length();
     UCharDirection direction=U_CHAR_DIRECTION_COUNT, firstCharDir=U_CHAR_DIRECTION_COUNT;
-    UBool leftToRight=FALSE, rightToLeft=FALSE;
+    UBool leftToRight=false, rightToLeft=false;
     int32_t rtlPos =-1, ltrPos =-1;
 
     for(int32_t b2Index=0; b2Index<b2Len;){
@@ -730,41 +726,41 @@ usprep_prepare(   const UStringPrepProfile* profile,
             ((result < _SPREP_TYPE_THRESHOLD) && (result & 0x01) /* first bit says it the code point is prohibited*/)
            ){
             *status = U_STRINGPREP_PROHIBITED_ERROR;
-            uprv_syntaxError(b1, b2Index-U16_LENGTH(ch), b2Len, parseError);
+            uprv_syntaxError(b2, b2Index-U16_LENGTH(ch), b2Len, parseError);
             return 0;
         }
 
         if(profile->checkBiDi) {
-            direction = ubidi_getClass(profile->bdp, ch);
+            direction = ubidi_getClass(ch);
             if(firstCharDir == U_CHAR_DIRECTION_COUNT){
                 firstCharDir = direction;
             }
             if(direction == U_LEFT_TO_RIGHT){
-                leftToRight = TRUE;
+                leftToRight = true;
                 ltrPos = b2Index-1;
             }
             if(direction == U_RIGHT_TO_LEFT || direction == U_RIGHT_TO_LEFT_ARABIC){
-                rightToLeft = TRUE;
+                rightToLeft = true;
                 rtlPos = b2Index-1;
             }
         }
     }
-    if(profile->checkBiDi == TRUE){
+    if(profile->checkBiDi){
         // satisfy 2
-        if( leftToRight == TRUE && rightToLeft == TRUE){
+        if( leftToRight && rightToLeft){
             *status = U_STRINGPREP_CHECK_BIDI_ERROR;
             uprv_syntaxError(b2,(rtlPos>ltrPos) ? rtlPos : ltrPos, b2Len, parseError);
             return 0;
         }
 
         //satisfy 3
-        if( rightToLeft == TRUE && 
+        if( rightToLeft && 
             !((firstCharDir == U_RIGHT_TO_LEFT || firstCharDir == U_RIGHT_TO_LEFT_ARABIC) &&
               (direction == U_RIGHT_TO_LEFT || direction == U_RIGHT_TO_LEFT_ARABIC))
            ){
             *status = U_STRINGPREP_CHECK_BIDI_ERROR;
             uprv_syntaxError(b2, rtlPos, b2Len, parseError);
-            return FALSE;
+            return false;
         }
     }
     return s2.extract(dest, destCapacity, *status);
@@ -790,7 +786,7 @@ usprep_swap(const UDataSwapper *ds,
 
     /* udata_swapDataHeader checks the arguments */
     headerSize=udata_swapDataHeader(ds, inData, length, outData, pErrorCode);
-    if(pErrorCode==NULL || U_FAILURE(*pErrorCode)) {
+    if(pErrorCode==nullptr || U_FAILURE(*pErrorCode)) {
         return 0;
     }
 
@@ -812,7 +808,7 @@ usprep_swap(const UDataSwapper *ds,
     }
 
     inBytes=(const uint8_t *)inData+headerSize;
-    outBytes=(uint8_t *)outData+headerSize;
+    outBytes= (outData == nullptr ) ? nullptr : (uint8_t *)outData+headerSize;
 
     inIndexes=(const int32_t *)inBytes;
 
