@@ -20,10 +20,11 @@
 #if !UCONFIG_NO_FORMATTING
 
 #include "cmemory.h"
+#include "cstring.h"
 #include "umutex.h"
 #include <float.h>
-#include "gregoimp.h" // Math
-#include "astro.h" // CalendarAstronomer
+#include "gregoimp.h" // ClockMath
+#include "astro.h" // CalendarCache
 #include "uhash.h"
 #include "ucln_in.h"
 
@@ -63,6 +64,7 @@ static const int32_t LIMITS[UCAL_FIELD_COUNT][4] = {
     {/*N/A*/-1,/*N/A*/-1,/*N/A*/-1,/*N/A*/-1}, // JULIAN_DAY
     {/*N/A*/-1,/*N/A*/-1,/*N/A*/-1,/*N/A*/-1}, // MILLISECONDS_IN_DAY
     {/*N/A*/-1,/*N/A*/-1,/*N/A*/-1,/*N/A*/-1}, // IS_LEAP_MONTH
+    {        0,        0,       11,       12}, // ORDINAL_MONTH
 };
 
 /**
@@ -134,13 +136,17 @@ static const int16_t  LEAP_MONTH_START[][3] = {
     {  383,        384,        385  },          // Elul
 };
 
-static icu::CalendarCache *gCache =  NULL;
+// There are 235 months in 19 years cycle.
+static const int32_t MONTHS_IN_CYCLE = 235;
+static const int32_t YEARS_IN_CYCLE = 19;
+
+static icu::CalendarCache *gCache =  nullptr;
 
 U_CDECL_BEGIN
-static UBool calendar_hebrew_cleanup(void) {
+static UBool calendar_hebrew_cleanup() {
     delete gCache;
-    gCache = NULL;
-    return TRUE;
+    gCache = nullptr;
+    return true;
 }
 U_CDECL_END
 
@@ -155,7 +161,7 @@ U_NAMESPACE_BEGIN
 * @internal
 */
 HebrewCalendar::HebrewCalendar(const Locale& aLocale, UErrorCode& success)
-:   Calendar(TimeZone::createDefault(), aLocale, success)
+:   Calendar(TimeZone::forLocaleOrDefault(aLocale), aLocale, success)
 
 {
     setTimeInMillis(getNow(), success); // Call this again now that the vtable is set up properly.
@@ -169,7 +175,7 @@ const char *HebrewCalendar::getType() const {
     return "hebrew";
 }
 
-Calendar* HebrewCalendar::clone() const {
+HebrewCalendar* HebrewCalendar::clone() const {
     return new HebrewCalendar(*this);
 }
 
@@ -217,19 +223,31 @@ void HebrewCalendar::add(UCalendarDateFields field, int32_t amount, UErrorCode&
         return;
     }
     switch (field) {
-  case UCAL_MONTH: 
+  case UCAL_MONTH:
+  case UCAL_ORDINAL_MONTH:
       {
           // We can't just do a set(MONTH, get(MONTH) + amount).  The
           // reason is ADAR_1.  Suppose amount is +2 and we land in
           // ADAR_1 -- then we have to bump to ADAR_2 aka ADAR.  But
           // if amount is -2 and we land in ADAR_1, then we have to
           // bump the other way -- down to SHEVAT.  - Alan 11/00
-          int32_t month = get(UCAL_MONTH, status);
+          int64_t month = get(UCAL_MONTH, status);
           int32_t year = get(UCAL_YEAR, status);
           UBool acrossAdar1;
           if (amount > 0) {
               acrossAdar1 = (month < ADAR_1); // started before ADAR_1?
               month += amount;
+              // We know there are total 235 months in every 19 years. To speed
+              // up the iteration, we first fast forward in the multiple of 235
+              // months for 19 years before the iteration which check the leap year.
+              if (month >= MONTHS_IN_CYCLE) {
+                  if (uprv_add32_overflow(year, (month / MONTHS_IN_CYCLE) * YEARS_IN_CYCLE, &year)) {
+                      status = U_ILLEGAL_ARGUMENT_ERROR;
+                      return;
+                  }
+                  month %= MONTHS_IN_CYCLE;
+              }
+
               for (;;) {
                   if (acrossAdar1 && month>=ADAR_1 && !isLeapYear(year)) {
                       ++month;
@@ -239,11 +257,21 @@ void HebrewCalendar::add(UCalendarDateFields field, int32_t amount, UErrorCode&
                   }
                   month -= ELUL+1;
                   ++year;
-                  acrossAdar1 = TRUE;
+                  acrossAdar1 = true;
               }
           } else {
               acrossAdar1 = (month > ADAR_1); // started after ADAR_1?
               month += amount;
+              // We know there are total 235 months in every 19 years. To speed
+              // up the iteration, we first fast forward in the multiple of 235
+              // months for 19 years before the iteration which check the leap year.
+              if (month <= -MONTHS_IN_CYCLE) {
+                  if (uprv_add32_overflow(year, (month / MONTHS_IN_CYCLE) * YEARS_IN_CYCLE, &year)) {
+                      status = U_ILLEGAL_ARGUMENT_ERROR;
+                      return;
+                  }
+                  month %= MONTHS_IN_CYCLE;
+              }
               for (;;) {
                   if (acrossAdar1 && month<=ADAR_1 && !isLeapYear(year)) {
                       --month;
@@ -253,7 +281,7 @@ void HebrewCalendar::add(UCalendarDateFields field, int32_t amount, UErrorCode&
                   }
                   month += ELUL+1;
                   --year;
-                  acrossAdar1 = TRUE;
+                  acrossAdar1 = true;
               }
           }
           set(UCAL_MONTH, month);
@@ -273,9 +301,15 @@ void HebrewCalendar::add(UCalendarDateFields field, int32_t amount, UErrorCode&
 */
 void HebrewCalendar::add(EDateFields field, int32_t amount, UErrorCode& status)
 {
-    add((UCalendarDateFields)field, amount, status);
+    add(static_cast<UCalendarDateFields>(field), amount, status);
 }
 
+namespace {
+
+int32_t monthsInYear(int32_t year);
+
+}  // namespace
+
 /**
 * Rolls (up/down) a specified amount time on the given field.  For
 * example, to roll the current date up by three days, you can call
@@ -315,6 +349,7 @@ void HebrewCalendar::roll(UCalendarDateFields field, int32_t amount, UErrorCode&
     }
     switch (field) {
   case UCAL_MONTH:
+  case UCAL_ORDINAL_MONTH:
       {
           int32_t month = get(UCAL_MONTH, status);
           int32_t year = get(UCAL_YEAR, status);
@@ -343,7 +378,7 @@ void HebrewCalendar::roll(UCalendarDateFields field, int32_t amount, UErrorCode&
 }
 
 void HebrewCalendar::roll(EDateFields field, int32_t amount, UErrorCode& status) {
-    roll((UCalendarDateFields)field, amount, status);
+    roll(static_cast<UCalendarDateFields>(field), amount, status);
 }
 
 //-------------------------------------------------------------------------
@@ -367,6 +402,8 @@ static const int32_t MONTH_PARTS = MONTH_DAYS*DAY_PARTS + MONTH_FRACT;
 // Bet (Monday), Hey (5 hours from sunset), Resh-Daled (204).
 static const int32_t BAHARAD = 11*HOUR_PARTS + 204;
 
+namespace {
+
 /**
 * Finds the day # of the first day in the given Hebrew year.
 * To do this, we want to calculate the time of the Tishri 1 new moon
@@ -387,16 +424,21 @@ static const int32_t BAHARAD = 11*HOUR_PARTS + 204;
 *      http://www.faqs.org/faqs/calendars/faq/</a>
 * </ul>
 */
-int32_t HebrewCalendar::startOfYear(int32_t year, UErrorCode &status)
+int32_t startOfYear(int32_t year, UErrorCode &status)
 {
     ucln_i18n_registerCleanup(UCLN_I18N_HEBREW_CALENDAR, calendar_hebrew_cleanup);
-    int32_t day = CalendarCache::get(&gCache, year, status);
+    int64_t day = CalendarCache::get(&gCache, year, status);
+    if(U_FAILURE(status)) {
+        return 0;
+    }
 
     if (day == 0) {
-        int32_t months = (235 * year - 234) / 19;           // # of months before year
+        // # of months before year
+        int64_t months = ClockMath::floorDivideInt64(
+            (235LL * static_cast<int64_t>(year) - 234LL), 19LL);
 
-        int64_t frac = (int64_t)months * MONTH_FRACT + BAHARAD;  // Fractional part of day #
-        day  = months * 29 + (int32_t)(frac / DAY_PARTS);        // Whole # part of calculation
+        int64_t frac = months * MONTH_FRACT + BAHARAD;  // Fractional part of day #
+        day  = months * 29LL + frac / DAY_PARTS;        // Whole # part of calculation
         frac = frac % DAY_PARTS;                        // Time of day
 
         int32_t wd = (day % 7);                        // Day of week (0 == Monday)
@@ -406,33 +448,34 @@ int32_t HebrewCalendar::startOfYear(int32_t year, UErrorCode &status)
             day += 1;
             wd = (day % 7);
         }
-        if (wd == 1 && frac > 15*HOUR_PARTS+204 && !isLeapYear(year) ) {
+        if (wd == 1 && frac > 15*HOUR_PARTS+204 && !HebrewCalendar::isLeapYear(year) ) {
             // If the new moon falls after 3:11:20am (15h204p from the previous noon)
             // on a Tuesday and it is not a leap year, postpone by 2 days.
             // This prevents 356-day years.
             day += 2;
         }
-        else if (wd == 0 && frac > 21*HOUR_PARTS+589 && isLeapYear(year-1) ) {
+        else if (wd == 0 && frac > 21*HOUR_PARTS+589 && HebrewCalendar::isLeapYear(year-1) ) {
             // If the new moon falls after 9:32:43 1/3am (21h589p from yesterday noon)
             // on a Monday and *last* year was a leap year, postpone by 1 day.
             // Prevents 382-day years.
             day += 1;
         }
-        CalendarCache::put(&gCache, year, day, status);
+        if (day > INT32_MAX || day < INT32_MIN) {
+            status = U_ILLEGAL_ARGUMENT_ERROR;
+            return 0;
+        }
+        CalendarCache::put(&gCache, year, static_cast<int32_t>(day), status);
     }
+    // Out of range value is alread rejected before putting into cache.
+    U_ASSERT(INT32_MIN <= day  &&  day <= INT32_MAX);
     return day;
 }
 
-/**
-* Find the day of the week for a given day
-*
-* @param day   The # of days since the start of the Hebrew calendar,
-*              1-based (i.e. 1/1/1 AM is day 1).
-*/
-int32_t HebrewCalendar::absoluteDayToDayOfWeek(int32_t day)
-{
-    // We know that 1/1/1 AM is a Monday, which makes the math easy...
-    return (day % 7) + 1;
+int32_t daysInYear(int32_t eyear, UErrorCode& status) {
+    if (U_FAILURE(status)) {
+       return 0;
+    }
+    return startOfYear(eyear+1, status) - startOfYear(eyear, status);
 }
 
 /**
@@ -441,9 +484,15 @@ int32_t HebrewCalendar::absoluteDayToDayOfWeek(int32_t day)
 *  1   "Normal"    year with 354 or 384 days
 *  2   "Complete"  year with 355 or 385 days
 */
-int32_t HebrewCalendar::yearType(int32_t year) const
+int32_t yearType(int32_t year, UErrorCode& status)
 {
-    int32_t yearLength = handleGetYearLength(year);
+    if (U_FAILURE(status)) {
+        return 0;
+    }
+    int32_t yearLength = daysInYear(year, status);
+    if (U_FAILURE(status)) {
+        return 0;
+    }
 
     if (yearLength > 380) {
         yearLength -= 30;        // Subtract length of leap month.
@@ -465,6 +514,8 @@ int32_t HebrewCalendar::yearType(int32_t year) const
     return type;
 }
 
+}  // namespace
+   //
 /**
 * Determine whether a given Hebrew year is a leap year
 *
@@ -473,14 +524,18 @@ int32_t HebrewCalendar::yearType(int32_t year) const
 */
 UBool HebrewCalendar::isLeapYear(int32_t year) {
     //return (year * 12 + 17) % 19 >= 12;
-    int32_t x = (year*12 + 17) % 19;
+    int64_t x = (year*12LL + 17) % YEARS_IN_CYCLE;
     return x >= ((x < 0) ? -7 : 12);
 }
 
-int32_t HebrewCalendar::monthsInYear(int32_t year) {
-    return isLeapYear(year) ? 13 : 12;
+namespace{
+
+int32_t monthsInYear(int32_t year) {
+    return HebrewCalendar::isLeapYear(year) ? 13 : 12;
 }
 
+}  // namespace
+
 //-------------------------------------------------------------------------
 // Calendar framework
 //-------------------------------------------------------------------------
@@ -496,7 +551,10 @@ int32_t HebrewCalendar::handleGetLimit(UCalendarDateFields field, ELimitType lim
 * Returns the length of the given month in the given year
 * @internal
 */
-int32_t HebrewCalendar::handleGetMonthLength(int32_t extendedYear, int32_t month) const {
+int32_t HebrewCalendar::handleGetMonthLength(int32_t extendedYear, int32_t month, UErrorCode& status) const {
+    if(U_FAILURE(status)) {
+        return 0;
+    }
     // Resolve out-of-range months.  This is necessary in order to
     // obtain the correct year.  We correct to
     // a 12- or 13-month year (add/subtract 12 or 13, depending
@@ -514,8 +572,14 @@ int32_t HebrewCalendar::handleGetMonthLength(int32_t extendedYear, int32_t month
     switch (month) {
     case HESHVAN:
     case KISLEV:
-      // These two month lengths can vary
-      return MONTH_LENGTH[month][yearType(extendedYear)];
+      {
+          // These two month lengths can vary
+          int32_t type = yearType(extendedYear, status);
+          if(U_FAILURE(status)) {
+              return 0;
+          }
+          return MONTH_LENGTH[month][type];
+      }
 
     default:
       // The rest are a fixed length
@@ -529,11 +593,19 @@ int32_t HebrewCalendar::handleGetMonthLength(int32_t extendedYear, int32_t month
 */
 int32_t HebrewCalendar::handleGetYearLength(int32_t eyear) const {
     UErrorCode status = U_ZERO_ERROR;
-    return startOfYear(eyear+1, status) - startOfYear(eyear, status);
+    int32_t len = daysInYear(eyear, status);
+    if (U_FAILURE(status)) {
+        return 12;
+    }
+    return len;
 }
 
 void HebrewCalendar::validateField(UCalendarDateFields field, UErrorCode &status) {
-    if (field == UCAL_MONTH && !isLeapYear(handleGetExtendedYear()) && internalGet(UCAL_MONTH) == ADAR_1) {
+    if ((field == UCAL_MONTH || field == UCAL_ORDINAL_MONTH)
+        && !isLeapYear(handleGetExtendedYear(status)) && internalGetMonth(status) == ADAR_1) {
+        if (U_FAILURE(status)) {
+            return;
+        }
         status = U_ILLEGAL_ARGUMENT_ERROR;
         return;
     }
@@ -565,26 +637,39 @@ void HebrewCalendar::validateField(UCalendarDateFields field, UErrorCode &status
 * @internal
 */
 void HebrewCalendar::handleComputeFields(int32_t julianDay, UErrorCode &status) {
+    if (U_FAILURE(status)) {
+        return;
+    }
     int32_t d = julianDay - 347997;
-    double m = ((d * (double)DAY_PARTS)/ (double) MONTH_PARTS);         // Months (approx)
-    int32_t year = (int32_t)( ((19. * m + 234.) / 235.) + 1.);     // Years (approx)
+    double m = ClockMath::floorDivide((d * static_cast<double>(DAY_PARTS)), static_cast<double>(MONTH_PARTS)); // Months (approx)
+    int32_t year = static_cast<int32_t>(ClockMath::floorDivide((19. * m + 234.), 235.) + 1.); // Years (approx)
     int32_t ys  = startOfYear(year, status);                   // 1st day of year
+    if (U_FAILURE(status)) {
+        return;
+    }
     int32_t dayOfYear = (d - ys);
 
     // Because of the postponement rules, it's possible to guess wrong.  Fix it.
     while (dayOfYear < 1) {
         year--;
         ys  = startOfYear(year, status);
+        if (U_FAILURE(status)) {
+            return;
+        }
         dayOfYear = (d - ys);
     }
 
     // Now figure out which month we're in, and the date within that month
-    int32_t type = yearType(year);
+    int32_t type = yearType(year, status);
+    if (U_FAILURE(status)) {
+        return;
+    }
     UBool isLeap = isLeapYear(year);
 
     int32_t month = 0;
     int32_t momax = UPRV_LENGTHOF(MONTH_START);
-    while (month < momax && dayOfYear > (  isLeap ? LEAP_MONTH_START[month][type] : MONTH_START[month][type] ) ) {
+    while (month < momax &&
+           dayOfYear > (  isLeap ? LEAP_MONTH_START[month][type] : MONTH_START[month][type] ) ) {
         month++;
     }
     if (month >= momax || month<=0) {
@@ -604,11 +689,33 @@ void HebrewCalendar::handleComputeFields(int32_t julianDay, UErrorCode &status)
     int dayOfMonth = dayOfYear - (isLeap ? LEAP_MONTH_START[month][type] : MONTH_START[month][type]);
 
     internalSet(UCAL_ERA, 0);
+    // Check out of bound year
+    int32_t min_year = handleGetLimit(UCAL_EXTENDED_YEAR, UCAL_LIMIT_MINIMUM);
+    if (year < min_year) {
+        if (!isLenient()) {
+            status = U_ILLEGAL_ARGUMENT_ERROR;
+            return;
+        }
+        year = min_year;
+    }
+    int32_t max_year = handleGetLimit(UCAL_EXTENDED_YEAR, UCAL_LIMIT_MAXIMUM);
+    if (max_year < year) {
+        if (!isLenient()) {
+            status = U_ILLEGAL_ARGUMENT_ERROR;
+            return;
+        }
+        year = max_year;
+    }
     internalSet(UCAL_YEAR, year);
     internalSet(UCAL_EXTENDED_YEAR, year);
+    int32_t ordinal_month = month;
+    if (!isLeap && ordinal_month > ADAR_1) {
+        ordinal_month--;
+    }
+    internalSet(UCAL_ORDINAL_MONTH, ordinal_month);
     internalSet(UCAL_MONTH, month);
     internalSet(UCAL_DAY_OF_MONTH, dayOfMonth);
-    internalSet(UCAL_DAY_OF_YEAR, dayOfYear);       
+    internalSet(UCAL_DAY_OF_YEAR, dayOfYear);
 }
 
 //-------------------------------------------------------------------------
@@ -618,111 +725,159 @@ void HebrewCalendar::handleComputeFields(int32_t julianDay, UErrorCode &status)
 /**
 * @internal
 */
-int32_t HebrewCalendar::handleGetExtendedYear() {
-    int32_t year;
-    if (newerField(UCAL_EXTENDED_YEAR, UCAL_YEAR) == UCAL_EXTENDED_YEAR) {
-        year = internalGet(UCAL_EXTENDED_YEAR, 1); // Default to year 1
-    } else {
-        year = internalGet(UCAL_YEAR, 1); // Default to year 1
+int32_t HebrewCalendar::handleGetExtendedYear(UErrorCode& status ) {
+    if (U_FAILURE(status)) {
+        return 0;
     }
-    return year;
+    if (newerField(UCAL_EXTENDED_YEAR, UCAL_YEAR) == UCAL_EXTENDED_YEAR) {
+        return internalGet(UCAL_EXTENDED_YEAR, 1); // Default to year 1
+    }
+    return internalGet(UCAL_YEAR, 1); // Default to year 1
 }
 
 /**
 * Return JD of start of given month/year.
 * @internal
 */
-int32_t HebrewCalendar::handleComputeMonthStart(int32_t eyear, int32_t month, UBool /*useMonth*/) const {
-    UErrorCode status = U_ZERO_ERROR;
+int64_t HebrewCalendar::handleComputeMonthStart(
+    int32_t eyear, int32_t month, UBool /*useMonth*/, UErrorCode& status) const {
+    if (U_FAILURE(status)) {
+        return 0;
+    }
     // Resolve out-of-range months.  This is necessary in order to
     // obtain the correct year.  We correct to
     // a 12- or 13-month year (add/subtract 12 or 13, depending
     // on the year) but since we _always_ number from 0..12, and
     // the leap year determines whether or not month 5 (Adar 1)
     // is present, we allow 0..12 in any given year.
+
+    // The month could be in large value, we first roll 235 months to 19 years
+    // before the while loop.
+    if (month <= -MONTHS_IN_CYCLE || month >= MONTHS_IN_CYCLE) {
+        if (uprv_add32_overflow(eyear, (month / MONTHS_IN_CYCLE) * YEARS_IN_CYCLE, &eyear)) {
+            status = U_ILLEGAL_ARGUMENT_ERROR;
+            return 0;
+        }
+        month %= MONTHS_IN_CYCLE;
+    }
     while (month < 0) {
-        month += monthsInYear(--eyear);
+        if (uprv_add32_overflow(eyear, -1, &eyear) ||
+            uprv_add32_overflow(month, monthsInYear(eyear), &month)) {
+            status = U_ILLEGAL_ARGUMENT_ERROR;
+            return 0;
+        }
     }
     // Careful: allow 0..12 in all years
     while (month > 12) {
-        month -= monthsInYear(eyear++);
+        if (uprv_add32_overflow(month, -monthsInYear(eyear), &month) ||
+            uprv_add32_overflow(eyear, 1, &eyear)) {
+            status = U_ILLEGAL_ARGUMENT_ERROR;
+            return 0;
+        }
     }
 
-    int32_t day = startOfYear(eyear, status);
+    int64_t day = startOfYear(eyear, status);
 
     if(U_FAILURE(status)) {
         return 0;
     }
 
     if (month != 0) {
+        int32_t type = yearType(eyear, status);
+        if (U_FAILURE(status)) {
+            return 0;
+        }
         if (isLeapYear(eyear)) {
-            day += LEAP_MONTH_START[month][yearType(eyear)];
+            day += LEAP_MONTH_START[month][type];
         } else {
-            day += MONTH_START[month][yearType(eyear)];
+            day += MONTH_START[month][type];
         }
     }
 
-    return (int) (day + 347997);
+    return day + 347997LL;
 }
 
-UBool
-HebrewCalendar::inDaylightTime(UErrorCode& status) const
+constexpr uint32_t kHebrewRelatedYearDiff = -3760;
+
+int32_t HebrewCalendar::getRelatedYear(UErrorCode &status) const
 {
-    // copied from GregorianCalendar
-    if (U_FAILURE(status) || !getTimeZone().useDaylightTime()) 
-        return FALSE;
-
-    // Force an update of the state of the Calendar.
-    ((HebrewCalendar*)this)->complete(status); // cast away const
-
-    return (UBool)(U_SUCCESS(status) ? (internalGet(UCAL_DST_OFFSET) != 0) : FALSE);
-}
-
-/**
- * The system maintains a static default century start date and Year.  They are
- * initialized the first time they are used.  Once the system default century date 
- * and year are set, they do not change.
- */
-static UDate           gSystemDefaultCenturyStart       = DBL_MIN;
-static int32_t         gSystemDefaultCenturyStartYear   = -1;
-static icu::UInitOnce  gSystemDefaultCenturyInit        = U_INITONCE_INITIALIZER;
-
-UBool HebrewCalendar::haveDefaultCentury() const
-{
-    return TRUE;
-}
-
-static void U_CALLCONV initializeSystemDefaultCentury()
-{
-    // initialize systemDefaultCentury and systemDefaultCenturyYear based
-    // on the current time.  They'll be set to 80 years before
-    // the current time.
-    UErrorCode status = U_ZERO_ERROR;
-    HebrewCalendar calendar(Locale("@calendar=hebrew"),status);
-    if (U_SUCCESS(status)) {
-        calendar.setTime(Calendar::getNow(), status);
-        calendar.add(UCAL_YEAR, -80, status);
-
-        gSystemDefaultCenturyStart = calendar.getTime(status);
-        gSystemDefaultCenturyStartYear = calendar.get(UCAL_YEAR, status);
+    int32_t year = get(UCAL_EXTENDED_YEAR, status);
+    if (U_FAILURE(status)) {
+        return 0;
     }
-    // We have no recourse upon failure unless we want to propagate the failure
-    // out.
+    return year + kHebrewRelatedYearDiff;
 }
 
-
-UDate HebrewCalendar::defaultCenturyStart() const {
-    // lazy-evaluate systemDefaultCenturyStart
-    umtx_initOnce(gSystemDefaultCenturyInit, &initializeSystemDefaultCentury);
-    return gSystemDefaultCenturyStart;
+void HebrewCalendar::setRelatedYear(int32_t year)
+{
+    // set extended year
+    set(UCAL_EXTENDED_YEAR, year - kHebrewRelatedYearDiff);
 }
 
-int32_t HebrewCalendar::defaultCenturyStartYear() const {
-    // lazy-evaluate systemDefaultCenturyStartYear
-    umtx_initOnce(gSystemDefaultCenturyInit, &initializeSystemDefaultCentury);
-    return gSystemDefaultCenturyStartYear;
+IMPL_SYSTEM_DEFAULT_CENTURY(HebrewCalendar, "@calendar=hebrew")
+
+bool HebrewCalendar::inTemporalLeapYear(UErrorCode& status) const {
+    if (U_FAILURE(status)) {
+        return false;
+    }
+    int32_t eyear = get(UCAL_EXTENDED_YEAR, status);
+    if (U_FAILURE(status)) {
+        return false;
+    }
+    return isLeapYear(eyear);
 }
 
+static const char * const gTemporalMonthCodesForHebrew[] = {
+    "M01", "M02", "M03", "M04", "M05", "M05L", "M06",
+    "M07", "M08", "M09", "M10", "M11", "M12", nullptr
+};
+
+const char* HebrewCalendar::getTemporalMonthCode(UErrorCode& status) const {
+    int32_t month = get(UCAL_MONTH, status);
+    if (U_FAILURE(status)) {
+        return nullptr;
+    }
+    return gTemporalMonthCodesForHebrew[month];
+}
+
+void HebrewCalendar::setTemporalMonthCode(const char* code, UErrorCode& status )
+{
+    if (U_FAILURE(status)) {
+        return;
+    }
+    int32_t len = static_cast<int32_t>(uprv_strlen(code));
+    if (len == 3 || len == 4) {
+        for (int m = 0; gTemporalMonthCodesForHebrew[m] != nullptr; m++) {
+            if (uprv_strcmp(code, gTemporalMonthCodesForHebrew[m]) == 0) {
+                set(UCAL_MONTH, m);
+                return;
+            }
+        }
+    }
+    status = U_ILLEGAL_ARGUMENT_ERROR;
+}
+
+int32_t HebrewCalendar::internalGetMonth(UErrorCode& status) const {
+    if (U_FAILURE(status)) {
+        return 0;
+    }
+    if (resolveFields(kMonthPrecedence) == UCAL_ORDINAL_MONTH) {
+        int32_t ordinalMonth = internalGet(UCAL_ORDINAL_MONTH);
+        HebrewCalendar* nonConstThis = const_cast<HebrewCalendar*>(this); // cast away const
+
+        int32_t year = nonConstThis->handleGetExtendedYear(status);
+        if (U_FAILURE(status)) {
+            return 0;
+        }
+        if (isLeapYear(year) || ordinalMonth <= ADAR_1) {
+            return ordinalMonth;
+        }
+        if (!uprv_add32_overflow(ordinalMonth, 1, &ordinalMonth)) {
+            return ordinalMonth;
+        }
+    }
+    return Calendar::internalGetMonth(status);
+}
 
 UOBJECT_DEFINE_RTTI_IMPLEMENTATION(HebrewCalendar)
 
