@@ -20,6 +20,7 @@
 #include "charstr.h"
 #include "number_utils.h"
 #include "uassert.h"
+#include "util.h"
 
 using namespace icu;
 using namespace icu::number;
@@ -43,7 +44,7 @@ inline int32_t safeSubtract(int32_t a, int32_t b) {
     return diff;
 }
 
-static double DOUBLE_MULTIPLIERS[] = {
+double DOUBLE_MULTIPLIERS[] = {
         1e0,
         1e1,
         1e2,
@@ -88,7 +89,7 @@ DecimalQuantity::DecimalQuantity(const DecimalQuantity &other) {
     *this = other;
 }
 
-DecimalQuantity::DecimalQuantity(DecimalQuantity&& src) U_NOEXCEPT {
+DecimalQuantity::DecimalQuantity(DecimalQuantity&& src) noexcept {
     *this = std::move(src);
 }
 
@@ -101,7 +102,7 @@ DecimalQuantity &DecimalQuantity::operator=(const DecimalQuantity &other) {
     return *this;
 }
 
-DecimalQuantity& DecimalQuantity::operator=(DecimalQuantity&& src) U_NOEXCEPT {
+DecimalQuantity& DecimalQuantity::operator=(DecimalQuantity&& src) noexcept {
     if (this == &src) {
         return *this;
     }
@@ -120,6 +121,7 @@ void DecimalQuantity::copyFieldsFrom(const DecimalQuantity& other) {
     origDouble = other.origDouble;
     origDelta = other.origDelta;
     isApproximate = other.isApproximate;
+    exponent = other.exponent;
 }
 
 void DecimalQuantity::clear() {
@@ -129,18 +131,24 @@ void DecimalQuantity::clear() {
     setBcdToZero(); // sets scale, precision, hasDouble, origDouble, origDelta, and BCD data
 }
 
-void DecimalQuantity::setMinInteger(int32_t minInt) {
+void DecimalQuantity::decreaseMinIntegerTo(int32_t minInt) {
+    // Validation should happen outside of DecimalQuantity, e.g., in the Precision class.
+    U_ASSERT(minInt >= 0);
+
+    if (lReqPos > minInt) {
+        lReqPos = minInt;
+    }
+}
+
+void DecimalQuantity::increaseMinIntegerTo(int32_t minInt) {
     // Validation should happen outside of DecimalQuantity, e.g., in the Precision class.
     U_ASSERT(minInt >= 0);
 
     // Special behavior: do not set minInt to be less than what is already set.
     // This is so significant digits rounding can set the integer length.
-    if (minInt < lReqPos) {
-        minInt = lReqPos;
+    if (lReqPos < minInt) {
+        lReqPos = minInt;
     }
-
-    // Save values into internal state
-    lReqPos = minInt;
 }
 
 void DecimalQuantity::setMinFraction(int32_t minFrac) {
@@ -179,20 +187,22 @@ uint64_t DecimalQuantity::getPositionFingerprint() const {
     return fingerprint;
 }
 
-void DecimalQuantity::roundToIncrement(double roundingIncrement, RoundingMode roundingMode,
-                                       UErrorCode& status) {
+void DecimalQuantity::roundToIncrement(
+        uint64_t increment,
+        digits_t magnitude,
+        RoundingMode roundingMode,
+        UErrorCode& status) {
     // Do not call this method with an increment having only a 1 or a 5 digit!
     // Use a more efficient call to either roundToMagnitude() or roundToNickel().
     // Check a few popular rounding increments; a more thorough check is in Java.
-    U_ASSERT(roundingIncrement != 0.01);
-    U_ASSERT(roundingIncrement != 0.05);
-    U_ASSERT(roundingIncrement != 0.1);
-    U_ASSERT(roundingIncrement != 0.5);
-    U_ASSERT(roundingIncrement != 1);
-    U_ASSERT(roundingIncrement != 5);
+    U_ASSERT(increment != 1);
+    U_ASSERT(increment != 5);
 
+    DecimalQuantity incrementDQ;
+    incrementDQ.setToLong(increment);
+    incrementDQ.adjustMagnitude(magnitude);
     DecNum incrementDN;
-    incrementDN.setTo(roundingIncrement, status);
+    incrementDQ.toDecNum(incrementDN, status);
     if (U_FAILURE(status)) { return; }
 
     // Divide this DecimalQuantity by the increment, round, then multiply back.
@@ -252,6 +262,12 @@ bool DecimalQuantity::adjustMagnitude(int32_t delta) {
     return false;
 }
 
+int32_t DecimalQuantity::adjustToZeroScale() {
+    int32_t retval = scale;
+    scale = 0;
+    return retval;
+}
+
 double DecimalQuantity::getPluralOperand(PluralOperand operand) const {
     // If this assertion fails, you need to call roundToInfinity() or some other rounding method.
     // See the comment at the top of this file explaining the "isApproximate" field.
@@ -269,11 +285,29 @@ double DecimalQuantity::getPluralOperand(PluralOperand operand) const {
             return fractionCount();
         case PLURAL_OPERAND_W:
             return fractionCountWithoutTrailingZeros();
+        case PLURAL_OPERAND_E:
+            return static_cast<double>(getExponent());
+        case PLURAL_OPERAND_C:
+            // Plural operand `c` is currently an alias for `e`.
+            return static_cast<double>(getExponent());
         default:
             return std::abs(toDouble());
     }
 }
 
+int32_t DecimalQuantity::getExponent() const {
+    return exponent;
+}
+
+void DecimalQuantity::adjustExponent(int delta) {
+    exponent = exponent + delta;
+}
+
+void DecimalQuantity::resetExponent() {
+    adjustMagnitude(exponent);
+    exponent = 0;
+}
+
 bool DecimalQuantity::hasIntegerValue() const {
     return scale >= 0;
 }
@@ -307,11 +341,13 @@ int8_t DecimalQuantity::getDigit(int32_t magnitude) const {
 }
 
 int32_t DecimalQuantity::fractionCount() const {
-    return -getLowerDisplayMagnitude();
+    int32_t fractionCountWithExponent = -getLowerDisplayMagnitude() - exponent;
+    return fractionCountWithExponent > 0 ? fractionCountWithExponent : 0;
 }
 
 int32_t DecimalQuantity::fractionCountWithoutTrailingZeros() const {
-    return -scale > 0 ? -scale : 0;  // max(-scale, 0)
+    int32_t fractionCountWithExponent = -scale - exponent;
+    return fractionCountWithExponent > 0 ? fractionCountWithExponent : 0;  // max(-fractionCountWithExponent, 0)
 }
 
 bool DecimalQuantity::isNegative() const {
@@ -319,10 +355,14 @@ bool DecimalQuantity::isNegative() const {
 }
 
 Signum DecimalQuantity::signum() const {
-    if (isNegative()) {
+    bool isZero = (isZeroish() && !isInfinite());
+    bool isNeg = isNegative();
+    if (isZero && isNeg) {
+        return SIGNUM_NEG_ZERO;
+    } else if (isZero) {
+        return SIGNUM_POS_ZERO;
+    } else if (isNeg) {
         return SIGNUM_NEG;
-    } else if (isZeroish() && !isInfinite()) {
-        return SIGNUM_ZERO;
     } else {
         return SIGNUM_POS;
     }
@@ -422,9 +462,6 @@ void DecimalQuantity::_setToDoubleFast(double n) {
     // TODO: Make a fast path for other types of doubles.
     if (!std::numeric_limits<double>::is_iec559) {
         convertToAccurateDouble();
-        // Turn off the approximate double flag, since the value is now exact.
-        isApproximate = false;
-        origDouble = 0.0;
         return;
     }
 
@@ -439,8 +476,14 @@ void DecimalQuantity::_setToDoubleFast(double n) {
         return;
     }
 
+    if (exponent == -1023 || exponent == 1024) {
+        // The extreme values of exponent are special; use slow path.
+        convertToAccurateDouble();
+        return;
+    }
+
     // 3.3219... is log2(10)
-    auto fracLength = static_cast<int32_t> ((52 - exponent) / 3.32192809489);
+    auto fracLength = static_cast<int32_t> ((52 - exponent) / 3.32192809488736234787031942948939017586);
     if (fracLength >= 0) {
         int32_t i = fracLength;
         // 1e22 is the largest exact double.
@@ -452,7 +495,7 @@ void DecimalQuantity::_setToDoubleFast(double n) {
         for (; i <= -22; i += 22) n /= 1e22;
         n /= DOUBLE_MULTIPLIERS[-i];
     }
-    auto result = static_cast<int64_t>(std::round(n));
+    auto result = static_cast<int64_t>(uprv_round(n));
     if (result != 0) {
         _setToLong(result);
         scale -= fracLength;
@@ -510,23 +553,86 @@ void DecimalQuantity::_setToDecNum(const DecNum& decnum, UErrorCode& status) {
     if (decnum.isNegative()) {
         flags |= NEGATIVE_FLAG;
     }
-    if (!decnum.isZero()) {
+    if (decnum.isNaN()) {
+        flags |= NAN_FLAG;
+    } else if (decnum.isInfinity()) {
+        flags |= INFINITY_FLAG;
+    } else if (!decnum.isZero()) {
         readDecNumberToBcd(decnum);
         compact();
     }
 }
 
+DecimalQuantity DecimalQuantity::fromExponentString(UnicodeString num, UErrorCode& status) {
+    if (num.indexOf(u'e') >= 0 || num.indexOf(u'c') >= 0
+                || num.indexOf(u'E') >= 0 || num.indexOf(u'C') >= 0) {
+        int32_t ePos = num.lastIndexOf('e');
+        if (ePos < 0) {
+            ePos = num.lastIndexOf('c');
+        }
+        if (ePos < 0) {
+            ePos = num.lastIndexOf('E');
+        }
+        if (ePos < 0) {
+            ePos = num.lastIndexOf('C');
+        }
+        int32_t expNumPos = ePos + 1;
+        UnicodeString exponentStr = num.tempSubString(expNumPos, num.length() - expNumPos);
+
+        // parse exponentStr into exponent, but note that parseAsciiInteger doesn't handle the minus sign
+        bool isExpStrNeg = num[expNumPos] == u'-';
+        int32_t exponentParsePos = isExpStrNeg ? 1 : 0;
+        int32_t exponent = ICU_Utility::parseAsciiInteger(exponentStr, exponentParsePos);
+        exponent = isExpStrNeg ? -exponent : exponent;
+
+        // Compute the decNumber representation
+        UnicodeString fractionStr = num.tempSubString(0, ePos);
+        CharString fracCharStr = CharString();
+        fracCharStr.appendInvariantChars(fractionStr, status);
+        DecNum decnum;
+        decnum.setTo(fracCharStr.toStringPiece(), status);
+
+        // Clear and set this DecimalQuantity instance
+        DecimalQuantity dq;
+        dq.setToDecNum(decnum, status);
+        int32_t numFracDigit = getVisibleFractionCount(fractionStr);
+        dq.setMinFraction(numFracDigit);
+        dq.adjustExponent(exponent);
+
+        return dq;
+    } else {
+        DecimalQuantity dq;
+        int numFracDigit = getVisibleFractionCount(num);
+
+        CharString numCharStr = CharString();
+        numCharStr.appendInvariantChars(num, status);
+        dq.setToDecNumber(numCharStr.toStringPiece(), status);
+
+        dq.setMinFraction(numFracDigit);
+        return dq;
+    }
+}
+
+int32_t DecimalQuantity::getVisibleFractionCount(UnicodeString value) {
+    int decimalPos = value.indexOf('.') + 1;
+    if (decimalPos == 0) {
+        return 0;
+    } else {
+        return value.length() - decimalPos;
+    }
+}
+
 int64_t DecimalQuantity::toLong(bool truncateIfOverflow) const {
     // NOTE: Call sites should be guarded by fitsInLong(), like this:
     // if (dq.fitsInLong()) { /* use dq.toLong() */ } else { /* use some fallback */ }
     // Fallback behavior upon truncateIfOverflow is to truncate at 17 digits.
     uint64_t result = 0L;
-    int32_t upperMagnitude = scale + precision - 1;
+    int32_t upperMagnitude = exponent + scale + precision - 1;
     if (truncateIfOverflow) {
         upperMagnitude = std::min(upperMagnitude, 17);
     }
     for (int32_t magnitude = upperMagnitude; magnitude >= 0; magnitude--) {
-        result = result * 10 + getDigitPos(magnitude - scale);
+        result = result * 10 + getDigitPos(magnitude - scale - exponent);
     }
     if (isNegative()) {
         return static_cast<int64_t>(0LL - result); // i.e., -result
@@ -536,7 +642,7 @@ int64_t DecimalQuantity::toLong(bool truncateIfOverflow) const {
 
 uint64_t DecimalQuantity::toFractionLong(bool includeTrailingZeros) const {
     uint64_t result = 0L;
-    int32_t magnitude = -1;
+    int32_t magnitude = -1 - exponent;
     int32_t lowerMagnitude = scale;
     if (includeTrailingZeros) {
         lowerMagnitude = std::min(lowerMagnitude, rReqPos);
@@ -560,7 +666,7 @@ bool DecimalQuantity::fitsInLong(bool ignoreFraction) const {
     if (isZeroish()) {
         return true;
     }
-    if (scale < 0 && !ignoreFraction) {
+    if (exponent + scale < 0 && !ignoreFraction) {
         return false;
     }
     int magnitude = getMagnitude();
@@ -606,19 +712,24 @@ double DecimalQuantity::toDouble() const {
             &count);
 }
 
-void DecimalQuantity::toDecNum(DecNum& output, UErrorCode& status) const {
+DecNum& DecimalQuantity::toDecNum(DecNum& output, UErrorCode& status) const {
     // Special handling for zero
     if (precision == 0) {
         output.setTo("0", status);
+        return output;
     }
 
     // Use the BCD constructor. We need to do a little bit of work to convert, though.
     // The decNumber constructor expects most-significant first, but we store least-significant first.
-    MaybeStackArray<uint8_t, 20> ubcd(precision);
+    MaybeStackArray<uint8_t, 20> ubcd(precision, status);
+    if (U_FAILURE(status)) {
+        return output;
+    }
     for (int32_t m = 0; m < precision; m++) {
         ubcd[precision - m - 1] = static_cast<uint8_t>(getDigitPos(m));
     }
     output.setTo(ubcd.getAlias(), precision, scale, isNegative(), status);
+    return output;
 }
 
 void DecimalQuantity::truncate() {
@@ -794,6 +905,7 @@ void DecimalQuantity::roundToMagnitude(int32_t magnitude, RoundingMode roundingM
 
         // Perform truncation
         if (position >= precision) {
+            U_ASSERT(trailingDigit == 0);
             setBcdToZero();
             scale = magnitude;
         } else {
@@ -811,6 +923,10 @@ void DecimalQuantity::roundToMagnitude(int32_t magnitude, RoundingMode roundingM
                 // do not return: use the bubbling logic below
             } else {
                 setDigitPos(0, 5);
+                // If the quantity was set to 0, we may need to restore a digit.
+                if (precision == 0) {
+                    precision = 1;
+                }
                 // compact not necessary: digit at position 0 is nonzero
                 return;
             }
@@ -877,13 +993,69 @@ UnicodeString DecimalQuantity::toPlainString() const {
     if (isNegative()) {
         sb.append(u'-');
     }
-    if (precision == 0 || getMagnitude() < 0) {
+    if (precision == 0) {
+        sb.append(u'0');
+        return sb;
+    }
+    int32_t upper = scale + precision + exponent - 1;
+    int32_t lower = scale + exponent;
+    if (upper < lReqPos - 1) {
+        upper = lReqPos - 1;
+    }
+    if (lower > rReqPos) {
+        lower = rReqPos;
+    }    
+    int32_t p = upper;
+    if (p < 0) {
         sb.append(u'0');
     }
-    for (int m = getUpperDisplayMagnitude(); m >= getLowerDisplayMagnitude(); m--) {
-        if (m == -1) { sb.append(u'.'); }
-        sb.append(getDigit(m) + u'0');
+    for (; p >= 0; p--) {
+        sb.append(u'0' + getDigitPos(p - scale - exponent));
     }
+    if (lower < 0) {
+        sb.append(u'.');
+    }
+    for(; p >= lower; p--) {
+        sb.append(u'0' + getDigitPos(p - scale - exponent));
+    }
+    return sb;
+}
+
+
+UnicodeString DecimalQuantity::toExponentString() const {
+    U_ASSERT(!isApproximate);
+    UnicodeString sb;
+    if (isNegative()) {
+        sb.append(u'-');
+    }
+
+    int32_t upper = scale + precision - 1;
+    int32_t lower = scale;
+    if (upper < lReqPos - 1) {
+        upper = lReqPos - 1;
+    }
+    if (lower > rReqPos) {
+        lower = rReqPos;
+    }    
+    int32_t p = upper;
+    if (p < 0) {
+        sb.append(u'0');
+    }
+    for (; p >= 0; p--) {
+        sb.append(u'0' + getDigitPos(p - scale));
+    }
+    if (lower < 0) {
+        sb.append(u'.');
+    }
+    for(; p >= lower; p--) {
+        sb.append(u'0' + getDigitPos(p - scale));
+    }
+
+    if (exponent != 0) {
+        sb.append(u'c');
+        ICU_Utility::appendNumber(sb, exponent);        
+    }
+
     return sb;
 }
 
@@ -908,9 +1080,9 @@ UnicodeString DecimalQuantity::toScientificString() const {
         }
     }
     result.append(u'E');
-    int32_t _scale = upperPos + scale;
+    int32_t _scale = upperPos + scale + exponent;
     if (_scale == INT32_MIN) {
-        result.append({u"-2147483648", -1});
+        result.append(u"-2147483648");
         return result;
     } else if (_scale < 0) {
         _scale *= -1;
@@ -941,7 +1113,7 @@ int8_t DecimalQuantity::getDigitPos(int32_t position) const {
         return fBCD.bcdBytes.ptr[position];
     } else {
         if (position < 0 || position >= 16) { return 0; }
-        return (int8_t) ((fBCD.bcdLong >> (position * 4)) & 0xf);
+        return static_cast<int8_t>((fBCD.bcdLong >> (position * 4)) & 0xf);
     }
 }
 
@@ -956,7 +1128,7 @@ void DecimalQuantity::setDigitPos(int32_t position, int8_t value) {
         fBCD.bcdBytes.ptr[position] = value;
     } else {
         int shift = position * 4;
-        fBCD.bcdLong = (fBCD.bcdLong & ~(0xfL << shift)) | ((long) value << shift);
+        fBCD.bcdLong = (fBCD.bcdLong & ~(0xfL << shift)) | (static_cast<long>(value) << shift);
     }
 }
 
@@ -966,13 +1138,8 @@ void DecimalQuantity::shiftLeft(int32_t numDigits) {
     }
     if (usingBytes) {
         ensureCapacity(precision + numDigits);
-        int i = precision + numDigits - 1;
-        for (; i >= numDigits; i--) {
-            fBCD.bcdBytes.ptr[i] = fBCD.bcdBytes.ptr[i - numDigits];
-        }
-        for (; i >= 0; i--) {
-            fBCD.bcdBytes.ptr[i] = 0;
-        }
+        uprv_memmove(fBCD.bcdBytes.ptr + numDigits, fBCD.bcdBytes.ptr, precision);
+        uprv_memset(fBCD.bcdBytes.ptr, 0, numDigits);
     } else {
         fBCD.bcdLong <<= (numDigits * 4);
     }
@@ -1021,6 +1188,7 @@ void DecimalQuantity::setBcdToZero() {
     isApproximate = false;
     origDouble = 0;
     origDelta = 0;
+    exponent = 0;
 }
 
 void DecimalQuantity::readIntToBcd(int32_t n) {
@@ -1160,7 +1328,7 @@ void DecimalQuantity::ensureCapacity(int32_t capacity) {
         // Initialize the byte array to zeros (this is done automatically in Java)
         uprv_memset(fBCD.bcdBytes.ptr, 0, capacity * sizeof(int8_t));
     } else if (oldCapacity < capacity) {
-        auto bcd1 = static_cast<int8_t*>(uprv_malloc(capacity * 2 * sizeof(int8_t)));
+        auto* bcd1 = static_cast<int8_t*>(uprv_malloc(capacity * 2 * sizeof(int8_t)));
         uprv_memcpy(bcd1, fBCD.bcdBytes.ptr, oldCapacity * sizeof(int8_t));
         // Initialize the rest of the byte array to zeros (this is done automatically in Java)
         uprv_memset(bcd1 + oldCapacity, 0, (capacity - oldCapacity) * sizeof(int8_t));
@@ -1285,7 +1453,11 @@ bool DecimalQuantity::operator==(const DecimalQuantity& other) const {
 }
 
 UnicodeString DecimalQuantity::toString() const {
-    MaybeStackArray<char, 30> digits(precision + 1);
+    UErrorCode localStatus = U_ZERO_ERROR;
+    MaybeStackArray<char, 30> digits(precision + 1, localStatus);
+    if (U_FAILURE(localStatus)) {
+        return ICU_Utility::makeBogusString();
+    }
     for (int32_t i = 0; i < precision; i++) {
         digits[i] = getDigitPos(precision - i - 1) + '0';
     }
