@@ -1,12 +1,14 @@
+// © 2016 and later: Unicode, Inc. and others.
+// License & terms of use: http://www.unicode.org/copyright.html
 /*
 *******************************************************************************
 *
-*   Copyright (C) 1999-2003, International Business Machines
+*   Copyright (C) 1999-2012, International Business Machines
 *   Corporation and others.  All Rights Reserved.
 *
 *******************************************************************************
 *   file name:  umsg.cpp
-*   encoding:   US-ASCII
+*   encoding:   UTF-8
 *   tab size:   8 (not used)
 *   indentation:4
 *
@@ -29,21 +31,44 @@
 #include "unicode/msgfmt.h"
 #include "unicode/unistr.h"
 #include "cpputils.h"
+#include "uassert.h"
+#include "ustr_imp.h"
+
+U_NAMESPACE_BEGIN
+/**
+ * This class isolates our access to private internal methods of
+ * MessageFormat.  It is never instantiated; it exists only for C++
+ * access management.
+ */
+class MessageFormatAdapter {
+public:
+    static const Formattable::Type* getArgTypeList(const MessageFormat& m,
+                                                   int32_t& count);
+    static UBool hasArgTypeConflicts(const MessageFormat& m) {
+        return m.hasArgTypeConflicts;
+    }
+};
+const Formattable::Type*
+MessageFormatAdapter::getArgTypeList(const MessageFormat& m,
+                                     int32_t& count) {
+    return m.getArgTypeList(count);
+}
+U_NAMESPACE_END
 
 U_NAMESPACE_USE
 
 U_CAPI int32_t
 u_formatMessage(const char  *locale,
-                const UChar *pattern,
+                const char16_t *pattern,
                 int32_t     patternLength,
-                UChar       *result,
+                char16_t    *result,
                 int32_t     resultLength,
                 UErrorCode  *status,
                 ...)
 {
     va_list    ap;
     int32_t actLen;        
-    //argument checking defered to subsequent method calls
+    //argument checking deferred to subsequent method calls
     // start vararg processing
     va_start(ap, status);
 
@@ -56,16 +81,16 @@ u_formatMessage(const char  *locale,
 
 U_CAPI int32_t U_EXPORT2
 u_vformatMessage(   const char  *locale,
-                    const UChar *pattern,
+                    const char16_t *pattern,
                     int32_t     patternLength,
-                    UChar       *result,
+                    char16_t    *result,
                     int32_t     resultLength,
                     va_list     ap,
                     UErrorCode  *status)
 
 {
-    //argument checking defered to subsequent method calls
-    UMessageFormat *fmt = umsg_open(pattern,patternLength,locale,NULL,status);
+    //argument checking deferred to subsequent method calls
+    UMessageFormat *fmt = umsg_open(pattern,patternLength,locale,nullptr,status);
     int32_t retVal = umsg_vformat(fmt,result,resultLength,ap,status);
     umsg_close(fmt);
     return retVal;
@@ -73,9 +98,9 @@ u_vformatMessage(   const char  *locale,
 
 U_CAPI int32_t
 u_formatMessageWithError(const char *locale,
-                        const UChar *pattern,
+                        const char16_t *pattern,
                         int32_t     patternLength,
-                        UChar       *result,
+                        char16_t    *result,
                         int32_t     resultLength,
                         UParseError *parseError,
                         UErrorCode  *status,
@@ -83,7 +108,7 @@ u_formatMessageWithError(const char *locale,
 {
     va_list    ap;
     int32_t actLen;
-    //argument checking defered to subsequent method calls
+    //argument checking deferred to subsequent method calls
     // start vararg processing
     va_start(ap, status);
 
@@ -96,16 +121,16 @@ u_formatMessageWithError(const char *locale,
 
 U_CAPI int32_t U_EXPORT2
 u_vformatMessageWithError(  const char  *locale,
-                            const UChar *pattern,
+                            const char16_t *pattern,
                             int32_t     patternLength,
-                            UChar       *result,
+                            char16_t    *result,
                             int32_t     resultLength,
                             UParseError *parseError,
                             va_list     ap,
                             UErrorCode  *status)
 
 {
-    //argument checking defered to subsequent method calls
+    //argument checking deferred to subsequent method calls
     UMessageFormat *fmt = umsg_open(pattern,patternLength,locale,parseError,status);
     int32_t retVal = umsg_vformat(fmt,result,resultLength,ap,status);
     umsg_close(fmt);
@@ -119,15 +144,15 @@ u_vformatMessageWithError(  const char  *locale,
 //  3. Iterate through each formattable returned, and assign to the arguments
 U_CAPI void
 u_parseMessage( const char   *locale,
-                const UChar  *pattern,
+                const char16_t  *pattern,
                 int32_t      patternLength,
-                const UChar  *source,
+                const char16_t  *source,
                 int32_t      sourceLength,
                 UErrorCode   *status,
                 ...)
 {
     va_list    ap;
-    //argument checking defered to subsequent method calls
+    //argument checking deferred to subsequent method calls
 
     // start vararg processing
     va_start(ap, status);
@@ -139,15 +164,15 @@ u_parseMessage( const char   *locale,
 
 U_CAPI void U_EXPORT2
 u_vparseMessage(const char  *locale,
-                const UChar *pattern,
+                const char16_t *pattern,
                 int32_t     patternLength,
-                const UChar *source,
+                const char16_t *source,
                 int32_t     sourceLength,
                 va_list     ap,
                 UErrorCode  *status)
 {
-    //argument checking defered to subsequent method calls
-    UMessageFormat *fmt = umsg_open(pattern,patternLength,locale,NULL,status);
+    //argument checking deferred to subsequent method calls
+    UMessageFormat *fmt = umsg_open(pattern,patternLength,locale,nullptr,status);
     int32_t count = 0;
     umsg_vparse(fmt,source,sourceLength,&count,ap,status);
     umsg_close(fmt);
@@ -155,9 +180,9 @@ u_vparseMessage(const char  *locale,
 
 U_CAPI void
 u_parseMessageWithError(const char  *locale,
-                        const UChar *pattern,
+                        const char16_t *pattern,
                         int32_t     patternLength,
-                        const UChar *source,
+                        const char16_t *source,
                         int32_t     sourceLength,
                         UParseError *error,
                         UErrorCode  *status,
@@ -165,7 +190,7 @@ u_parseMessageWithError(const char  *locale,
 {
     va_list    ap;
 
-    //argument checking defered to subsequent method calls
+    //argument checking deferred to subsequent method calls
 
     // start vararg processing
     va_start(ap, status);
@@ -176,15 +201,15 @@ u_parseMessageWithError(const char  *locale,
 }
 U_CAPI void U_EXPORT2
 u_vparseMessageWithError(const char  *locale,
-                         const UChar *pattern,
+                         const char16_t *pattern,
                          int32_t     patternLength,
-                         const UChar *source,
+                         const char16_t *source,
                          int32_t     sourceLength,
                          va_list     ap,
                          UParseError *error,
                          UErrorCode* status)
 {
-    //argument checking defered to subsequent method calls
+    //argument checking deferred to subsequent method calls
     UMessageFormat *fmt = umsg_open(pattern,patternLength,locale,error,status);
     int32_t count = 0;
     umsg_vparse(fmt,source,sourceLength,&count,ap,status);
@@ -198,49 +223,47 @@ u_vparseMessageWithError(const char  *locale,
 
 
 U_CAPI UMessageFormat* U_EXPORT2
-umsg_open(  const UChar     *pattern,
+umsg_open(  const char16_t  *pattern,
             int32_t         patternLength,
             const  char     *locale,
             UParseError     *parseError,
             UErrorCode      *status)
 {
     //check arguments
-    if(status==NULL || U_FAILURE(*status))
+    if(status==nullptr || U_FAILURE(*status))
     {
-      return 0;
+      return nullptr;
     }
-    if(pattern==NULL||patternLength<-1){
+    if(pattern==nullptr||patternLength<-1){
         *status=U_ILLEGAL_ARGUMENT_ERROR;
-        return 0;
+        return nullptr;
     }
 
     UParseError tErr;
-   
-    if(parseError==NULL)
+    if(parseError==nullptr)
     {
         parseError = &tErr;
     }
-        
-    UMessageFormat* retVal = 0;
 
     int32_t len = (patternLength == -1 ? u_strlen(pattern) : patternLength);
-    
-    UnicodeString patString((patternLength == -1 ? TRUE:FALSE), pattern,len);
+    UnicodeString patString(patternLength == -1, pattern, len);
 
-    retVal = (UMessageFormat*) new MessageFormat(pattern,Locale(locale),*parseError,*status);
-    
-    if(retVal == 0) {
+    MessageFormat* retVal = new MessageFormat(patString,Locale(locale),*parseError,*status);
+    if(retVal == nullptr) {
         *status = U_MEMORY_ALLOCATION_ERROR;
-        return 0;
+        return nullptr;
     }
-    return retVal;
+    if (U_SUCCESS(*status) && MessageFormatAdapter::hasArgTypeConflicts(*retVal)) {
+        *status = U_ARGUMENT_TYPE_MISMATCH;
+    }
+    return (UMessageFormat*)retVal;
 }
 
 U_CAPI void U_EXPORT2
 umsg_close(UMessageFormat* format)
 {
     //check arguments
-    if(format==NULL){
+    if(format==nullptr){
         return;
     }
     delete (MessageFormat*) format;
@@ -251,17 +274,17 @@ umsg_clone(const UMessageFormat *fmt,
            UErrorCode *status)
 {
     //check arguments
-    if(status==NULL || U_FAILURE(*status)){
-        return NULL;
+    if(status==nullptr || U_FAILURE(*status)){
+        return nullptr;
     }
-    if(fmt==NULL){
+    if(fmt==nullptr){
         *status = U_ILLEGAL_ARGUMENT_ERROR;
-        return NULL;
+        return nullptr;
     }
     UMessageFormat retVal = (UMessageFormat)((MessageFormat*)fmt)->clone();
-    if(retVal == 0) {
+    if (retVal == nullptr) {
         *status = U_MEMORY_ALLOCATION_ERROR;
-        return 0;
+        return nullptr;
     }
     return retVal;    
 }
@@ -270,86 +293,84 @@ U_CAPI void  U_EXPORT2
 umsg_setLocale(UMessageFormat *fmt, const char* locale)
 {
     //check arguments
-    if(fmt==NULL){
+    if(fmt==nullptr){
         return;
     }
     ((MessageFormat*)fmt)->setLocale(Locale(locale));   
 }
 
 U_CAPI const char*  U_EXPORT2
-umsg_getLocale(UMessageFormat *fmt)
+umsg_getLocale(const UMessageFormat *fmt)
 {
     //check arguments
-    if(fmt==NULL){
+    if(fmt==nullptr){
         return "";
     }
-    return ((MessageFormat*)fmt)->getLocale().getName();
+    return ((const MessageFormat*)fmt)->getLocale().getName();
 }
 
 U_CAPI void  U_EXPORT2
 umsg_applyPattern(UMessageFormat *fmt,
-                           const UChar* pattern,
+                           const char16_t* pattern,
                            int32_t patternLength,
                            UParseError* parseError,
                            UErrorCode* status)
 {
     //check arguments
     UParseError tErr;
-    if(status ==NULL||U_FAILURE(*status)){
+    if(status ==nullptr||U_FAILURE(*status)){
         return ;
     }
-    if(fmt==NULL||pattern==NULL||patternLength<-1){
+    if(fmt==nullptr || (pattern==nullptr && patternLength!=0) || patternLength<-1) {
         *status=U_ILLEGAL_ARGUMENT_ERROR;
         return ;
     }
 
-    if(parseError==NULL){
+    if(parseError==nullptr){
       parseError = &tErr;
     }
-    if(patternLength<-1){
-        patternLength=u_strlen(pattern);
-    }
 
+    // UnicodeString(pattern, -1) calls u_strlen().
     ((MessageFormat*)fmt)->applyPattern(UnicodeString(pattern,patternLength),*parseError,*status);  
 }
 
 U_CAPI int32_t  U_EXPORT2
-umsg_toPattern(UMessageFormat *fmt,
-               UChar* result, 
+umsg_toPattern(const UMessageFormat *fmt,
+               char16_t* result,
                int32_t resultLength,
                UErrorCode* status)
 {
     //check arguments
-    if(status ==NULL||U_FAILURE(*status)){
+    if(status ==nullptr||U_FAILURE(*status)){
         return -1;
     }
-    if(fmt==NULL||resultLength<0 || (resultLength>0 && result==0)){
+    if (fmt == nullptr || resultLength < 0 || (resultLength > 0 && result == nullptr)) {
         *status=U_ILLEGAL_ARGUMENT_ERROR;
         return -1;
     }
 
 
     UnicodeString res;
-    if(!(result==NULL && resultLength==0)) {
-        // NULL destination for pure preflighting: empty dummy string
+    if(!(result==nullptr && resultLength==0)) {
+        // nullptr destination for pure preflighting: empty dummy string
         // otherwise, alias the destination buffer
         res.setTo(result, 0, resultLength);
     }
-    ((MessageFormat*)fmt)->toPattern(res);
+    ((const MessageFormat*)fmt)->toPattern(res);
     return res.extract(result, resultLength, *status);
 }
 
 U_CAPI int32_t
-umsg_format(    UMessageFormat *fmt,
-                UChar          *result,
+umsg_format(    const UMessageFormat *fmt,
+                char16_t       *result,
                 int32_t        resultLength,
                 UErrorCode     *status,
                 ...)
 {
     va_list    ap;
     int32_t actLen;  
-    //argument checking defered to last method call umsg_vformat which
-    //saves time when arguments are valid and we dont care when arguments are not
+    //argument checking deferred to last method call umsg_vformat which
+    //saves time when arguments are valid and we don't care when arguments are not
     //since we return an error anyway
 
     
@@ -364,44 +385,26 @@ umsg_format(    UMessageFormat *fmt,
     return actLen;
 }
 
-U_NAMESPACE_BEGIN
-/**
- * This class isolates our access to private internal methods of
- * MessageFormat.  It is never instantiated; it exists only for C++
- * access management.
- */
-class MessageFormatAdapter {
-public:
-    static const Formattable::Type* getArgTypeList(const MessageFormat& m,
-                                                   int32_t& count);
-};
-const Formattable::Type*
-MessageFormatAdapter::getArgTypeList(const MessageFormat& m,
-                                     int32_t& count) {
-    return m.getArgTypeList(count);
-}
-U_NAMESPACE_END
-
 U_CAPI int32_t U_EXPORT2
-umsg_vformat(   UMessageFormat *fmt,
-                UChar          *result,
+umsg_vformat(   const UMessageFormat *fmt,
+                char16_t       *result,
                 int32_t        resultLength,
                 va_list        ap,
                 UErrorCode     *status)
 {
     //check arguments
-    if(status==0 || U_FAILURE(*status))
+    if (status == nullptr || U_FAILURE(*status))
     {
         return -1;
     }
-    if(fmt==NULL||resultLength<0 || (resultLength>0 && result==0)) {
+    if (fmt == nullptr || resultLength < 0 || (resultLength > 0 && result == nullptr)) {
         *status=U_ILLEGAL_ARGUMENT_ERROR;
         return -1;
     }
 
     int32_t count =0;
     const Formattable::Type* argTypes =
-        MessageFormatAdapter::getArgTypeList(*(MessageFormat*)fmt, count);
+        MessageFormatAdapter::getArgTypeList(*(const MessageFormat*)fmt, count);
     // Allocate at least one element.  Allocating an array of length
     // zero causes problems on some platforms (e.g. Win32).
     Formattable* args = new Formattable[count ? count : 1];
@@ -409,7 +412,7 @@ umsg_vformat(   UMessageFormat *fmt,
     // iterate through the vararg list, and get the arguments out
     for(int32_t i = 0; i < count; ++i) {
         
-        UChar *stringVal;
+        char16_t *stringVal;
         double tDouble=0;
         int32_t tInt =0;
         int64_t tInt64 = 0;
@@ -437,9 +440,9 @@ umsg_vformat(   UMessageFormat *fmt,
             
         case Formattable::kString:
             // For some reason, a temporary is needed
-            stringVal = va_arg(ap, UChar*);
+            stringVal = va_arg(ap, char16_t*);
             if(stringVal){
-                args[i].setString(stringVal);
+                args[i].setString(UnicodeString(stringVal));
             }else{
                 *status=U_ILLEGAL_ARGUMENT_ERROR;
             }
@@ -453,13 +456,21 @@ umsg_vformat(   UMessageFormat *fmt,
             va_arg(ap, int);
             break;
 
+        case Formattable::kObject:
+            // Unused argument number. Read and ignore a pointer argument.
+            va_arg(ap, void*);
+            break;
+
+        default:
+            // Unknown/unsupported argument type.
+            UPRV_UNREACHABLE_EXIT;
         }
     }
     UnicodeString resultStr;
-    FieldPosition fieldPosition(0);
+    FieldPosition fieldPosition(FieldPosition::DONT_CARE);
     
     /* format the message */
-    ((MessageFormat*)fmt)->format(args,count,resultStr,fieldPosition,*status);
+    ((const MessageFormat*)fmt)->format(args,count,resultStr,fieldPosition,*status);
 
     delete[] args;
 
@@ -471,16 +482,16 @@ umsg_vformat(   UMessageFormat *fmt,
 }
 
 U_CAPI void
-umsg_parse( UMessageFormat *fmt,
-            const UChar    *source,
+umsg_parse( const UMessageFormat *fmt,
+            const char16_t *source,
             int32_t        sourceLength,
             int32_t        *count,
             UErrorCode     *status,
             ...)
 {
     va_list    ap;
-    //argument checking defered to last method call umsg_vparse which
-    //saves time when arguments are valid and we dont care when arguments are not
+    //argument checking deferred to last method call umsg_vparse which
+    //saves time when arguments are valid and we don't care when arguments are not
     //since we return an error anyway
 
     // start vararg processing
@@ -493,19 +504,19 @@ umsg_parse( UMessageFormat *fmt,
 }
 
 U_CAPI void U_EXPORT2
-umsg_vparse(UMessageFormat *fmt,
-            const UChar    *source,
+umsg_vparse(const UMessageFormat *fmt,
+            const char16_t *source,
             int32_t        sourceLength,
             int32_t        *count,
             va_list        ap,
             UErrorCode     *status)
 {
     //check arguments
-    if(status==NULL||U_FAILURE(*status))
+    if(status==nullptr||U_FAILURE(*status))
     {
         return;
     }
-    if(fmt==NULL||source==NULL || sourceLength<-1 || count==NULL){
+    if(fmt==nullptr||source==nullptr || sourceLength<-1 || count==nullptr){
         *status=U_ILLEGAL_ARGUMENT_ERROR;
         return;
     }
@@ -514,10 +525,10 @@ umsg_vparse(UMessageFormat *fmt,
     }
 
     UnicodeString srcString(source,sourceLength);
-    Formattable *args = ((MessageFormat*)fmt)->parse(source,*count,*status);
+    Formattable *args = ((const MessageFormat*)fmt)->parse(srcString,*count,*status);
     UDate *aDate;
     double *aDouble;
-    UChar *aString;
+    char16_t *aString;
     int32_t* aInt;
     int64_t* aInt64;
     UnicodeString temp;
@@ -547,7 +558,7 @@ umsg_vparse(UMessageFormat *fmt,
         case Formattable::kLong:
             aInt = va_arg(ap, int32_t*);
             if(aInt){
-                *aInt = (int32_t) args[i].getLong();
+                *aInt = args[i].getLong();
             }else{
                 *status=U_ILLEGAL_ARGUMENT_ERROR;
             }
@@ -563,7 +574,7 @@ umsg_vparse(UMessageFormat *fmt,
             break;
 
         case Formattable::kString:
-            aString = va_arg(ap, UChar*);
+            aString = va_arg(ap, char16_t*);
             if(aString){
                 args[i].getString(temp);
                 len = temp.length();
@@ -574,10 +585,16 @@ umsg_vparse(UMessageFormat *fmt,
             }
             break;
 
+        case Formattable::kObject:
+            // This will never happen because MessageFormat doesn't
+            // support kObject.  When MessageFormat is changed to
+            // understand MeasureFormats, modify this code to do the
+            // right thing. [alan]
+            UPRV_UNREACHABLE_EXIT;
+
         // better not happen!
         case Formattable::kArray:
-            // DIE
-            break;
+            UPRV_UNREACHABLE_EXIT;
         }
     }
 
@@ -585,18 +602,107 @@ umsg_vparse(UMessageFormat *fmt,
     delete [] args;
 }
 
-U_CAPI const char* U_EXPORT2
-umsg_getLocaleByType(const UMessageFormat *fmt,
-                     ULocDataLocaleType type,
-                     UErrorCode* status)
+#define SINGLE_QUOTE      ((char16_t)0x0027)
+#define CURLY_BRACE_LEFT  ((char16_t)0x007B)
+#define CURLY_BRACE_RIGHT ((char16_t)0x007D)
+
+#define STATE_INITIAL 0
+#define STATE_SINGLE_QUOTE 1
+#define STATE_IN_QUOTE 2
+#define STATE_MSG_ELEMENT 3
+
+#define MAppend(c) if (len < destCapacity) dest[len++] = c; else len++
+
+int32_t umsg_autoQuoteApostrophe(const char16_t* pattern,
+                 int32_t patternLength,
+                 char16_t* dest,
+                 int32_t destCapacity,
+                 UErrorCode* ec)
 {
-    if (fmt == NULL) {
-        if (U_SUCCESS(*status)) {
-            *status = U_ILLEGAL_ARGUMENT_ERROR;
-        }
-        return NULL;
+    int32_t state = STATE_INITIAL;
+    int32_t braceCount = 0;
+    int32_t len = 0;
+
+    if (ec == nullptr || U_FAILURE(*ec)) {
+        return -1;
     }
-    return ((Format*)fmt)->getLocaleID(type, *status);
+
+    if (pattern == nullptr || patternLength < -1 || (dest == nullptr && destCapacity > 0)) {
+        *ec = U_ILLEGAL_ARGUMENT_ERROR;
+        return -1;
+    }
+    U_ASSERT(destCapacity >= 0);
+
+    if (patternLength == -1) {
+        patternLength = u_strlen(pattern);
+    }
+
+    for (int i = 0; i < patternLength; ++i) {
+        char16_t c = pattern[i];
+        switch (state) {
+        case STATE_INITIAL:
+            switch (c) {
+            case SINGLE_QUOTE:
+                state = STATE_SINGLE_QUOTE;
+                break;
+            case CURLY_BRACE_LEFT:
+                state = STATE_MSG_ELEMENT;
+                ++braceCount;
+                break;
+            }
+            break;
+
+        case STATE_SINGLE_QUOTE:
+            switch (c) {
+            case SINGLE_QUOTE:
+                state = STATE_INITIAL;
+                break;
+            case CURLY_BRACE_LEFT:
+            case CURLY_BRACE_RIGHT:
+                state = STATE_IN_QUOTE;
+                break;
+            default:
+                MAppend(SINGLE_QUOTE);
+                state = STATE_INITIAL;
+                break;
+            }
+        break;
+
+        case STATE_IN_QUOTE:
+            switch (c) {
+            case SINGLE_QUOTE:
+                state = STATE_INITIAL;
+                break;
+            }
+            break;
+
+        case STATE_MSG_ELEMENT:
+            switch (c) {
+            case CURLY_BRACE_LEFT:
+                ++braceCount;
+                break;
+            case CURLY_BRACE_RIGHT:
+                if (--braceCount == 0) {
+                    state = STATE_INITIAL;
+                }
+                break;
+            }
+            break;
+
+        default: // Never happens.
+            break;
+        }
+
+        U_ASSERT(len >= 0);
+        MAppend(c);
+    }
+
+    // End of scan
+    if (state == STATE_SINGLE_QUOTE || state == STATE_IN_QUOTE) {
+        MAppend(SINGLE_QUOTE);
+    }
+
+    return u_terminateUChars(dest, destCapacity, len, ec);
 }
 
 #endif /* #if !UCONFIG_NO_FORMATTING */
