@@ -6,10 +6,13 @@
 #if !UCONFIG_NO_FORMATTING
 
 #include "bytesinkutil.h"
+#include "charstr.h"
 #include "cstring.h"
+#include "measunit_impl.h"
 #include "number_decimalquantity.h"
 #include "resource.h"
 #include "uassert.h"
+#include "ulocimp.h"
 #include "unicode/locid.h"
 #include "unicode/unistr.h"
 #include "unicode/ures.h"
@@ -78,6 +81,8 @@ class ConversionRateDataSink : public ResourceSink {
             UnicodeString baseUnit = ICU_Utility::makeBogusString();
             UnicodeString factor = ICU_Utility::makeBogusString();
             UnicodeString offset = ICU_Utility::makeBogusString();
+            UnicodeString special = ICU_Utility::makeBogusString();
+            UnicodeString systems = ICU_Utility::makeBogusString();
             for (int32_t i = 0; unitTable.getKeyAndValue(i, key, value); i++) {
                 if (uprv_strcmp(key, "target") == 0) {
                     baseUnit = value.getUnicodeString(status);
@@ -85,10 +90,14 @@ class ConversionRateDataSink : public ResourceSink {
                     factor = value.getUnicodeString(status);
                 } else if (uprv_strcmp(key, "offset") == 0) {
                     offset = value.getUnicodeString(status);
+                } else if (uprv_strcmp(key, "special") == 0) {
+                    special = value.getUnicodeString(status); // the name of a special mapping used instead of factor + optional offset.
+                } else if (uprv_strcmp(key, "systems") == 0) {
+                    systems = value.getUnicodeString(status);
                 }
             }
             if (U_FAILURE(status)) { return; }
-            if (baseUnit.isBogus() || factor.isBogus()) {
+            if (baseUnit.isBogus() || (factor.isBogus() && special.isBogus())) {
                 // We could not find a usable conversion rate: bad resource.
                 status = U_MISSING_RESOURCE_ERROR;
                 return;
@@ -102,12 +111,15 @@ class ConversionRateDataSink : public ResourceSink {
             } else {
                 cr->sourceUnit.append(srcUnit, status);
                 cr->baseUnit.appendInvariantChars(baseUnit, status);
-                cr->factor.appendInvariantChars(factor, status);
-                trimSpaces(cr->factor, status);
+                if (!factor.isBogus()) {
+                    cr->factor.appendInvariantChars(factor, status);
+                    trimSpaces(cr->factor, status);
+                }
                 if (!offset.isBogus()) cr->offset.appendInvariantChars(offset, status);
+                if (!special.isBogus()) cr->specialMappingName.appendInvariantChars(special, status);
+                cr->systems.appendInvariantChars(systems, status);
             }
         }
-        return;
     }
 
   private:
@@ -205,11 +217,11 @@ class UnitPreferencesSink : public ResourceSink {
                         for (int32_t i = 0; unitPref.getKeyAndValue(i, key, value); ++i) {
                             if (uprv_strcmp(key, "unit") == 0) {
                                 int32_t length;
-                                const UChar *u = value.getString(length, status);
+                                const char16_t *u = value.getString(length, status);
                                 up->unit.appendInvariantChars(u, length, status);
                             } else if (uprv_strcmp(key, "geq") == 0) {
                                 int32_t length;
-                                const UChar *g = value.getString(length, status);
+                                const char16_t *g = value.getString(length, status);
                                 CharString geq;
                                 geq.appendInvariantChars(g, length, status);
                                 DecimalQuantity dq;
@@ -368,7 +380,7 @@ int32_t UnitPreferenceMetadata::compareTo(const UnitPreferenceMetadata &other, b
 
 // TODO: this may be unnecessary. Fold into ConversionRates class? Or move to anonymous namespace?
 void U_I18N_API getAllConversionRates(MaybeStackVector<ConversionRateInfo> &result, UErrorCode &status) {
-    LocalUResourceBundlePointer unitsBundle(ures_openDirect(NULL, "units", &status));
+    LocalUResourceBundlePointer unitsBundle(ures_openDirect(nullptr, "units", &status));
     ConversionRateDataSink sink(&result);
     ures_getAllItemsWithFallback(unitsBundle.getAlias(), "convertUnits", sink, status);
 }
@@ -376,7 +388,7 @@ void U_I18N_API getAllConversionRates(MaybeStackVector<ConversionRateInfo> &resu
 const ConversionRateInfo *ConversionRates::extractConversionInfo(StringPiece source,
                                                                  UErrorCode &status) const {
     for (size_t i = 0, n = conversionInfo_.length(); i < n; ++i) {
-        if (conversionInfo_[i]->sourceUnit.toStringPiece() == source) return conversionInfo_[i];
+        if (conversionInfo_[i]->sourceUnit == source) return conversionInfo_[i];
     }
 
     status = U_INTERNAL_PROGRAM_ERROR;
@@ -384,18 +396,14 @@ const ConversionRateInfo *ConversionRates::extractConversionInfo(StringPiece sou
 }
 
 U_I18N_API UnitPreferences::UnitPreferences(UErrorCode &status) {
-    LocalUResourceBundlePointer unitsBundle(ures_openDirect(NULL, "units", &status));
+    LocalUResourceBundlePointer unitsBundle(ures_openDirect(nullptr, "units", &status));
     UnitPreferencesSink sink(&unitPrefs_, &metadata_);
     ures_getAllItemsWithFallback(unitsBundle.getAlias(), "unitPreferenceData", sink, status);
 }
 
 CharString getKeyWordValue(const Locale &locale, StringPiece kw, UErrorCode &status) {
-    CharString result;
-    if (U_FAILURE(status)) { return result; }
-    {
-        CharStringByteSink sink(&result);
-        locale.getKeywordValue(kw, sink, status);
-    }
+    if (U_FAILURE(status)) { return {}; }
+    auto result = locale.getKeywordValue<CharString>(kw, status);
     if (U_SUCCESS(status) && result.isEmpty()) {
         status = U_MISSING_RESOURCE_ERROR;
     }
@@ -409,14 +417,19 @@ MaybeStackVector<UnitPreference>
     MaybeStackVector<UnitPreference> result;
 
     // TODO: remove this once all the categories are allowed.
+    // WARNING: when this is removed please make sure to keep the "fahrenhe" => "fahrenheit" mapping
     UErrorCode internalMuStatus = U_ZERO_ERROR;
     if (category.compare("temperature") == 0) {
         CharString localeUnitCharString = getKeyWordValue(locale, "mu", internalMuStatus);
         if (U_SUCCESS(internalMuStatus)) {
+            // The value for -u-mu- is `fahrenhe`, but CLDR and everything else uses `fahrenheit`
+            if (localeUnitCharString == "fahrenhe") {
+                localeUnitCharString = CharString("fahrenheit", status);
+            }
             // TODO: use the unit category as Java especially when all the categories are allowed..
-            if (localeUnitCharString == "celsius"       //
-                || localeUnitCharString == "fahrenheit" //
-                || localeUnitCharString == "kelvin"     //
+            if (localeUnitCharString == "celsius"
+                || localeUnitCharString == "fahrenheit"
+                || localeUnitCharString == "kelvin"
             ) {
                 UnitPreference unitPref;
                 unitPref.unit.append(localeUnitCharString, status);
@@ -426,46 +439,14 @@ MaybeStackVector<UnitPreference>
         }
     }
 
-    CharString region(locale.getCountry(), status);
+    CharString region = ulocimp_getRegionForSupplementalData(locale.getName(), true, status);
 
     // Check the locale system tag, e.g `ms=metric`.
     UErrorCode internalMeasureTagStatus = U_ZERO_ERROR;
     CharString localeSystem = getKeyWordValue(locale, "measure", internalMeasureTagStatus);
     bool isLocaleSystem = false;
-    if (U_SUCCESS(internalMeasureTagStatus)) {
-        if (localeSystem == "metric") {
-            region.clear();
-            region.append("001", status);
-            isLocaleSystem = true;
-        } else if (localeSystem == "ussystem") {
-            region.clear();
-            region.append("US", status);
-            isLocaleSystem = true;
-        } else if (localeSystem == "uksystem") {
-            region.clear();
-            region.append("GB", status);
-            isLocaleSystem = true;
-        }
-    }
-
-    // Check the region tag, e.g. `rg=uszzz`.
-    if (!isLocaleSystem) {
-        UErrorCode internalRgTagStatus = U_ZERO_ERROR;
-        CharString localeRegion = getKeyWordValue(locale, "rg", internalRgTagStatus);
-        if (U_SUCCESS(internalRgTagStatus) && localeRegion.length() >= 3) {
-            if (localeRegion == "default") {
-                region.clear();
-                region.append(localeRegion, status);
-            } else if (localeRegion[0] >= '0' && localeRegion[0] <= '9') {
-                region.clear();
-                region.append(localeRegion.data(), 3, status);
-            } else {
-                // Take the first two character and capitalize them.
-                region.clear();
-                region.append(uprv_toupper(localeRegion[0]), status);
-                region.append(uprv_toupper(localeRegion[1]), status);
-            }
-        }
+    if (U_SUCCESS(internalMeasureTagStatus) && (localeSystem == "metric" || localeSystem == "ussystem" || localeSystem == "uksystem")) {
+        isLocaleSystem = true;
     }
 
     int32_t idx =
@@ -476,6 +457,48 @@ MaybeStackVector<UnitPreference>
 
     U_ASSERT(idx >= 0); // Failures should have been taken care of by `status`.
     const UnitPreferenceMetadata *m = metadata_[idx];
+        
+    if (isLocaleSystem) {
+        // if the locale ID specifies a measurment system, check if ALL of the units we got back
+        // are members of that system (or are "metric_adjacent", which we consider to match all
+        // the systems)
+        bool unitsMatchSystem = true;
+        ConversionRates rates(status);
+        for (int32_t i = 0; unitsMatchSystem && i < m->prefsCount; i++) {
+            const UnitPreference& unitPref = *(unitPrefs_[i + m->prefsOffset]);
+            MeasureUnitImpl measureUnit = MeasureUnitImpl::forIdentifier(unitPref.unit.data(), status);
+            for (int32_t j = 0; unitsMatchSystem && j < measureUnit.singleUnits.length(); j++) {
+                const SingleUnitImpl* singleUnit = measureUnit.singleUnits[j];
+                const ConversionRateInfo* rateInfo = rates.extractConversionInfo(singleUnit->getSimpleUnitID(), status);
+                CharString systems(rateInfo->systems, status);
+                if (!systems.contains("metric_adjacent")) { // "metric-adjacent" is considered to match all the locale systems
+                    if (!systems.contains(localeSystem.data())) {
+                        unitsMatchSystem = false;
+                    }
+                }
+            }
+        }
+        
+        // if any of the units we got back above don't match the mearurement system the locale ID asked for,
+        // throw out the region and just load the units for the base region for the requested measurement system
+        if (!unitsMatchSystem) {
+            region.clear();
+            if (localeSystem == "ussystem") {
+                region.append("US", status);
+            } else if (localeSystem == "uksystem") {
+                region.append("GB", status);
+            } else {
+                region.append("001", status);
+            }
+            idx = getPreferenceMetadataIndex(&metadata_, category, usage, region.toStringPiece(), status);
+            if (U_FAILURE(status)) {
+                return result;
+            }
+            
+            m = metadata_[idx];
+        }
+    }
+        
     for (int32_t i = 0; i < m->prefsCount; i++) {
         result.emplaceBackAndCheckErrorCode(status, *(unitPrefs_[i + m->prefsOffset]));
     }
