@@ -25,41 +25,41 @@
 U_NAMESPACE_BEGIN
 
 // Smybol characters used by RFC2445 VTIMEZONE
-static const UChar COLON = 0x3A; /* : */
-static const UChar SEMICOLON = 0x3B; /* ; */
-static const UChar EQUALS_SIGN = 0x3D; /* = */
-static const UChar COMMA = 0x2C; /* , */
-static const UChar PLUS = 0x2B; /* + */
-static const UChar MINUS = 0x2D; /* - */
+static const char16_t COLON = 0x3A; /* : */
+static const char16_t SEMICOLON = 0x3B; /* ; */
+static const char16_t EQUALS_SIGN = 0x3D; /* = */
+static const char16_t COMMA = 0x2C; /* , */
+static const char16_t PLUS = 0x2B; /* + */
+static const char16_t MINUS = 0x2D; /* - */
 
 // RFC2445 VTIMEZONE tokens
-static const UChar ICAL_BEGIN_VTIMEZONE[] = {0x42, 0x45, 0x47, 0x49, 0x4E, 0x3A, 0x56, 0x54, 0x49, 0x4D, 0x45, 0x5A, 0x4F, 0x4E, 0x45, 0}; /* "BEGIN:VTIMEZONE" */
-static const UChar ICAL_END_VTIMEZONE[] = {0x45, 0x4E, 0x44, 0x3A, 0x56, 0x54, 0x49, 0x4D, 0x45, 0x5A, 0x4F, 0x4E, 0x45, 0}; /* "END:VTIMEZONE" */
-static const UChar ICAL_BEGIN[] = {0x42, 0x45, 0x47, 0x49, 0x4E, 0}; /* "BEGIN" */
-static const UChar ICAL_END[] = {0x45, 0x4E, 0x44, 0}; /* "END" */
-static const UChar ICAL_VTIMEZONE[] = {0x56, 0x54, 0x49, 0x4D, 0x45, 0x5A, 0x4F, 0x4E, 0x45, 0}; /* "VTIMEZONE" */
-static const UChar ICAL_TZID[] = {0x54, 0x5A, 0x49, 0x44, 0}; /* "TZID" */
-static const UChar ICAL_STANDARD[] = {0x53, 0x54, 0x41, 0x4E, 0x44, 0x41, 0x52, 0x44, 0}; /* "STANDARD" */
-static const UChar ICAL_DAYLIGHT[] = {0x44, 0x41, 0x59, 0x4C, 0x49, 0x47, 0x48, 0x54, 0}; /* "DAYLIGHT" */
-static const UChar ICAL_DTSTART[] = {0x44, 0x54, 0x53, 0x54, 0x41, 0x52, 0x54, 0}; /* "DTSTART" */
-static const UChar ICAL_TZOFFSETFROM[] = {0x54, 0x5A, 0x4F, 0x46, 0x46, 0x53, 0x45, 0x54, 0x46, 0x52, 0x4F, 0x4D, 0}; /* "TZOFFSETFROM" */
-static const UChar ICAL_TZOFFSETTO[] = {0x54, 0x5A, 0x4F, 0x46, 0x46, 0x53, 0x45, 0x54, 0x54, 0x4F, 0}; /* "TZOFFSETTO" */
-static const UChar ICAL_RDATE[] = {0x52, 0x44, 0x41, 0x54, 0x45, 0}; /* "RDATE" */
-static const UChar ICAL_RRULE[] = {0x52, 0x52, 0x55, 0x4C, 0x45, 0}; /* "RRULE" */
-static const UChar ICAL_TZNAME[] = {0x54, 0x5A, 0x4E, 0x41, 0x4D, 0x45, 0}; /* "TZNAME" */
-static const UChar ICAL_TZURL[] = {0x54, 0x5A, 0x55, 0x52, 0x4C, 0}; /* "TZURL" */
-static const UChar ICAL_LASTMOD[] = {0x4C, 0x41, 0x53, 0x54, 0x2D, 0x4D, 0x4F, 0x44, 0x49, 0x46, 0x49, 0x45, 0x44, 0}; /* "LAST-MODIFIED" */
+static const char16_t ICAL_BEGIN_VTIMEZONE[] = {0x42, 0x45, 0x47, 0x49, 0x4E, 0x3A, 0x56, 0x54, 0x49, 0x4D, 0x45, 0x5A, 0x4F, 0x4E, 0x45, 0}; /* "BEGIN:VTIMEZONE" */
+static const char16_t ICAL_END_VTIMEZONE[] = {0x45, 0x4E, 0x44, 0x3A, 0x56, 0x54, 0x49, 0x4D, 0x45, 0x5A, 0x4F, 0x4E, 0x45, 0}; /* "END:VTIMEZONE" */
+static const char16_t ICAL_BEGIN[] = {0x42, 0x45, 0x47, 0x49, 0x4E, 0}; /* "BEGIN" */
+static const char16_t ICAL_END[] = {0x45, 0x4E, 0x44, 0}; /* "END" */
+static const char16_t ICAL_VTIMEZONE[] = {0x56, 0x54, 0x49, 0x4D, 0x45, 0x5A, 0x4F, 0x4E, 0x45, 0}; /* "VTIMEZONE" */
+static const char16_t ICAL_TZID[] = {0x54, 0x5A, 0x49, 0x44, 0}; /* "TZID" */
+static const char16_t ICAL_STANDARD[] = {0x53, 0x54, 0x41, 0x4E, 0x44, 0x41, 0x52, 0x44, 0}; /* "STANDARD" */
+static const char16_t ICAL_DAYLIGHT[] = {0x44, 0x41, 0x59, 0x4C, 0x49, 0x47, 0x48, 0x54, 0}; /* "DAYLIGHT" */
+static const char16_t ICAL_DTSTART[] = {0x44, 0x54, 0x53, 0x54, 0x41, 0x52, 0x54, 0}; /* "DTSTART" */
+static const char16_t ICAL_TZOFFSETFROM[] = {0x54, 0x5A, 0x4F, 0x46, 0x46, 0x53, 0x45, 0x54, 0x46, 0x52, 0x4F, 0x4D, 0}; /* "TZOFFSETFROM" */
+static const char16_t ICAL_TZOFFSETTO[] = {0x54, 0x5A, 0x4F, 0x46, 0x46, 0x53, 0x45, 0x54, 0x54, 0x4F, 0}; /* "TZOFFSETTO" */
+static const char16_t ICAL_RDATE[] = {0x52, 0x44, 0x41, 0x54, 0x45, 0}; /* "RDATE" */
+static const char16_t ICAL_RRULE[] = {0x52, 0x52, 0x55, 0x4C, 0x45, 0}; /* "RRULE" */
+static const char16_t ICAL_TZNAME[] = {0x54, 0x5A, 0x4E, 0x41, 0x4D, 0x45, 0}; /* "TZNAME" */
+static const char16_t ICAL_TZURL[] = {0x54, 0x5A, 0x55, 0x52, 0x4C, 0}; /* "TZURL" */
+static const char16_t ICAL_LASTMOD[] = {0x4C, 0x41, 0x53, 0x54, 0x2D, 0x4D, 0x4F, 0x44, 0x49, 0x46, 0x49, 0x45, 0x44, 0}; /* "LAST-MODIFIED" */
 
-static const UChar ICAL_FREQ[] = {0x46, 0x52, 0x45, 0x51, 0}; /* "FREQ" */
-static const UChar ICAL_UNTIL[] = {0x55, 0x4E, 0x54, 0x49, 0x4C, 0}; /* "UNTIL" */
-static const UChar ICAL_YEARLY[] = {0x59, 0x45, 0x41, 0x52, 0x4C, 0x59, 0}; /* "YEARLY" */
-static const UChar ICAL_BYMONTH[] = {0x42, 0x59, 0x4D, 0x4F, 0x4E, 0x54, 0x48, 0}; /* "BYMONTH" */
-static const UChar ICAL_BYDAY[] = {0x42, 0x59, 0x44, 0x41, 0x59, 0}; /* "BYDAY" */
-static const UChar ICAL_BYMONTHDAY[] = {0x42, 0x59, 0x4D, 0x4F, 0x4E, 0x54, 0x48, 0x44, 0x41, 0x59, 0}; /* "BYMONTHDAY" */
+static const char16_t ICAL_FREQ[] = {0x46, 0x52, 0x45, 0x51, 0}; /* "FREQ" */
+static const char16_t ICAL_UNTIL[] = {0x55, 0x4E, 0x54, 0x49, 0x4C, 0}; /* "UNTIL" */
+static const char16_t ICAL_YEARLY[] = {0x59, 0x45, 0x41, 0x52, 0x4C, 0x59, 0}; /* "YEARLY" */
+static const char16_t ICAL_BYMONTH[] = {0x42, 0x59, 0x4D, 0x4F, 0x4E, 0x54, 0x48, 0}; /* "BYMONTH" */
+static const char16_t ICAL_BYDAY[] = {0x42, 0x59, 0x44, 0x41, 0x59, 0}; /* "BYDAY" */
+static const char16_t ICAL_BYMONTHDAY[] = {0x42, 0x59, 0x4D, 0x4F, 0x4E, 0x54, 0x48, 0x44, 0x41, 0x59, 0}; /* "BYMONTHDAY" */
 
-static const UChar ICAL_NEWLINE[] = {0x0D, 0x0A, 0}; /* CRLF */
+static const char16_t ICAL_NEWLINE[] = {0x0D, 0x0A, 0}; /* CRLF */
 
-static const UChar ICAL_DOW_NAMES[7][3] = {
+static const char16_t ICAL_DOW_NAMES[7][3] = {
     {0x53, 0x55, 0}, /* "SU" */
     {0x4D, 0x4F, 0}, /* "MO" */
     {0x54, 0x55, 0}, /* "TU" */
@@ -72,9 +72,9 @@ static const UChar ICAL_DOW_NAMES[7][3] = {
 static const int32_t MONTHLENGTH[] = {31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31};
 
 // ICU custom property
-static const UChar ICU_TZINFO_PROP[] = {0x58, 0x2D, 0x54, 0x5A, 0x49, 0x4E, 0x46, 0x4F, 0x3A, 0}; /* "X-TZINFO:" */
-static const UChar ICU_TZINFO_PARTIAL[] = {0x2F, 0x50, 0x61, 0x72, 0x74, 0x69, 0x61, 0x6C, 0x40, 0}; /* "/Partial@" */
-static const UChar ICU_TZINFO_SIMPLE[] = {0x2F, 0x53, 0x69, 0x6D, 0x70, 0x6C, 0x65, 0x40, 0}; /* "/Simple@" */
+static const char16_t ICU_TZINFO_PROP[] = {0x58, 0x2D, 0x54, 0x5A, 0x49, 0x4E, 0x46, 0x4F, 0x3A, 0}; /* "X-TZINFO:" */
+static const char16_t ICU_TZINFO_PARTIAL[] = {0x2F, 0x50, 0x61, 0x72, 0x74, 0x69, 0x61, 0x6C, 0x40, 0}; /* "/Partial@" */
+static const char16_t ICU_TZINFO_SIMPLE[] = {0x2F, 0x53, 0x69, 0x6D, 0x70, 0x6C, 0x65, 0x40, 0}; /* "/Simple@" */
 
 
 /*
@@ -139,7 +139,7 @@ static UnicodeString& appendAsciiDigits(int32_t number, uint8_t length, UnicodeS
         str.append(MINUS);
     }
     for (i = length - 1; i >= 0; i--) {
-        str.append((UChar)(digits[i] + 0x0030));
+        str.append(static_cast<char16_t>(digits[i] + 0x0030));
     }
     return str;
 }
@@ -151,11 +151,11 @@ static UnicodeString& appendMillis(UDate date, UnicodeString& str) {
     int64_t number;
 
     if (date < MIN_MILLIS) {
-        number = (int64_t)MIN_MILLIS;
+        number = static_cast<int64_t>(MIN_MILLIS);
     } else if (date > MAX_MILLIS) {
-        number = (int64_t)MAX_MILLIS;
+        number = static_cast<int64_t>(MAX_MILLIS);
     } else {
-        number = (int64_t)date;
+        number = static_cast<int64_t>(date);
     }
     if (number < 0) {
         negative = true;
@@ -163,7 +163,7 @@ static UnicodeString& appendMillis(UDate date, UnicodeString& str) {
     }
     i = 0;
     do {
-        digits[i++] = (int32_t)(number % 10);
+        digits[i++] = static_cast<int32_t>(number % 10);
         number /= 10;
     } while (number != 0);
 
@@ -172,7 +172,7 @@ static UnicodeString& appendMillis(UDate date, UnicodeString& str) {
     }
     i--;
     while (i >= 0) {
-        str.append((UChar)(digits[i--] + 0x0030));
+        str.append(static_cast<char16_t>(digits[i--] + 0x0030));
     }
     return str;
 }
@@ -180,15 +180,17 @@ static UnicodeString& appendMillis(UDate date, UnicodeString& str) {
 /*
  * Convert date/time to RFC2445 Date-Time form #1 DATE WITH LOCAL TIME
  */
-static UnicodeString& getDateTimeString(UDate time, UnicodeString& str) {
+static UnicodeString& getDateTimeString(UDate time, UnicodeString& str, UErrorCode& status) {
+    if (U_FAILURE(status)) {return str;}
     int32_t year, month, dom, dow, doy, mid;
-    Grego::timeToFields(time, year, month, dom, dow, doy, mid);
+    Grego::timeToFields(time, year, month, dom, dow, doy, mid, status);
+    if (U_FAILURE(status)) {return str;}
 
     str.remove();
     appendAsciiDigits(year, 4, str);
     appendAsciiDigits(month + 1, 2, str);
     appendAsciiDigits(dom, 2, str);
-    str.append((UChar)0x0054 /*'T'*/);
+    str.append(static_cast<char16_t>(0x0054) /*'T'*/);
 
     int32_t t = mid;
     int32_t hour = t / U_MILLIS_PER_HOUR;
@@ -206,9 +208,9 @@ static UnicodeString& getDateTimeString(UDate time, UnicodeString& str) {
 /*
  * Convert date/time to RFC2445 Date-Time form #2 DATE WITH UTC TIME
  */
-static UnicodeString& getUTCDateTimeString(UDate time, UnicodeString& str) {
-    getDateTimeString(time, str);
-    str.append((UChar)0x005A /*'Z'*/);
+static UnicodeString& getUTCDateTimeString(UDate time, UnicodeString& str, UErrorCode& status) {
+    getDateTimeString(time, str, status);
+    str.append(static_cast<char16_t>(0x005A) /*'Z'*/);
     return str;
 }
 
@@ -295,7 +297,7 @@ static int32_t offsetStrToMillis(const UnicodeString& str, UErrorCode& status) {
             break;
         }
         // sign
-        UChar s = str.charAt(0);
+        char16_t s = str.charAt(0);
         if (s == PLUS) {
             sign = 1;
         } else if (s == MINUS) {
@@ -529,7 +531,7 @@ static TimeZoneRule* createRuleByRRULE(const UnicodeString& zonename, int rawOff
     DateTimeRule *adtr = nullptr;
 
     // Parse the first rule
-    UnicodeString rrule = *((UnicodeString*)dates->elementAt(0));
+    UnicodeString rrule = *static_cast<UnicodeString*>(dates->elementAt(0));
     int32_t month, dayOfWeek, nthDayOfWeek, dayOfMonth = 0;
     int32_t days[7];
     int32_t daysCount = UPRV_LENGTHOF(days);
@@ -605,7 +607,7 @@ static TimeZoneRule* createRuleByRRULE(const UnicodeString& zonename, int rawOff
 
         int32_t anotherMonth = -1;
         for (i = 1; i < dates->size(); i++) {
-            rrule = *((UnicodeString*)dates->elementAt(i));
+            rrule = *static_cast<UnicodeString*>(dates->elementAt(i));
             UDate tmp_until;
             int32_t tmp_month, tmp_dayOfWeek, tmp_nthDayOfWeek;
             int32_t tmp_days[7];
@@ -675,7 +677,10 @@ static TimeZoneRule* createRuleByRRULE(const UnicodeString& zonename, int rawOff
     // Calculate start/end year and missing fields
     int32_t startYear, startMonth, startDOM, startDOW, startDOY, startMID;
     Grego::timeToFields(start + fromOffset, startYear, startMonth, startDOM,
-        startDOW, startDOY, startMID);
+        startDOW, startDOY, startMID, status);
+    if (U_FAILURE(status)) {
+        return nullptr;
+    }
     if (month == -1) {
         // If BYMONTH is not set, use the month of DTSTART
         month = startMonth;
@@ -688,7 +693,8 @@ static TimeZoneRule* createRuleByRRULE(const UnicodeString& zonename, int rawOff
     int32_t endYear;
     if (until != MIN_MILLIS) {
         int32_t endMonth, endDOM, endDOW, endDOY, endMID;
-        Grego::timeToFields(until, endYear, endMonth, endDOM, endDOW, endDOY, endMID);
+        Grego::timeToFields(until, endYear, endMonth, endDOM, endDOW, endDOY, endMID, status);
+        if (U_FAILURE(status)) return nullptr;
     } else {
         endYear = AnnualTimeZoneRule::MAX_YEAR;
     }
@@ -731,13 +737,13 @@ static TimeZoneRule* createRuleByRDATE(const UnicodeString& zonename, int32_t ra
     } else {
         // Create an array of transition times
         int32_t size = dates->size();
-        UDate* times = (UDate*)uprv_malloc(sizeof(UDate) * size);
+        UDate* times = static_cast<UDate*>(uprv_malloc(sizeof(UDate) * size));
         if (times == nullptr) {
             status = U_MEMORY_ALLOCATION_ERROR;
             return nullptr;
         }
         for (int32_t i = 0; i < size; i++) {
-            UnicodeString *datestr = (UnicodeString*)dates->elementAt(i);
+            UnicodeString* datestr = static_cast<UnicodeString*>(dates->elementAt(i));
             times[i] = parseDateTimeString(*datestr, fromOffset, status);
             if (U_FAILURE(status)) {
                 uprv_free(times);
@@ -885,9 +891,9 @@ public:
     ~VTZWriter();
 
     void write(const UnicodeString& str);
-    void write(UChar ch);
-    void write(const UChar* str);
-    //void write(const UChar* str, int32_t length);
+    void write(char16_t ch);
+    void write(const char16_t* str);
+    //void write(const char16_t* str, int32_t length);
 private:
     UnicodeString* out;
 };
@@ -905,18 +911,18 @@ VTZWriter::write(const UnicodeString& str) {
 }
 
 void
-VTZWriter::write(UChar ch) {
+VTZWriter::write(char16_t ch) {
     out->append(ch);
 }
 
 void
-VTZWriter::write(const UChar* str) {
+VTZWriter::write(const char16_t* str) {
     out->append(str, -1);
 }
 
 /*
 void
-VTZWriter::write(const UChar* str, int32_t length) {
+VTZWriter::write(const char16_t* str, int32_t length) {
     out->append(str, length);
 }
 */
@@ -926,7 +932,7 @@ public:
     VTZReader(const UnicodeString& input);
     ~VTZReader();
 
-    UChar read(void);
+    char16_t read();
 private:
     const UnicodeString* in;
     int32_t index;
@@ -940,9 +946,9 @@ VTZReader::VTZReader(const UnicodeString& input) {
 VTZReader::~VTZReader() {
 }
 
-UChar
-VTZReader::read(void) {
-    UChar ch = 0xFFFF;
+char16_t
+VTZReader::read() {
+    char16_t ch = 0xFFFF;
     if (index < in->length()) {
         ch = in->charAt(index);
     }
@@ -974,7 +980,7 @@ VTimeZone::VTimeZone(const VTimeZone& source)
             return;
         }
         for (int32_t i = 0; i < size; i++) {
-            UnicodeString *line = ((UnicodeString*)source.vtzlines->elementAt(i))->clone();
+            UnicodeString* line = static_cast<UnicodeString*>(source.vtzlines->elementAt(i))->clone();
             lpVtzLines->adoptElement(line, status);
             if (U_FAILURE(status) || line == nullptr) {
                 return;
@@ -985,12 +991,8 @@ VTimeZone::VTimeZone(const VTimeZone& source)
 }
 
 VTimeZone::~VTimeZone() {
-    if (tz != nullptr) {
-        delete tz;
-    }
-    if (vtzlines != nullptr) {
-        delete vtzlines;
-    }
+    delete tz;
+    delete vtzlines;
 }
 
 VTimeZone&
@@ -1019,7 +1021,7 @@ VTimeZone::operator=(const VTimeZone& right) {
             if (U_SUCCESS(status)) {
                 for (int32_t i = 0; i < size; i++) {
                     LocalPointer<UnicodeString> line(
-                        ((UnicodeString*)right.vtzlines->elementAt(i))->clone(), status);
+                        static_cast<UnicodeString*>(right.vtzlines->elementAt(i))->clone(), status);
                     lpVtzLines->adoptElement(line.orphan(), status);
                     if (U_FAILURE(status)) {
                         break;
@@ -1074,7 +1076,7 @@ VTimeZone::createVTimeZoneByID(const UnicodeString& ID) {
     // Set ICU tzdata version
     UErrorCode status = U_ZERO_ERROR;
     UResourceBundle *bundle = nullptr;
-    const UChar* versionStr = nullptr;
+    const char16_t* versionStr = nullptr;
     int32_t len = 0;
     bundle = ures_openDirect(nullptr, "zoneinfo64", &status);
     versionStr = ures_getStringByKey(bundle, "TZVersion", &len, &status);
@@ -1105,7 +1107,7 @@ VTimeZone::createVTimeZoneFromBasicTimeZone(const BasicTimeZone& basic_time_zone
 
     // Set ICU tzdata version
     UResourceBundle *bundle = nullptr;
-    const UChar* versionStr = nullptr;
+    const char16_t* versionStr = nullptr;
     int32_t len = 0;
     bundle = ures_openDirect(nullptr, "zoneinfo64", &status);
     versionStr = ures_getStringByKey(bundle, "TZVersion", &len, &status);
@@ -1220,12 +1222,12 @@ VTimeZone::setRawOffset(int32_t offsetMillis) {
 }
 
 int32_t
-VTimeZone::getRawOffset(void) const {
+VTimeZone::getRawOffset() const {
     return tz->getRawOffset();
 }
 
 UBool
-VTimeZone::useDaylightTime(void) const {
+VTimeZone::useDaylightTime() const {
     return tz->useDaylightTime();
 }
 
@@ -1275,7 +1277,7 @@ VTimeZone::load(VTZReader& reader, UErrorCode& status) {
     UnicodeString line;
 
     while (true) {
-        UChar ch = reader.read();
+        char16_t ch = reader.read();
         if (ch == 0xFFFF) {
             // end of file
             if (start && line.startsWith(ICAL_END_VTIMEZONE, -1)) {
@@ -1399,7 +1401,7 @@ VTimeZone::parse(UErrorCode& status) {
     }
     
     for (n = 0; n < vtzlines->size(); n++) {
-        UnicodeString *line = (UnicodeString*)vtzlines->elementAt(n);
+        UnicodeString* line = static_cast<UnicodeString*>(vtzlines->elementAt(n));
         int32_t valueSep = line->indexOf(COLON);
         if (valueSep < 0) {
             continue;
@@ -1610,7 +1612,7 @@ VTimeZone::parse(UErrorCode& status) {
     }
 
     for (n = 0; n < rules.size(); n++) {
-        TimeZoneRule *r = (TimeZoneRule*)rules.elementAt(n);
+        TimeZoneRule* r = static_cast<TimeZoneRule*>(rules.elementAt(n));
         AnnualTimeZoneRule *atzrule = dynamic_cast<AnnualTimeZoneRule *>(r);
         if (atzrule != nullptr) {
             if (atzrule->getEndYear() == AnnualTimeZoneRule::MAX_YEAR) {
@@ -1633,7 +1635,7 @@ VTimeZone::parse(UErrorCode& status) {
             rules.removeAllElements();
         } else {
             // Normalize the final rule
-            AnnualTimeZoneRule *finalRule = (AnnualTimeZoneRule*)rules.elementAt(finalRuleIdx);
+            AnnualTimeZoneRule* finalRule = static_cast<AnnualTimeZoneRule*>(rules.elementAt(finalRuleIdx));
             int32_t tmpRaw = finalRule->getRawOffset();
             int32_t tmpDST = finalRule->getDSTSavings();
 
@@ -1645,7 +1647,7 @@ VTimeZone::parse(UErrorCode& status) {
                 if (finalRuleIdx == n) {
                     continue;
                 }
-                TimeZoneRule *r = (TimeZoneRule*)rules.elementAt(n);
+                TimeZoneRule* r = static_cast<TimeZoneRule*>(rules.elementAt(n));
                 UDate lastStart;
                 r->getFinalStart(tmpRaw, tmpDST, lastStart);
                 if (lastStart > start) {
@@ -1673,7 +1675,8 @@ VTimeZone::parse(UErrorCode& status) {
             } else {
                 // Update the end year
                 int32_t y, m, d, dow, doy, mid;
-                Grego::timeToFields(start, y, m, d, dow, doy, mid);
+                Grego::timeToFields(start, y, m, d, dow, doy, mid, status);
+                if (U_FAILURE(status)) return;
                 newRule.adoptInsteadAndCheckErrorCode(
                     new AnnualTimeZoneRule(
                             finalRule->getName(tznam),
@@ -1696,7 +1699,7 @@ VTimeZone::parse(UErrorCode& status) {
     }
 
     while (!rules.isEmpty()) {
-        TimeZoneRule *tzr = (TimeZoneRule*)rules.orphanElementAt(0);
+        TimeZoneRule* tzr = static_cast<TimeZoneRule*>(rules.orphanElementAt(0));
         rbtz->addTransitionRule(tzr, status);
         if (U_FAILURE(status)) {
             return;
@@ -1713,9 +1716,10 @@ VTimeZone::parse(UErrorCode& status) {
 
 void
 VTimeZone::write(VTZWriter& writer, UErrorCode& status) const {
+    if (U_FAILURE(status)) return;
     if (vtzlines != nullptr) {
         for (int32_t i = 0; i < vtzlines->size(); i++) {
-            UnicodeString *line = (UnicodeString*)vtzlines->elementAt(i);
+            UnicodeString* line = static_cast<UnicodeString*>(vtzlines->elementAt(i));
             if (line->startsWith(ICAL_TZURL, -1)
                 && line->charAt(u_strlen(ICAL_TZURL)) == COLON) {
                 writer.write(ICAL_TZURL);
@@ -1727,7 +1731,8 @@ VTimeZone::write(VTZWriter& writer, UErrorCode& status) const {
                 UnicodeString utcString;
                 writer.write(ICAL_LASTMOD);
                 writer.write(COLON);
-                writer.write(getUTCDateTimeString(lastmod, utcString));
+                writer.write(getUTCDateTimeString(lastmod, utcString, status));
+                if (U_FAILURE(status)) return;
                 writer.write(ICAL_NEWLINE);
             } else {
                 writer.write(*line);
@@ -1772,7 +1777,7 @@ VTimeZone::write(UDate start, VTZWriter& writer, UErrorCode& status) const {
     if (lpTransitionRules.isValid()) {
         U_ASSERT(transitionRules->hasDeleter());  // Assumed for U_FAILURE early return, below.
         while (!lpTransitionRules->isEmpty()) {
-            TimeZoneRule *tr = (TimeZoneRule*)lpTransitionRules->orphanElementAt(0);
+            TimeZoneRule* tr = static_cast<TimeZoneRule*>(lpTransitionRules->orphanElementAt(0));
             rbtz.addTransitionRule(tr, status);
             if (U_FAILURE(status)) {
                 return;
@@ -1791,11 +1796,11 @@ VTimeZone::write(UDate start, VTZWriter& writer, UErrorCode& status) const {
             return;
         }
         icutzprop->append(olsonzid);
-        icutzprop->append((UChar)0x005B/*'['*/);
+        icutzprop->append(static_cast<char16_t>(0x005B)/*'['*/);
         icutzprop->append(icutzver);
         icutzprop->append(ICU_TZINFO_PARTIAL, -1);
         appendMillis(start, *icutzprop);
-        icutzprop->append((UChar)0x005D/*']'*/);
+        icutzprop->append(static_cast<char16_t>(0x005D)/*']'*/);
         customProps.adoptElement(icutzprop, status);
         if (U_FAILURE(status)) {
             return;
@@ -1838,11 +1843,11 @@ VTimeZone::writeSimple(UDate time, VTZWriter& writer, UErrorCode& status) const
                return;
             }
             icutzprop->append(olsonzid);
-            icutzprop->append((UChar)0x005B/*'['*/);
+            icutzprop->append(static_cast<char16_t>(0x005B)/*'['*/);
             icutzprop->append(icutzver);
             icutzprop->append(ICU_TZINFO_SIMPLE, -1);
             appendMillis(time, *icutzprop);
-            icutzprop->append((UChar)0x005D/*']'*/);
+            icutzprop->append(static_cast<char16_t>(0x005D)/*']'*/);
             customProps.adoptElement(icutzprop.orphan(), status);
         }
         writeZone(writer, rbtz, &customProps, status);
@@ -1862,7 +1867,7 @@ VTimeZone::writeZone(VTZWriter& w, BasicTimeZone& basictz,
 
     if (customProps != nullptr) {
         for (int32_t i = 0; i < customProps->size(); i++) {
-            UnicodeString *custprop = (UnicodeString*)customProps->elementAt(i);
+            UnicodeString* custprop = static_cast<UnicodeString*>(customProps->elementAt(i));
             w.write(*custprop);
             w.write(ICAL_NEWLINE);
         }
@@ -1917,7 +1922,8 @@ VTimeZone::writeZone(VTZWriter& w, BasicTimeZone& basictz,
         int32_t fromOffset = tzt.getFrom()->getRawOffset() + tzt.getFrom()->getDSTSavings();
         int32_t fromDSTSavings = tzt.getFrom()->getDSTSavings();
         int32_t toOffset = tzt.getTo()->getRawOffset() + tzt.getTo()->getDSTSavings();
-        Grego::timeToFields(tzt.getTime() + fromOffset, year, month, dom, dow, doy, mid);
+        Grego::timeToFields(tzt.getTime() + fromOffset, year, month, dom, dow, doy, mid, status);
+        if (U_FAILURE(status)) return;
         int32_t weekInMonth = Grego::dayOfWeekInMonth(year, month, dom);
         UBool sameRule = false;
         const AnnualTimeZoneRule *atzrule;
@@ -2132,12 +2138,8 @@ VTimeZone::writeZone(VTZWriter& w, BasicTimeZone& basictz,
 
 cleanupWriteZone:
 
-    if (finalStdRule != nullptr) {
-        delete finalStdRule;
-    }
-    if (finalDstRule != nullptr) {
-        delete finalDstRule;
-    }
+    delete finalStdRule;
+    delete finalDstRule;
 }
 
 void
@@ -2166,7 +2168,7 @@ VTimeZone::writeHeaders(VTZWriter& writer, UErrorCode& status) const {
         UnicodeString lastmodStr;
         writer.write(ICAL_LASTMOD);
         writer.write(COLON);
-        writer.write(getUTCDateTimeString(lastmod, lastmodStr));
+        writer.write(getUTCDateTimeString(lastmod, lastmodStr, status));
         writer.write(ICAL_NEWLINE);
     }
 }
@@ -2203,8 +2205,11 @@ VTimeZone::writeZonePropsByTime(VTZWriter& writer, UBool isDst, const UnicodeStr
         writer.write(ICAL_RDATE);
         writer.write(COLON);
         UnicodeString timestr;
-        writer.write(getDateTimeString(time + fromOffset, timestr));
+        writer.write(getDateTimeString(time + fromOffset, timestr, status));
         writer.write(ICAL_NEWLINE);
+        if (U_FAILURE(status)) {
+            return;
+        }
     }
     endZoneProps(writer, isDst, status);
     if (U_FAILURE(status)) {
@@ -2237,7 +2242,7 @@ VTimeZone::writeZonePropsByDOM(VTZWriter& writer, UBool isDst, const UnicodeStri
     appendAsciiDigits(dayOfMonth, 0, dstr);
     writer.write(dstr);
     if (untilTime != MAX_MILLIS) {
-        appendUNTIL(writer, getDateTimeString(untilTime + fromOffset, dstr), status);
+        appendUNTIL(writer, getDateTimeString(untilTime + fromOffset, dstr, status), status);
         if (U_FAILURE(status)) {
             return;
         }
@@ -2273,7 +2278,7 @@ VTimeZone::writeZonePropsByDOW(VTZWriter& writer, UBool isDst, const UnicodeStri
     writer.write(ICAL_DOW_NAMES[dayOfWeek - 1]);    // SU, MO, TU...
 
     if (untilTime != MAX_MILLIS) {
-        appendUNTIL(writer, getDateTimeString(untilTime + fromOffset, dstr), status);
+        appendUNTIL(writer, getDateTimeString(untilTime + fromOffset, dstr, status), status);
         if (U_FAILURE(status)) {
             return;
         }
@@ -2398,7 +2403,7 @@ VTimeZone::writeZonePropsByDOW_GEQ_DOM_sub(VTZWriter& writer, int32_t month, int
     }
 
     if (untilTime != MAX_MILLIS) {
-        appendUNTIL(writer, getDateTimeString(untilTime + fromOffset, dstr), status);
+        appendUNTIL(writer, getDateTimeString(untilTime + fromOffset, dstr, status), status);
         if (U_FAILURE(status)) {
             return;
         }
@@ -2537,7 +2542,10 @@ VTimeZone::beginZoneProps(VTZWriter& writer, UBool isDst, const UnicodeString& z
     // DTSTART
     writer.write(ICAL_DTSTART);
     writer.write(COLON);
-    writer.write(getDateTimeString(startTime + fromOffset, dstr));
+    writer.write(getDateTimeString(startTime + fromOffset, dstr, status));
+    if (U_FAILURE(status)) {
+        return;
+    }
     writer.write(ICAL_NEWLINE);        
 }
 
