@@ -48,7 +48,7 @@ const char gHelpString[] =
  *    on the command line by the user.
  */
 char const *opt_locale      = "en_US";
-char * opt_rules       = 0;
+char *opt_rules        = nullptr;
 UBool  opt_help        = false;
 UBool  opt_norm        = false;
 UBool  opt_french      = false;
@@ -61,8 +61,8 @@ UBool  opt_canonical   = false;
 int    opt_level       = 0;
 char const *opt_source      = "International Components for Unicode";
 char const *opt_pattern     = "Unicode";
-UCollator * collator   = 0;
-UStringSearch * search = 0;
+UCollator *collator    = nullptr;
+UStringSearch *search  = nullptr;
 char16_t rules[100];
 char16_t source[100];
 char16_t pattern[100];
@@ -92,7 +92,7 @@ OptSpec opts[] = {
 	{"-canonical",   OptSpec::FLAG,   &opt_canonical},
     {"-help",        OptSpec::FLAG,   &opt_help},
     {"-?",           OptSpec::FLAG,   &opt_help},
-    {0, OptSpec::FLAG, 0}
+    {nullptr,        OptSpec::FLAG,   nullptr}
 };
 
 /**  
@@ -103,11 +103,11 @@ UBool processOptions(int argc, const char **argv, OptSpec opts[])
     for (int argNum = 1; argNum < argc; argNum ++) {
         const char *pArgName = argv[argNum];
         OptSpec *pOpt;
-        for (pOpt = opts;  pOpt->name != 0; pOpt ++) {
+        for (pOpt = opts; pOpt->name != nullptr; pOpt++) {
             if (strcmp(pOpt->name, pArgName) == 0) {
                 switch (pOpt->type) {
                 case OptSpec::FLAG:
-                    *(UBool *)(pOpt->pVar) = true;
+                    *static_cast<UBool*>(pOpt->pVar) = true;
                     break;
                 case OptSpec::STRING:
                     argNum ++;
@@ -116,7 +116,7 @@ UBool processOptions(int argc, const char **argv, OptSpec opts[])
 							    pOpt->name);
                         return false;
                     }
-                    *(const char **)(pOpt->pVar) = argv[argNum];
+                    *static_cast<const char**>(pOpt->pVar) = argv[argNum];
                     break;
                 case OptSpec::NUM:
                     argNum ++;
@@ -133,12 +133,12 @@ UBool processOptions(int argc, const char **argv, OptSpec opts[])
 								pOpt->name);
                         return false;
                     }
-                    *(int *)(pOpt->pVar) = i;
+                    *static_cast<int*>(pOpt->pVar) = i;
                 }
                 break;
             }
         }
-        if (pOpt->name == 0)
+        if (pOpt->name == nullptr)
         {
             fprintf(stderr, "Unrecognized option \"%s\"\n", pArgName);
             return false;
@@ -155,7 +155,7 @@ UBool processCollator()
 	// Set up an ICU collator
     UErrorCode status = U_ZERO_ERROR;
 
-    if (opt_rules != 0) {
+    if (opt_rules != nullptr) {
 		u_unescape(opt_rules, rules, 100);
         collator = ucol_openRules(rules, -1, UCOL_OFF, UCOL_TERTIARY, 
 			                  nullptr, &status);
