@@ -25,6 +25,8 @@
  **/
 static void TestBasicGetters(void);
 static void TestPrefixes(void);
+static void TestVariantLengthLimit(void);
+
 /**
  * Use Locale to access Resource file data and compare against expected values
  **/
@@ -33,6 +35,8 @@ static void TestSimpleResourceInfo(void);
  * Use Locale to access Resource file display names and compare against expected values
  **/
 static  void TestDisplayNames(void);
+static  void TestGetDisplayScriptPreFlighting21160(void);
+
 /**
  * Test getAvailableLocales
  **/
@@ -86,6 +90,7 @@ static void TestGetAvailableLocalesByType(void);
 
  static void TestCanonicalization(void);
  static void TestCanonicalizationBuffer(void);
+static  void TestCanonicalization21749StackUseAfterScope(void);
 
  static void TestDisplayKeywords(void);
 
@@ -98,7 +103,7 @@ static void TestTrailingNull(void);
 static void TestGetLocale(void);
 
 /**
- * additional intialization for datatables storing expected values
+ * additional initialization for datatables storing expected values
  */
 static void setUpDataTable(void);
 static void cleanUpDataTable(void);
@@ -121,6 +126,11 @@ static void TestOrientation(void);
 
 static void TestLikelySubtags(void);
 
+/**
+ * test terminate correctly.
+ */
+static void Test21157CorrectTerminating(void);
+
 /**
  * language tag
  */
@@ -134,6 +144,14 @@ static void TestToLegacyKey(void);
 static void TestToUnicodeLocaleType(void);
 static void TestToLegacyType(void);
 static void TestBug20149(void);
+static void TestCDefaultLocale(void);
+static void TestBug21449InfiniteLoop(void);
+
+
+/**
+ * U_USING_DEFAULT_WARNING
+ */
+static void TestUsingDefaultWarning(void);
 
 /**
  * locale data
