@@ -10,6 +10,8 @@
 
 #if !UCONFIG_NO_FORMATTING && !UCONFIG_NO_BREAK_ITERATION
 
+#include <stdbool.h>
+
 #include "unicode/ureldatefmt.h"
 #include "unicode/unum.h"
 #include "unicode/udisplaycontext.h"
@@ -197,13 +199,13 @@ static const FieldsDat en_attrDef_long_midSent_min[kNumOffsets*2] = {
 static const char* en_dec0_long_midSent_tues[kNumOffsets*2] = {
 /*  text                    numeric */
     "5 Tuesdays ago",       "5 Tuesdays ago",     /* -5   */
-    ""/*no data */,         ""/*no data */,       /* -2.2 */
+    "2.2 Tuesdays ago",     "2.2 Tuesdays ago",   /* -2.2 */
     "2 Tuesdays ago",       "2 Tuesdays ago",     /* -2   */
     "last Tuesday",         "1 Tuesday ago",      /* -1   */
-    ""/*no data */,         ""/*no data */,       /* -0.7 */
+    "0.7 Tuesdays ago",     "0.7 Tuesdays ago",   /* -0.7 */
     "this Tuesday",         "0 Tuesdays ago",     /* -0   */
     "this Tuesday",         "in 0 Tuesdays",      /*  0   */
-    ""/*no data */,         ""/*no data */,       /*  0.7 */
+    "in 0.7 Tuesdays",      "in 0.7 Tuesdays",    /*  0.7 */
     "next Tuesday",         "in 1 Tuesday",       /*  1   */
     "in 2 Tuesdays",        "in 2 Tuesdays",      /*  2   */
     "in 5 Tuesdays",        "in 5 Tuesdays",      /*  5   */
@@ -212,16 +214,16 @@ static const char* en_dec0_long_midSent_tues[kNumOffsets*2] = {
 static const FieldsDat en_attr0_long_midSent_tues[kNumOffsets*2] = {
 /*  text           numeric           text                    numeric */
     {UDAT_REL_NUMERIC_FIELD,  0,  1}, {UDAT_REL_NUMERIC_FIELD,  0,  1}, /* "5 Tuesdays ago",       "5 Tuesdays ago",      -5   */
-    { -1, -1, -1},                    { -1, -1, -1},                    /* ""            ,         ""            ,        -2.2 */
+    {UDAT_REL_NUMERIC_FIELD,  0,  3}, {UDAT_REL_NUMERIC_FIELD,  0,  3}, /* "2.2 Tuesdays ago",     "2.2 Tuesdays ago",    -2.2 */
     {UDAT_REL_NUMERIC_FIELD,  0,  1}, {UDAT_REL_NUMERIC_FIELD,  0,  1}, /* "2 Tuesdays ago",       "2 Tuesdays ago",      -2   */
     { -1, -1, -1},                    {UDAT_REL_NUMERIC_FIELD,  0,  1}, /* "last Tuesday",         "1 Tuesday ago",       -1   */
-    { -1, -1, -1},                    { -1, -1, -1},                    /* ""            ,         ""            ,        -0.7 */
+    {UDAT_REL_NUMERIC_FIELD,  0,  3}, {UDAT_REL_NUMERIC_FIELD,  0,  3}, /* "0.7 Tuesdays ago",     "0.7 Tuesdays ago",    -0.7 */
     { -1, -1, -1},                    {UDAT_REL_NUMERIC_FIELD,  0,  1}, /* "this Tuesday",         "0 Tuesdays ago",      -0   */
     { -1, -1, -1},                    {UDAT_REL_NUMERIC_FIELD,  3,  4}, /* "this Tuesday",         "in 0 Tuesdays",        0   */
-    { -1, -1, -1},                    { -1, -1, -1},                    /* ""            ,         ""            ,         0.7 */
+    {UDAT_REL_NUMERIC_FIELD,  3,  6}, {UDAT_REL_NUMERIC_FIELD,  3,  6}, /* "in 0.7 Tuesdays",      "in 0.7 Tuesdays",      0.7 */
     { -1, -1, -1},                    {UDAT_REL_NUMERIC_FIELD,  3,  4}, /* "next Tuesday",         "in 1 Tuesday",         1   */
-    {UDAT_REL_NUMERIC_FIELD,  0,  1}, {UDAT_REL_NUMERIC_FIELD,  3,  4}, /* "in 2 Tuesdays",        "in 2 Tuesdays",        2   */
-    {UDAT_REL_NUMERIC_FIELD,  0,  1}, {UDAT_REL_NUMERIC_FIELD,  3,  4}, /* "in 5 Tuesdays",        "in 5 Tuesdays",        5   */
+    {UDAT_REL_NUMERIC_FIELD,  3,  4}, {UDAT_REL_NUMERIC_FIELD,  3,  4}, /* "in 2 Tuesdays",        "in 2 Tuesdays",        2   */
+    {UDAT_REL_NUMERIC_FIELD,  3,  4}, {UDAT_REL_NUMERIC_FIELD,  3,  4}, /* "in 5 Tuesdays",        "in 5 Tuesdays",        5   */
 };
 
 static const char* fr_decDef_long_midSent_day[kNumOffsets*2] = {
@@ -256,46 +258,46 @@ static const FieldsDat fr_attrDef_long_midSent_day[kNumOffsets*2] = {
 
 static const char* ak_decDef_long_stdAlon_sec[kNumOffsets*2] = { // falls back to root
 /*  text                    numeric */
-    "-5 s",                 "-5 s",               /* -5   */
-    "-2.2 s",               "-2.2 s",             /* -2.2 */
-    "-2 s",                 "-2 s",               /* -2   */
-    "-1 s",                 "-1 s",               /* -1   */
-    "-0.7 s",               "-0.7 s",             /* -0.7 */
-    "now",                  "-0 s",               /*  -0  */
-    "now",                  "+0 s",               /*  0   */
-    "+0.7 s",               "+0.7 s",             /*  0.7 */
-    "+1 s",                 "+1 s",               /*  1   */
-    "+2 s",                 "+2 s",               /*  2   */
-    "+5 s",                 "+5 s",               /*  5   */
+    "nnibuo 5 a atwam",     "nnibuo 5 a atwam",               /* -5   */
+    "nnibuo 2.2 a atwam",   "nnibuo 2.2 a atwam",             /* -2.2 */
+    "nnibuo 2 a atwam",     "nnibuo 2 a atwam",               /* -2   */
+    "anibuo 1 a atwam",     "anibuo 1 a atwam",               /* -1   */
+    "nnibuo 0.7 a atwam",   "nnibuo 0.7 a atwam",             /* -0.7 */
+    "seesei",               "anibuo 0 a atwam",               /*  -0  */
+    "seesei",               "anibuo 0 mu",               /*  0   */
+    "nnibuo 0.7 mu",        "nnibuo 0.7 mu",             /*  0.7 */
+    "anibuo 1 mu",          "anibuo 1 mu",               /*  1   */
+    "nnibuo 2 mu",          "nnibuo 2 mu",               /*  2   */
+    "nnibuo 5 mu",          "nnibuo 5 mu",               /*  5   */
 };
 
 static const FieldsDat ak_attrDef_long_stdAlon_sec[kNumOffsets*2] = {
-    {UDAT_REL_NUMERIC_FIELD, 1, 2}, {UDAT_REL_NUMERIC_FIELD, 1, 2},
-    {UDAT_REL_NUMERIC_FIELD, 1, 4}, {UDAT_REL_NUMERIC_FIELD, 1, 4},
-    {UDAT_REL_NUMERIC_FIELD, 1, 2}, {UDAT_REL_NUMERIC_FIELD, 1, 2},
-    {UDAT_REL_NUMERIC_FIELD, 1, 2}, {UDAT_REL_NUMERIC_FIELD, 1, 2},
-    {UDAT_REL_NUMERIC_FIELD, 1, 4}, {UDAT_REL_NUMERIC_FIELD, 1, 4},
-    { -1, -1, -1},                  {UDAT_REL_NUMERIC_FIELD, 1, 2},
-    { -1, -1, -1},                  {UDAT_REL_NUMERIC_FIELD, 1, 2},
-    {UDAT_REL_NUMERIC_FIELD, 1, 4}, {UDAT_REL_NUMERIC_FIELD, 1, 4},
-    {UDAT_REL_NUMERIC_FIELD, 1, 2}, {UDAT_REL_NUMERIC_FIELD, 1, 2},
-    {UDAT_REL_NUMERIC_FIELD, 1, 2}, {UDAT_REL_NUMERIC_FIELD, 1, 2},
-    {UDAT_REL_NUMERIC_FIELD, 1, 2}, {UDAT_REL_NUMERIC_FIELD, 1, 2},
+    {UDAT_REL_NUMERIC_FIELD, 7, 8}, {UDAT_REL_NUMERIC_FIELD, 7, 8},
+    {UDAT_REL_NUMERIC_FIELD, 7, 10}, {UDAT_REL_NUMERIC_FIELD, 7, 10},
+    {UDAT_REL_NUMERIC_FIELD, 7, 8}, {UDAT_REL_NUMERIC_FIELD, 7, 8},
+    {UDAT_REL_NUMERIC_FIELD, 7, 8}, {UDAT_REL_NUMERIC_FIELD, 7, 8},
+    {UDAT_REL_NUMERIC_FIELD, 7, 10}, {UDAT_REL_NUMERIC_FIELD, 7, 10},
+    { -1, -1, -1},                  {UDAT_REL_NUMERIC_FIELD, 7, 8},
+    { -1, -1, -1},                  {UDAT_REL_NUMERIC_FIELD, 7, 8},
+    {UDAT_REL_NUMERIC_FIELD, 7, 10}, {UDAT_REL_NUMERIC_FIELD, 7, 10},
+    {UDAT_REL_NUMERIC_FIELD, 7, 8}, {UDAT_REL_NUMERIC_FIELD, 7, 8},
+    {UDAT_REL_NUMERIC_FIELD, 7, 8}, {UDAT_REL_NUMERIC_FIELD, 7, 8},
+    {UDAT_REL_NUMERIC_FIELD, 7, 8}, {UDAT_REL_NUMERIC_FIELD, 7, 8},
 };
 
 static const char* enIN_decDef_short_midSent_weds[kNumOffsets*2] = {
 /*  text                    numeric */
-    "5 Wed. ago",           "5 Wed. ago",         /* -5   */
-    "2.2 Wed. ago",         "2.2 Wed. ago",       /* -2.2 */
-    "2 Wed. ago",           "2 Wed. ago",         /* -2   */
-    "last Wed",             "1 Wed. ago",         /* -1   */
-    "0.7 Wed. ago",         "0.7 Wed. ago",       /* -0.7 */
-    "this Wed",             "0 Wed. ago",         /*  -0  */
-    "this Wed",             "in 0 Wed.",          /*  0   */
-    "in 0.7 Wed.",          "in 0.7 Wed.",        /*  0.7 */
+    "5 Wed ago",            "5 Wed ago",          /* -5   */
+    "2.2 Wed ago",          "2.2 Wed ago",        /* -2.2 */
+    "2 Wed ago",            "2 Wed ago",          /* -2   */
+    "last Wed",             "1 Wed ago",          /* -1   */
+    "0.7 Wed ago",          "0.7 Wed ago",        /* -0.7 */
+    "this Wed",             "0 Wed ago",          /*  -0  */
+    "this Wed",             "in 0 Wed",           /*  0   */
+    "in 0.7 Wed",           "in 0.7 Wed",         /*  0.7 */
     "next Wed",             "in 1 Wed",           /*  1   */ // in 1 Wed. missing in logical group
-    "in 2  Wed.",           "in 2 Wed.",          /*  2   */
-    "in 5  Wed.",           "in 5 Wed."           /*  5   */
+    "in 2 Wed",             "in 2 Wed",           /*  2   */
+    "in 5 Wed",             "in 5 Wed"            /*  5   */
 };
 
 static const FieldsDat enIN_attrDef_short_midSent_weds[kNumOffsets*2] = {
@@ -341,12 +343,14 @@ static const RelDateTimeFormatTestItem fmtTestItems[] = {
       ak_decDef_long_stdAlon_sec,   ak_attrDef_long_stdAlon_sec},
     { "en_IN", -1, UDAT_STYLE_SHORT, UDISPCTX_CAPITALIZATION_FOR_MIDDLE_OF_SENTENCE, UDAT_REL_UNIT_WEDNESDAY,
       enIN_decDef_short_midSent_weds, enIN_attrDef_short_midSent_weds},
+    { "en@calendar=iso8601", -1, UDAT_STYLE_LONG,  UDISPCTX_CAPITALIZATION_FOR_MIDDLE_OF_SENTENCE, UDAT_REL_UNIT_SECOND,
+      en_decDef_long_midSent_sec,   en_attrDef_long_midSent_sec },
     { NULL,  0, (UDateRelativeDateTimeFormatterStyle)0, (UDisplayContext)0, (URelativeDateTimeUnit)0, NULL, NULL } /* terminator */
 };
 
 enum { kUBufMax = 64, kBBufMax = 256 };
 
-static void TestRelDateFmt()
+static void TestRelDateFmt(void)
 {
     const RelDateTimeFormatTestItem *itemPtr;
     log_verbose("\nTesting ureldatefmt_open(), ureldatefmt_format(), ureldatefmt_formatNumeric() with various parameters\n");
@@ -378,10 +382,6 @@ static void TestRelDateFmt()
             UChar ubufget[kUBufMax];
             int32_t ulenget;
 
-            if (itemPtr->unit >= UDAT_REL_UNIT_SUNDAY && offsets[iOffset] != -1.0 && offsets[iOffset] != 0.0 && offsets[iOffset] != 1.0) {
-                continue; /* we do not currently have data for this */
-            }
-
             status = U_ZERO_ERROR;
             ulenget = ureldatefmt_format(reldatefmt, offsets[iOffset], itemPtr->unit, ubufget, kUBufMax, &status);
             if ( U_FAILURE(status) ) {
@@ -400,10 +400,6 @@ static void TestRelDateFmt()
                 }
             }
 
-            if (itemPtr->unit >= UDAT_REL_UNIT_SUNDAY) {
-                continue; /* we do not currently have numeric-style data for this */
-            }
-
             status = U_ZERO_ERROR;
             ulenget = ureldatefmt_formatNumeric(reldatefmt, offsets[iOffset], itemPtr->unit, ubufget, kUBufMax, &status);
             if ( U_FAILURE(status) ) {
@@ -427,7 +423,7 @@ static void TestRelDateFmt()
     }
 }
 
-static void TestNumericField()
+static void TestNumericField(void)
 {
     const RelDateTimeFormatTestItem *itemPtr;
     log_verbose("\nTesting ureldatefmt_open(), ureldatefmt_formatForFields(), ureldatefmt_formatNumericForFields() with various parameters\n");
@@ -456,10 +452,6 @@ static void TestNumericField()
         }
 
         for (iOffset = 0; iOffset < kNumOffsets; iOffset++) {
-            if (itemPtr->unit >= UDAT_REL_UNIT_SUNDAY && offsets[iOffset] != -1.0 && offsets[iOffset] != 0.0 && offsets[iOffset] != 1.0) {
-                continue; /* we do not currently have data for this */
-            }
-
             /* Depend on the next one to verify the data */
             status = U_ZERO_ERROR;
             UFormattedRelativeDateTime* fv = ureldatefmt_openResult(&status);
@@ -482,8 +474,8 @@ static void TestNumericField()
 
                 FieldsDat expectedAttr = itemPtr->expectedAttributes[iOffset*2];
                 UConstrainedFieldPosition* cfpos = ucfpos_open(&status);
-                UBool foundNumeric = FALSE;
-                while (TRUE) {
+                UBool foundNumeric = false;
+                while (true) {
                     foundNumeric = ufmtval_nextPosition(ureldatefmt_resultAsValue(fv, &status), cfpos, &status);
                     if (!foundNumeric) {
                         break;
@@ -515,11 +507,6 @@ static void TestNumericField()
                 ucfpos_close(cfpos);
             }
 
-            if (itemPtr->unit >= UDAT_REL_UNIT_SUNDAY) {
-                ureldatefmt_closeResult(fv);
-                continue; /* we do not currently have numeric-style data for this */
-            }
-
             /* Depend on the next one to verify the data */
             status = U_ZERO_ERROR;
             ureldatefmt_formatNumericToResult(reldatefmt, offsets[iOffset], itemPtr->unit, fv, &status);
@@ -537,8 +524,8 @@ static void TestNumericField()
 
                 FieldsDat expectedAttr = itemPtr->expectedAttributes[iOffset*2 + 1];
                 UConstrainedFieldPosition* cfpos = ucfpos_open(&status);
-                UBool foundNumeric = FALSE;
-                while (TRUE) {
+                UBool foundNumeric = false;
+                while (true) {
                     foundNumeric = ufmtval_nextPosition(ureldatefmt_resultAsValue(fv, &status), cfpos, &status);
                     if (!foundNumeric) {
                         break;
@@ -589,7 +576,7 @@ static const CombineDateTimeTestItem combTestItems[] = {
     { NULL,  (UDateRelativeDateTimeFormatterStyle)0, (UDisplayContext)0, NULL, NULL, NULL } /* terminator */
 };
 
-static void TestCombineDateTime()
+static void TestCombineDateTime(void)
 {
     const CombineDateTimeTestItem *itemPtr;
     log_verbose("\nTesting ureldatefmt_combineDateAndTime() with various parameters\n");
@@ -643,7 +630,7 @@ static void TestCombineDateTime()
     }
 }
 
-static void TestFields() {
+static void TestFields(void) {
     UErrorCode ec = U_ZERO_ERROR;
     URelativeDateTimeFormatter* fmt = ureldatefmt_open(
         "en-us",
