@@ -1,6 +1,8 @@
+// © 2016 and later: Unicode, Inc. and others.
+// License & terms of use: http://www.unicode.org/copyright.html
 /********************************************************************
  * COPYRIGHT:
- * Copyright (c) 2001-2014, International Business Machines Corporation and
+ * Copyright (c) 2001-2016, International Business Machines Corporation and
  * others. All Rights Reserved.
  ********************************************************************/
 /********************************************************************************
@@ -15,8 +17,9 @@
 /****************************************************************************/
 
 
-#include <stdlib.h>
+#include <stdbool.h>
 #include <stdio.h>
+#include <stdlib.h>
 #include <string.h>
 #include "unicode/utypes.h"
 #include "unicode/ustring.h"
@@ -205,7 +208,7 @@ static void Test_strToUTF32(void){
 }
 
 /* test unpaired surrogates */
-static void Test_strToUTF32_surrogates() {
+static void Test_strToUTF32_surrogates(void) {
     UErrorCode err = U_ZERO_ERROR;
     UChar32 u32Target[400];
     int32_t len16, u32DestLen;
@@ -385,7 +388,7 @@ static void Test_strFromUTF32(void){
 }
 
 /* test surrogate code points */
-static void Test_strFromUTF32_surrogates() {
+static void Test_strFromUTF32_surrogates(void) {
     UErrorCode err = U_ZERO_ERROR;
     UChar uTarget[400];
     int32_t len32, uDestLen;
@@ -514,7 +517,7 @@ static void Test_UChar_UTF8_API(void){
     char* u8Target = u8Temp;
     int32_t u8TargetLength =0;
     int32_t u8DestLen =0;
-    UBool failed = FALSE;
+    UBool failed = false;
     int i= 0;
     int32_t numSubstitutions;
 
@@ -539,7 +542,7 @@ static void Test_UChar_UTF8_API(void){
         else {
             log_err("Should have gotten U_BUFFER_OVERFLOW_ERROR");
         }
-        failed = FALSE;
+        failed = false;
         /*for(i=0; i< u8DestLen; i++){
             printf("0x%04X, ",u8Target[i]);
             if(i%10==0){
@@ -549,7 +552,7 @@ static void Test_UChar_UTF8_API(void){
         /*for(i=0; i< u8DestLen; i++){
             if(u8Target[i] != src8[i]){
                 log_verbose("u_strToUTF8() failed expected: %04X got: %04X \n", src8[i], u8Target[i]);
-                failed =TRUE;
+                failed =true;
             }
         }
         if(failed){
@@ -581,12 +584,12 @@ static void Test_UChar_UTF8_API(void){
         }*/
 
         if(U_FAILURE(err) || uDestLen != uTargetLength || uTarget[uTargetLength] != 0xfff0) {
-            failed = TRUE;
+            failed = true;
         }
         for(i=0; i< uSrcLen; i++){
             if(uTarget[i] != src16[i]){
                 log_verbose("u_strFromUTF8() failed expected: \\u%04X got: \\u%04X at index: %i \n", src16[i] ,uTarget[i],i);
-                failed =TRUE;
+                failed =true;
             }
         }
         if(failed){
@@ -601,7 +604,7 @@ static void Test_UChar_UTF8_API(void){
         uTargetLength = 0;
         uSrcLen =-1;
         u8TargetLength=0;
-        failed = FALSE;
+        failed = false;
         /* preflight */
         u_strToUTF8(NULL,u8TargetLength, &u8DestLen, uSrc, uSrcLen,&err);
         if(err == U_BUFFER_OVERFLOW_ERROR){
@@ -615,7 +618,7 @@ static void Test_UChar_UTF8_API(void){
         else {
             log_err("Should have gotten U_BUFFER_OVERFLOW_ERROR");
         }
-        failed = FALSE;
+        failed = false;
         /*for(i=0; i< u8DestLen; i++){
             printf("0x%04X, ",u8Target[i]);
             if(i%10==0){
@@ -625,7 +628,7 @@ static void Test_UChar_UTF8_API(void){
         /*for(i=0; i< u8DestLen; i++){
             if(u8Target[i] != src8[i]){
                 log_verbose("u_strToUTF8() failed expected: %04X got: %04X \n", src8[i], u8Target[i]);
-                failed =TRUE;
+                failed =true;
             }
         }
         if(failed){
@@ -656,7 +659,7 @@ static void Test_UChar_UTF8_API(void){
         for(i=0; i< uSrcLen; i++){
             if(uTarget[i] != src16[i]){
                 log_verbose("u_strFromUTF8() failed expected: \\u%04X got: \\u%04X at index: %i \n", src16[i] ,uTarget[i],i);
-                failed =TRUE;
+                failed =true;
             }
         }
         if(failed){
@@ -668,12 +671,13 @@ static void Test_UChar_UTF8_API(void){
     }
 
     /* test UTF-8 with single surrogates - illegal in Unicode 3.2 */
+    // Since ICU 60, each surrogate byte sequence is treated as 3 single-byte errors.
     {
         static const UChar
             withLead16[]={ 0x1800, 0xd89a, 0x0061 },
             withTrail16[]={ 0x1800, 0xdcba, 0x0061, 0 },
-            withTrail16SubFFFD[]={ 0x1800, 0xfffd, 0x0061, 0 }, /* sub==U+FFFD */
-            withTrail16Sub50005[]={ 0x1800, 0xd900, 0xdc05, 0x0061, 0 }; /* sub==U+50005 */
+            withTrail16SubFFFD[]={ 0x1800, 0xfffd, 0xfffd, 0xfffd, 0x0061, 0 }, /* sub==U+FFFD */
+            withTrail16Sub50005[]={ 0x1800, 0xd900, 0xdc05, 0xd900, 0xdc05, 0xd900, 0xdc05, 0x0061, 0 }; /* sub==U+50005 */
         static const uint8_t
             withLead8[]={ 0xe1, 0xa0, 0x80, 0xed, 0xa2, 0x9a, 0x61 },
             withTrail8[]={ 0xe1, 0xa0, 0x80, 0xed, 0xb2, 0xba, 0x61, 0 },
@@ -699,12 +703,12 @@ static void Test_UChar_UTF8_API(void){
         out16[0]=0x55aa;
         uDestLen=0;
         u_strFromUTF8WithSub(out16, UPRV_LENGTHOF(out16), &uDestLen,
-                             (const char *)withTrail8, uprv_strlen((const char *)withTrail8),
+                             (const char *)withTrail8, (int32_t)uprv_strlen((const char *)withTrail8),
                              0x50005, &numSubstitutions,
                              &err);
         if(U_FAILURE(err) || uDestLen!=u_strlen(withTrail16Sub50005) ||
                              0!=u_memcmp(withTrail16Sub50005, out16, uDestLen+1) ||
-                             numSubstitutions!=1) {
+                             numSubstitutions!=3) {
             log_err("error: u_strFromUTF8WithSub(length) failed\n");
         }
 
@@ -719,7 +723,7 @@ static void Test_UChar_UTF8_API(void){
                              &err);
         if(U_FAILURE(err) || uDestLen!=u_strlen(withTrail16SubFFFD) ||
                              0!=u_memcmp(withTrail16SubFFFD, out16, uDestLen+1) ||
-                             numSubstitutions!=1) {
+                             numSubstitutions!=3) {
             log_err("error: u_strFromUTF8WithSub(NUL termination) failed\n");
         }
 
@@ -732,7 +736,7 @@ static void Test_UChar_UTF8_API(void){
                              (const char *)withTrail8, -1,
                              0x50005, &numSubstitutions,
                              &err);
-        if(err!=U_BUFFER_OVERFLOW_ERROR || uDestLen!=u_strlen(withTrail16Sub50005) || numSubstitutions!=1) {
+        if(err!=U_BUFFER_OVERFLOW_ERROR || uDestLen!=u_strlen(withTrail16Sub50005) || numSubstitutions!=3) {
             log_err("error: u_strFromUTF8WithSub(preflight/NUL termination) failed\n");
         }
 
@@ -745,7 +749,7 @@ static void Test_UChar_UTF8_API(void){
                            withTrail16, u_strlen(withTrail16),
                            0xfffd, &numSubstitutions,
                            &err);
-        if(U_FAILURE(err) || u8DestLen!=uprv_strlen((const char *)withTrail8SubFFFD) ||
+        if(U_FAILURE(err) || u8DestLen!=(int32_t)uprv_strlen((const char *)withTrail8SubFFFD) ||
                              0!=uprv_memcmp((const char *)withTrail8SubFFFD, out8, u8DestLen+1) ||
                              numSubstitutions!=1) {
             log_err("error: u_strToUTF8WithSub(length) failed\n");
@@ -760,7 +764,7 @@ static void Test_UChar_UTF8_API(void){
                            withTrail16, -1,
                            0x1a, &numSubstitutions,
                            &err);
-        if(U_FAILURE(err) || u8DestLen!=uprv_strlen((const char *)withTrail8Sub1A) ||
+        if(U_FAILURE(err) || u8DestLen!=(int32_t)uprv_strlen((const char *)withTrail8Sub1A) ||
                              0!=uprv_memcmp((const char *)withTrail8Sub1A, out8, u8DestLen+1) ||
                              numSubstitutions!=1) {
             log_err("error: u_strToUTF8WithSub(NUL termination) failed\n");
@@ -775,7 +779,7 @@ static void Test_UChar_UTF8_API(void){
                            withTrail16, -1,
                            0xfffd, &numSubstitutions,
                            &err);
-        if(err!=U_BUFFER_OVERFLOW_ERROR || u8DestLen!=uprv_strlen((const char *)withTrail8SubFFFD) ||
+        if(err!=U_BUFFER_OVERFLOW_ERROR || u8DestLen!=(int32_t)uprv_strlen((const char *)withTrail8SubFFFD) ||
                                            numSubstitutions!=1) {
             log_err("error: u_strToUTF8WithSub(preflight/NUL termination) failed\n");
         }
@@ -868,11 +872,11 @@ equalAnyFFFD(const UChar *s, const UChar *t, int32_t length) {
         c1=*s++;
         c2=*t++;
         if(c1!=c2 && c2!=0xfffd) {
-            return FALSE;
+            return false;
         }
         --length;
     }
-    return TRUE;
+    return true;
 }
 
 /* test u_strFromUTF8Lenient() */
@@ -1013,14 +1017,6 @@ Test_FromUTF8Lenient(void) {
         log_err("u_strFromUTF8Lenient(U_MEMORY_ALLOCATION_ERROR) failed\n");
     }
 
-    dest[0]=0x1234;
-    destLength=-1;
-    errorCode=U_MEMORY_ALLOCATION_ERROR;
-    pDest=u_strFromUTF8Lenient(dest, 1, &destLength, (const char *)bytes, -1, NULL);
-    if(dest[0]!=0x1234) {
-        log_err("u_strFromUTF8Lenient(pErrorCode=NULL) failed\n");
-    }
-
     /* test normal behavior */
     number=0; /* string number for log_err() */
 
@@ -1028,7 +1024,7 @@ Test_FromUTF8Lenient(void) {
         *pb!=(char)0xff;
         pb+=srcLength+1, pu+=destLength0+1, ++number
     ) {
-        srcLength=uprv_strlen(pb);
+        srcLength=(int32_t)uprv_strlen(pb);
         destLength0=u_strlen(pu);
 
         /* preflighting with NUL-termination */
@@ -1176,7 +1172,7 @@ static void Test_UChar_WCHART_API(void){
     wchar_t* wDest = NULL;
     int32_t wDestLen = 0;
     int32_t reqLen= 0 ;
-    UBool failed = FALSE;
+    UBool failed = false;
     UChar* uDest = NULL;
     int32_t uDestLen = 0;
     int i =0;
@@ -1230,12 +1226,12 @@ static void Test_UChar_WCHART_API(void){
         for(i=0; i< uSrcLen; i++){
             if(uDest[i] != src16j[i]){
                 log_verbose("u_str*WCS() failed for unterminated string expected: \\u%04X got: \\u%04X at index: %i \n", src16j[i] ,uDest[i],i);
-                failed =TRUE;
+                failed =true;
             }
         }
 
         if(U_FAILURE(err)){
-            failed = TRUE;
+            failed = true;
         }
         if(failed){
             log_err("u_strToWCS() failed \n");
@@ -1244,7 +1240,7 @@ static void Test_UChar_WCHART_API(void){
         free(uDest);
         
        
-        /* test with embeded nulls */
+        /* test with embedded nulls */
         uSrc = src16WithNulls;
         uSrcLen = sizeof(src16WithNulls)/2;
         wDestLen =0;
@@ -1275,13 +1271,13 @@ static void Test_UChar_WCHART_API(void){
          for(i=0; i< uSrcLen; i++){
             if(uDest[i] != src16WithNulls[i]){
                 log_verbose("u_str*WCS() failed for string with nulls expected: \\u%04X got: \\u%04X at index: %i \n", src16WithNulls[i] ,uDest[i],i);
-                failed =TRUE;
+                failed =true;
             }
          }
         }
 
         if(U_FAILURE(err)){
-            failed = TRUE;
+            failed = true;
         }
         if(failed){
             log_err("u_strToWCS() failed \n");
@@ -1325,13 +1321,13 @@ static void Test_UChar_WCHART_API(void){
          for(i=0; i< uSrcLen; i++){
             if(uDest[i] != src16j[i]){
                 log_verbose("u_str*WCS() failed for null terminated string expected: \\u%04X got: \\u%04X at index: %i \n", src16j[i] ,uDest[i],i);
-                failed =TRUE;
+                failed =true;
             }
          }
         }
 
         if(U_FAILURE(err)){
-            failed = TRUE;
+            failed = true;
         }
         if(failed){
             log_err("u_strToWCS() failed \n");
@@ -1385,20 +1381,20 @@ static void Test_UChar_WCHART_API(void){
 #endif
 } 
 
-static void Test_widestrs()
+static void Test_widestrs(void)
 {
 #if (defined(U_WCHAR_IS_UTF16) || defined(U_WCHAR_IS_UTF32)) || (!UCONFIG_NO_CONVERSION && !UCONFIG_NO_LEGACY_CONVERSION)
         wchar_t ws[100];
         UChar rts[100];
-        int32_t wcap = sizeof(ws) / sizeof(*ws);
+        int32_t wcap = UPRV_LENGTHOF(ws);
         int32_t wl;
-        int32_t rtcap = sizeof(rts) / sizeof(*rts);
+        int32_t rtcap = UPRV_LENGTHOF(rts);
         int32_t rtl;
         wchar_t *wcs;
         UChar *cp;
         const char *errname;
         UChar ustr[] = {'h', 'e', 'l', 'l', 'o', 0};
-        int32_t ul = sizeof(ustr)/sizeof(*ustr) -1;
+        int32_t ul = UPRV_LENGTHOF(ustr) -1;
         char astr[100];
 
         UErrorCode err;
@@ -1429,7 +1425,7 @@ static void Test_widestrs()
 }
 
 static void
-Test_WCHART_LongString(){
+Test_WCHART_LongString(void){
 #if (defined(U_WCHAR_IS_UTF16) || defined(U_WCHAR_IS_UTF32)) || (!UCONFIG_NO_CONVERSION && !UCONFIG_NO_LEGACY_CONVERSION)
     UErrorCode status = U_ZERO_ERROR;
     const char* testdatapath=loadTestData(&status);
@@ -1442,7 +1438,7 @@ Test_WCHART_LongString(){
     int32_t uDestLen =0;
     wchar_t* wDest = NULL;
     UChar* uDest = NULL;
-    UBool failed = FALSE;
+    UBool failed = false;
 
     log_verbose("Loaded string of %d UChars\n", uSrcLen);
 
@@ -1498,12 +1494,12 @@ Test_WCHART_LongString(){
     for(i=0; i< uSrcLen; i++){
         if(uDest[i] != str[i]){
             log_verbose("u_str*WCS() failed for null terminated string expected: \\u%04X got: \\u%04X at index: %i \n", str[i], uDest[i],i);
-            failed =TRUE;
+            failed =true;
         }
     }
 
     if(U_FAILURE(status)){
-        failed = TRUE;
+        failed = true;
     }
     if(failed){
         log_err("u_strToWCS() failed \n");
@@ -1517,7 +1513,7 @@ Test_WCHART_LongString(){
 #endif
 }
 
-static void Test_strToJavaModifiedUTF8() {
+static void Test_strToJavaModifiedUTF8(void) {
     static const UChar src[]={
         0x61, 0x62, 0x63, 0xe1, 0xe2, 0xe3,
         0xe01, 0xe02, 0xe03, 0xe001, 0xe002, 0xe003,
@@ -1693,7 +1689,7 @@ static void Test_strToJavaModifiedUTF8() {
     }
 }
 
-static void Test_strFromJavaModifiedUTF8() {
+static void Test_strFromJavaModifiedUTF8(void) {
     static const uint8_t src[]={
         0x61, 0x62, 0x63, 0xc3, 0xa1, 0xc3, 0xa2, 0xc3, 0xa3,
         0xe0, 0xb8, 0x81, 0xe0, 0xb8, 0x82, 0xe0, 0xb8, 0x83,
@@ -1996,7 +1992,7 @@ static void Test_strFromJavaModifiedUTF8() {
 }
 
 /* test that string transformation functions permit NULL source pointer when source length==0 */
-static void TestNullEmptySource() {
+static void TestNullEmptySource(void) {
     char dest8[4]={ 3, 3, 3, 3 };
     UChar dest16[4]={ 3, 3, 3, 3 };
     UChar32 dest32[4]={ 3, 3, 3, 3 };
