@@ -20,7 +20,7 @@ public:
     void TestULocale();
     void TestUResourceBundle();
     void TestDisplayName(); 
-    void runIndexedTest( int32_t index, UBool exec, const char* &name, char* par = NULL );
+    void runIndexedTest( int32_t index, UBool exec, const char* &name, char* par = nullptr ) override;
     LocaleAliasTest();
     virtual ~LocaleAliasTest();
 private:
