@@ -13,6 +13,7 @@
 #ifndef ALPHAINDEXTST_H
 #define ALPHAINDEXTST_H
 
+#include "unicode/uscript.h"
 #include "intltest.h"
 
 class AlphabeticIndexTest: public IntlTest {
@@ -20,7 +21,7 @@ public:
     AlphabeticIndexTest();
     virtual ~AlphabeticIndexTest();
 
-    virtual void runIndexedTest(int32_t index, UBool exec, const char* &name, char* par = NULL );
+    virtual void runIndexedTest(int32_t index, UBool exec, const char* &name, char* par = nullptr ) override;
 
     virtual void APITest();
     virtual void ManyLocalesTest();
@@ -49,6 +50,9 @@ public:
     void TestChineseZhuyin();
     void TestJapaneseKanji();
     void TestChineseUnihan();
+
+    void testHasBuckets();
+    void checkHasBuckets(const Locale &locale, UScriptCode script);
 };
 
 #endif
