@@ -23,7 +23,7 @@
 
 class CollationAPITest: public IntlTestCollator {
 public:
-    void runIndexedTest( int32_t index, UBool exec, const char* &name, char* /*par = NULL */) override;
+    void runIndexedTest( int32_t index, UBool exec, const char* &name, char* /*par = nullptr */) override;
     void doAssert(UBool condition, const char *message);
 
     /**
@@ -165,15 +165,16 @@ public:
     void TestUClassID();
 
     /**
-    * Test NULL
+    * Test nullptr
     */
-    void TestNULLCharTailoring();
+    void TestNullptrCharTailoring();
 
     void TestClone();
     void TestCloneBinary();
     void TestIterNumeric();
     void TestBadKeywords();
     void TestGapTooSmall();
+    void TestNFCNull();
 
 private:
     // If this is too small for the test data, just increase it.
