@@ -34,7 +34,7 @@ public:
     BytesTrieTest();
     virtual ~BytesTrieTest();
 
-    void runIndexedTest(int32_t index, UBool exec, const char *&name, char *par=NULL);
+    void runIndexedTest(int32_t index, UBool exec, const char *&name, char *par=nullptr) override;
     void TestBuilder();
     void TestEmpty();
     void Test_a();
@@ -56,6 +56,7 @@ public:
     void TestTruncatingIteratorFromLinearMatchLong();
     void TestIteratorFromBytes();
     void TestFailedIterator();
+    void TestDelta();
 
     void checkData(const StringAndValue data[], int32_t dataLength);
     void checkData(const StringAndValue data[], int32_t dataLength, UStringTrieBuildOption buildOption);
@@ -77,7 +78,7 @@ extern IntlTest *createBytesTrieTest() {
     return new BytesTrieTest();
 }
 
-BytesTrieTest::BytesTrieTest() : builder_(NULL) {
+BytesTrieTest::BytesTrieTest() : builder_(nullptr) {
     IcuTestErrorCode errorCode(*this, "BytesTrieTest()");
     builder_=new BytesTrieBuilder(errorCode);
 }
@@ -110,6 +111,7 @@ void BytesTrieTest::runIndexedTest(int32_t index, UBool exec, const char *&name,
     TESTCASE_AUTO(TestTruncatingIteratorFromLinearMatchLong);
     TESTCASE_AUTO(TestIteratorFromBytes);
     TESTCASE_AUTO(TestFailedIterator);
+    TESTCASE_AUTO(TestDelta);
     TESTCASE_AUTO_END;
 }
 
@@ -174,10 +176,10 @@ void BytesTrieTest::TestBranches() {
         { "t", 0x400000 },
         { "uu", 0x800000 },
         { "vv", 0x7fffffff },
-        { "zz", (int32_t)0x80000000 }
+        { "zz", static_cast<int32_t>(0x80000000) }
     };
     for(int32_t length=2; length<=UPRV_LENGTHOF(data); ++length) {
-        logln("TestBranches length=%d", (int)length);
+        logln("TestBranches length=%d", static_cast<int>(length));
         checkData(data, length);
     }
 }
@@ -221,7 +223,7 @@ void BytesTrieTest::TestLongBranch() {
         { "r", 0x333333 },
         { "s2345", 0x4444444 },
         { "t234567890", 0x77777777 },
-        { "z", (int32_t)0x80000001 }
+        { "z", static_cast<int32_t>(0x80000001) }
     };
     checkData(data, UPRV_LENGTHOF(data));
 }
@@ -592,17 +594,56 @@ void BytesTrieTest::TestIteratorFromBytes() {
 
 void BytesTrieTest::TestFailedIterator() {
     UErrorCode failure = U_ILLEGAL_ARGUMENT_ERROR;
-    BytesTrie::Iterator iter(NULL, 0, failure);
+    BytesTrie::Iterator iter(nullptr, 0, failure);
     StringPiece sp = iter.getString();
     if (!sp.empty()) {
         errln("failed iterator returned garbage data");
     }
 }
 
+void BytesTrieTest::TestDelta() {
+    char intBytes0[5];
+    char intBytes1[5];
+    static constexpr int32_t sampleDeltas[] = {
+        -1, 0, 1, 2, 3, 0xa5, 0xbe, 0xbf,
+        -2, 0xc0, 0xc1, 0xeee, 0x1234, 0x2ffe, 0x2fff,
+        -3, 0x3000, 0x3001, 0x3003, 0x50005, 0xdfffe, 0xdffff,
+        -4, 0xe0000, 0xe0001, 0xef0123, 0xfffffe, 0xffffff,
+        -5, 0x1000000, 0x1000001, 0x7fffffff
+    };
+    int32_t expectedLength = 0;
+    for (int32_t delta : sampleDeltas) {
+        if (delta < 0) {
+            expectedLength = -delta;
+            continue;
+        }
+        // Encoding twice into differently-initialized arrays
+        // catches bytes that are not written to.
+        memset(intBytes0, 0, sizeof(intBytes0));
+        memset(intBytes1, 1, sizeof(intBytes1));
+        int32_t length0 = BytesTrieBuilder::internalEncodeDelta(delta, intBytes0);
+        int32_t length1 = BytesTrieBuilder::internalEncodeDelta(delta, intBytes1);
+        assertTrue(UnicodeString(u"non-zero length to encode delta ") + delta, length0 > 0);
+        assertEquals(UnicodeString(u"consistent length to encode delta ") + delta, length0, length1);
+        assertEquals(UnicodeString(u"expected length to encode delta ") + delta,
+                     expectedLength, length0);
+        for (int32_t i = 0; i < length0; ++i) {
+            uint8_t b0 = intBytes0[i];
+            uint8_t b1 = intBytes1[i];
+            assertEquals(UnicodeString(u"differently encoded delta ") + delta +
+                            u" at byte index " + i, b0, b1);
+        }
+        const uint8_t* start = reinterpret_cast<const uint8_t*>(intBytes0);
+        const uint8_t *pos = BytesTrie::jumpByDelta(start);
+        assertEquals(UnicodeString(u"roundtrip for delta ") + delta,
+                     delta, static_cast<size_t>(pos - start) - length0);
+    }
+}
+
 void BytesTrieTest::checkData(const StringAndValue data[], int32_t dataLength) {
-    logln("checkData(dataLength=%d, fast)", (int)dataLength);
+    logln("checkData(dataLength=%d, fast)", static_cast<int>(dataLength));
     checkData(data, dataLength, USTRINGTRIE_BUILD_FAST);
-    logln("checkData(dataLength=%d, small)", (int)dataLength);
+    logln("checkData(dataLength=%d, small)", static_cast<int>(dataLength));
     checkData(data, dataLength, USTRINGTRIE_BUILD_SMALL);
 }
 
@@ -649,13 +690,13 @@ BytesTrie *BytesTrieTest::buildTrie(const StringAndValue data[], int32_t dataLen
             errln("builder.build().add(zzz) did not set U_NO_WRITE_PERMISSION");
         }
     }
-    logln("serialized trie size: %ld bytes\n", (long)sp.length());
+    logln("serialized trie size: %ld bytes\n", static_cast<long>(sp.length()));
     StringPiece sp2=builder_->buildStringPiece(buildOption, errorCode);
     if(sp.data()==sp2.data()) {
         errln("builder.buildStringPiece() before & after build() returned same array");
     }
     if(errorCode.isFailure()) {
-        return NULL;
+        return nullptr;
     }
     // Tries from either build() method should be identical but
     // BytesTrie does not implement equals().
@@ -702,8 +743,8 @@ void BytesTrieTest::checkNext(BytesTrie &trie,
         } else if(trie.getValue()!=data[i].value) {
             errln("trie value for %s is %ld=0x%lx instead of expected %ld=0x%lx",
                   data[i].s,
-                  (long)trie.getValue(), (long)trie.getValue(),
-                  (long)data[i].value, (long)data[i].value);
+                  static_cast<long>(trie.getValue()), static_cast<long>(trie.getValue()),
+                  static_cast<long>(data[i].value), static_cast<long>(data[i].value));
         } else if(result!=trie.current() || trie.getValue()!=data[i].value) {
             errln("trie value for %s changes when repeating current()/getValue()", data[i].s);
         }
@@ -743,7 +784,7 @@ void BytesTrieTest::checkNext(BytesTrie &trie,
         }
         // Compare the final current() with whether next() can actually continue.
         trie.saveState(state);
-        UBool nextContinues=FALSE;
+        UBool nextContinues=false;
         // Try all graphic characters; we only use those in test strings in this file.
 #if U_CHARSET_FAMILY==U_ASCII_FAMILY
         const int32_t minChar=0x20;
@@ -757,7 +798,7 @@ void BytesTrieTest::checkNext(BytesTrie &trie,
 #endif
         for(int32_t c=minChar; c<=maxChar; ++c) {
             if(trie.resetToState(state).next(c)) {
-                nextContinues=TRUE;
+                nextContinues=true;
                 break;
             }
         }
@@ -820,8 +861,8 @@ void BytesTrieTest::checkNextWithState(BytesTrie &trie,
         } else if(trie.getValue()!=data[i].value) {
             errln("trie value for %s is %ld=0x%lx instead of expected %ld=0x%lx",
                   data[i].s,
-                  (long)trie.getValue(), (long)trie.getValue(),
-                  (long)data[i].value, (long)data[i].value);
+                  static_cast<long>(trie.getValue()), static_cast<long>(trie.getValue()),
+                  static_cast<long>(data[i].value), static_cast<long>(data[i].value));
         }
         trie.reset();
     }
@@ -875,8 +916,8 @@ void BytesTrieTest::checkNextWithState64(BytesTrie &trie,
         } else if(trie.getValue()!=data[i].value) {
             errln("trie value for %s is %ld=0x%lx instead of expected %ld=0x%lx",
                   data[i].s,
-                  (long)trie.getValue(), (long)trie.getValue(),
-                  (long)data[i].value, (long)data[i].value);
+                  static_cast<long>(trie.getValue()), static_cast<long>(trie.getValue()),
+                  static_cast<long>(data[i].value), static_cast<long>(data[i].value));
         }
         trie.reset();
     }
@@ -916,34 +957,34 @@ void BytesTrieTest::checkIterator(BytesTrie::Iterator &iter,
     IcuTestErrorCode errorCode(*this, "checkIterator()");
     for(int32_t i=0; i<dataLength; ++i) {
         if(!iter.hasNext()) {
-            errln("trie iterator hasNext()=FALSE for item %d: %s", (int)i, data[i].s);
+            errln("trie iterator hasNext()=false for item %d: %s", static_cast<int>(i), data[i].s);
             break;
         }
         UBool hasNext=iter.next(errorCode);
-        if(errorCode.errIfFailureAndReset("trie iterator next() for item %d: %s", (int)i, data[i].s)) {
+        if (errorCode.errIfFailureAndReset("trie iterator next() for item %d: %s", static_cast<int>(i), data[i].s)) {
             break;
         }
         if(!hasNext) {
-            errln("trie iterator next()=FALSE for item %d: %s", (int)i, data[i].s);
+            errln("trie iterator next()=false for item %d: %s", static_cast<int>(i), data[i].s);
             break;
         }
         if(iter.getString()!=StringPiece(data[i].s)) {
             errln("trie iterator next().getString()=%s but expected %s for item %d",
-                  iter.getString().data(), data[i].s, (int)i);
+                  iter.getString().data(), data[i].s, static_cast<int>(i));
         }
         if(iter.getValue()!=data[i].value) {
             errln("trie iterator next().getValue()=%ld=0x%lx but expected %ld=0x%lx for item %d: %s",
-                  (long)iter.getValue(), (long)iter.getValue(),
-                  (long)data[i].value, (long)data[i].value,
-                  (int)i, data[i].s);
+                  static_cast<long>(iter.getValue()), static_cast<long>(iter.getValue()),
+                  static_cast<long>(data[i].value), static_cast<long>(data[i].value),
+                  static_cast<int>(i), data[i].s);
         }
     }
     if(iter.hasNext()) {
-        errln("trie iterator hasNext()=TRUE after all items");
+        errln("trie iterator hasNext()=true after all items");
     }
     UBool hasNext=iter.next(errorCode);
     errorCode.errIfFailureAndReset("trie iterator next() after all items");
     if(hasNext) {
-        errln("trie iterator next()=TRUE after all items");
+        errln("trie iterator next()=true after all items");
     }
 }
