@@ -68,8 +68,8 @@ class ConversionTest : public IntlTest {
 public:
     ConversionTest();
     virtual ~ConversionTest();
-    
-    void runIndexedTest(int32_t index, UBool exec, const char *&name, char *par=0) override;
+
+    void runIndexedTest(int32_t index, UBool exec, const char *&name, char *par = nullptr) override;
 
     void TestToUnicode();
     void TestFromUnicode();
