@@ -1,3 +1,5 @@
+// © 2016 and later: Unicode, Inc. and others.
+// License & terms of use: http://www.unicode.org/copyright.html
 /********************************************************************
  * COPYRIGHT: 
  * Copyright (c) 1997-2006, International Business Machines Corporation and
@@ -23,19 +25,19 @@
 class DateFormatRoundTripTest : public IntlTest {    
     
     // IntlTest override
-    void runIndexedTest( int32_t index, UBool exec, const char* &name, char* par );
+    void runIndexedTest( int32_t index, UBool exec, const char* &name, char* par ) override;
 
 public:
     DateFormatRoundTripTest();
     virtual ~DateFormatRoundTripTest();
     
-    void TestDateFormatRoundTrip(void);
-    void TestCentury(void);
+    void TestDateFormatRoundTrip();
+    void TestCentury();
     void test(const Locale& loc);
-    void test(DateFormat *fmt, const Locale &origLocale, UBool timeOnly = FALSE );
+    void test(DateFormat *fmt, const Locale &origLocale, UBool timeOnly = false );
     int32_t getField(UDate d, int32_t f);
     UnicodeString& escape(const UnicodeString& src, UnicodeString& dst);
-    UDate generateDate(void); 
+    UDate generateDate(); 
     UDate generateDate(UDate minDate); 
 
 
@@ -49,8 +51,8 @@ public:
 static uint32_t randLong() {
     // The portable IntlTest::random() function has sufficient
     // resolution for a 16-bit value, but not for 32 bits.
-    return ((uint32_t) (IntlTest::random() * (1<<16))) |
-          (((uint32_t) (IntlTest::random() * (1<<16))) << 16);
+    return static_cast<uint32_t>(IntlTest::random() * (1 << 16)) |
+           (static_cast<uint32_t>(IntlTest::random() * (1 << 16)) << 16);
 }
 
 /**
@@ -58,7 +60,7 @@ static uint32_t randLong() {
  **/
 static double randFraction()
 {
-    return (double)randLong() / (double)0xFFFFFFFF;
+    return static_cast<double>(randLong()) / static_cast<double>(0xFFFFFFFF);
 }
 
 /**
@@ -84,7 +86,7 @@ private:
     static int32_t TRIALS;
     static int32_t DEPTH;
 
-    UBool optionv; // TRUE if @v option is given on command line
+    UBool optionv; // true if @v option is given on command line
     SimpleDateFormat *dateFormat;
     UnicodeString fgStr;
     Calendar *getFieldCal;
