@@ -46,6 +46,7 @@ extern IntlTest *createPluralMapTest();
 #if !UCONFIG_NO_FORMATTING
 extern IntlTest *createStaticUnicodeSetsTest();
 #endif
+static IntlTest *createUHashTest();
 
 void IntlTestUtilities::runIndexedTest( int32_t index, UBool exec, const char* &name, char* par )
 {
@@ -80,6 +81,7 @@ void IntlTestUtilities::runIndexedTest( int32_t index, UBool exec, const char* &
 #endif
     TESTCASE_AUTO_CLASS(LocaleBuilderTest);
     TESTCASE_AUTO_CREATE_CLASS(LocaleMatcherTest);
+    TESTCASE_AUTO_CREATE_CLASS(UHashTest);
     TESTCASE_AUTO_END;
 }
 
@@ -93,8 +95,8 @@ void ErrorCodeTest::runIndexedTest(int32_t index, UBool exec, const char* &name,
     }
 }
 
-static void RefPlusOne(UErrorCode &code) { code=(UErrorCode)(code+1); }
-static void PtrPlusTwo(UErrorCode *code) { *code=(UErrorCode)(*code+2); }
+static void RefPlusOne(UErrorCode &code) { code = static_cast<UErrorCode>(code + 1); }
+static void PtrPlusTwo(UErrorCode *code) { *code = static_cast<UErrorCode>(*code + 2); }
 
 void ErrorCodeTest::TestErrorCode() {
     ErrorCode errorCode;
@@ -202,7 +204,7 @@ void ErrorCodeTest::TestSubclass() {
 
 class IcuTestErrorCodeTestHelper : public IntlTest {
   public:
-    void errln( const UnicodeString &message ) U_OVERRIDE {
+    void errln( const UnicodeString &message ) override {
         test->assertFalse("Already saw an error", seenError);
         seenError = true;
         test->assertEquals("Message for Error", expectedErrln, message);
@@ -211,7 +213,7 @@ class IcuTestErrorCodeTestHelper : public IntlTest {
         }
     }
 
-    void dataerrln( const UnicodeString &message ) U_OVERRIDE {
+    void dataerrln( const UnicodeString &message ) override {
         test->assertFalse("Already saw an error", seenError);
         seenError = true;
         test->assertEquals("Message for Error", expectedErrln, message);
@@ -312,7 +314,7 @@ class LocalPointerTest : public IntlTest {
 public:
     LocalPointerTest() {}
 
-    void runIndexedTest(int32_t index, UBool exec, const char *&name, char *par=NULL) override;
+    void runIndexedTest(int32_t index, UBool exec, const char *&name, char *par=nullptr) override;
 
     void TestLocalPointer();
     void TestLocalPointerMoveSwap();
@@ -351,10 +353,10 @@ void LocalPointerTest::runIndexedTest(int32_t index, UBool exec, const char *&na
 // Exercise almost every LocalPointer and LocalPointerBase method.
 void LocalPointerTest::TestLocalPointer() {
     // constructor
-    LocalPointer<UnicodeString> s(new UnicodeString((UChar32)0x50005));
+    LocalPointer<UnicodeString> s(new UnicodeString(static_cast<UChar32>(0x50005)));
     // isNULL(), isValid(), operator==(), operator!=()
-    if(s.isNull() || !s.isValid() || s==NULL || !(s!=NULL)) {
-        errln("LocalPointer constructor or NULL test failure");
+    if(s.isNull() || !s.isValid() || s==nullptr || !(s!=nullptr)) {
+        errln("LocalPointer constructor or nullptr test failure");
         return;
     }
     // getAlias(), operator->, operator*
@@ -362,12 +364,12 @@ void LocalPointerTest::TestLocalPointer() {
         errln("LocalPointer access failure");
     }
     // adoptInstead(), orphan()
-    s.adoptInstead(new UnicodeString((UChar)0xfffc));
+    s.adoptInstead(new UnicodeString(static_cast<char16_t>(0xfffc)));
     if(s->length()!=1) {
         errln("LocalPointer adoptInstead(U+FFFC) failure");
     }
     UnicodeString *orphan=s.orphan();
-    if(orphan==NULL || orphan->length()!=1 || s.isValid() || s!=NULL) {
+    if(orphan==nullptr || orphan->length()!=1 || s.isValid() || s!=nullptr) {
         errln("LocalPointer orphan() failure");
     }
     delete orphan;
@@ -376,7 +378,7 @@ void LocalPointerTest::TestLocalPointer() {
         errln("LocalPointer adoptInstead(empty) failure");
     }
 
-    // LocalPointer(p, errorCode) sets U_MEMORY_ALLOCATION_ERROR if p==NULL.
+    // LocalPointer(p, errorCode) sets U_MEMORY_ALLOCATION_ERROR if p==nullptr.
     UErrorCode errorCode = U_ZERO_ERROR;
     LocalPointer<CharString> csx(new CharString("some chars", errorCode), errorCode);
     if(csx.isNull() && U_SUCCESS(errorCode)) {
@@ -397,19 +399,19 @@ void LocalPointerTest::TestLocalPointer() {
         return;
     }
     errorCode = U_ZERO_ERROR;
-    csx.adoptInsteadAndCheckErrorCode(NULL, errorCode);
+    csx.adoptInsteadAndCheckErrorCode(nullptr, errorCode);
     if(errorCode != U_MEMORY_ALLOCATION_ERROR) {
-        errln("adoptInsteadAndCheckErrorCode(NULL, errorCode) did not set U_MEMORY_ALLOCATION_ERROR");
+        errln("adoptInsteadAndCheckErrorCode(nullptr, errorCode) did not set U_MEMORY_ALLOCATION_ERROR");
         return;
     }
     if(csx.isValid()) {
-        errln("adoptInsteadAndCheckErrorCode(NULL, errorCode) kept the object");
+        errln("adoptInsteadAndCheckErrorCode(nullptr, errorCode) kept the object");
         return;
     }
     errorCode = U_ZERO_ERROR;
-    LocalPointer<CharString> null(NULL, errorCode);
+    LocalPointer<CharString> null(nullptr, errorCode);
     if(errorCode != U_MEMORY_ALLOCATION_ERROR) {
-        errln("LocalPointer(NULL, errorCode) did not set U_MEMORY_ALLOCATION_ERROR");
+        errln("LocalPointer(nullptr, errorCode) did not set U_MEMORY_ALLOCATION_ERROR");
         return;
     }
 
@@ -423,8 +425,8 @@ void moveFrom(T &dest, T &src) {
 }
 
 void LocalPointerTest::TestLocalPointerMoveSwap() {
-    UnicodeString *p1 = new UnicodeString((UChar)0x61);
-    UnicodeString *p2 = new UnicodeString((UChar)0x62);
+    UnicodeString* p1 = new UnicodeString(static_cast<char16_t>(0x61));
+    UnicodeString* p2 = new UnicodeString(static_cast<char16_t>(0x62));
     LocalPointer<UnicodeString> s1(p1);
     LocalPointer<UnicodeString> s2(p2);
     s1.swap(s2);
@@ -458,7 +460,7 @@ void LocalPointerTest::TestLocalPointerMoveSwap() {
 }
 
 void LocalPointerTest::TestLocalPointerStdUniquePtr() {
-    auto* ptr = new UnicodeString((UChar32)0x50005);
+    auto* ptr = new UnicodeString(static_cast<UChar32>(0x50005));
     // Implicit conversion operator
     std::unique_ptr<UnicodeString> s = LocalPointer<UnicodeString>(ptr);
     // Explicit move constructor
@@ -475,19 +477,19 @@ void LocalPointerTest::TestLocalArray() {
     // constructor
     LocalArray<UnicodeString> a(new UnicodeString[2]);
     // operator[]()
-    a[0].append((UChar)0x61);
-    a[1].append((UChar32)0x60006);
+    a[0].append(static_cast<char16_t>(0x61));
+    a[1].append(static_cast<UChar32>(0x60006));
     if(a[0].length()!=1 || a[1].length()!=2) {
         errln("LocalArray access failure");
     }
     // adoptInstead()
     a.adoptInstead(new UnicodeString[4]);
-    a[3].append((UChar)0x62).append((UChar)0x63).reverse();
+    a[3].append(static_cast<char16_t>(0x62)).append(static_cast<char16_t>(0x63)).reverse();
     if(a[3].length()!=2 || a[3][1]!=0x62) {
         errln("LocalArray adoptInstead() failure");
     }
 
-    // LocalArray(p, errorCode) sets U_MEMORY_ALLOCATION_ERROR if p==NULL.
+    // LocalArray(p, errorCode) sets U_MEMORY_ALLOCATION_ERROR if p==nullptr.
     UErrorCode errorCode = U_ZERO_ERROR;
     LocalArray<UnicodeString> ua(new UnicodeString[3], errorCode);
     if(ua.isNull() && U_SUCCESS(errorCode)) {
@@ -509,19 +511,19 @@ void LocalPointerTest::TestLocalArray() {
         return;
     }
     errorCode = U_ZERO_ERROR;
-    ua.adoptInsteadAndCheckErrorCode(NULL, errorCode);
+    ua.adoptInsteadAndCheckErrorCode(nullptr, errorCode);
     if(errorCode != U_MEMORY_ALLOCATION_ERROR) {
-        errln("adoptInsteadAndCheckErrorCode(NULL, errorCode) did not set U_MEMORY_ALLOCATION_ERROR");
+        errln("adoptInsteadAndCheckErrorCode(nullptr, errorCode) did not set U_MEMORY_ALLOCATION_ERROR");
         return;
     }
     if(ua.isValid()) {
-        errln("adoptInsteadAndCheckErrorCode(NULL, errorCode) kept the array");
+        errln("adoptInsteadAndCheckErrorCode(nullptr, errorCode) kept the array");
         return;
     }
     errorCode = U_ZERO_ERROR;
-    LocalArray<UnicodeString> null(NULL, errorCode);
+    LocalArray<UnicodeString> null(nullptr, errorCode);
     if(errorCode != U_MEMORY_ALLOCATION_ERROR) {
-        errln("LocalArray(NULL, errorCode) did not set U_MEMORY_ALLOCATION_ERROR");
+        errln("LocalArray(nullptr, errorCode) did not set U_MEMORY_ALLOCATION_ERROR");
         return;
     }
 
@@ -593,7 +595,7 @@ void LocalPointerTest::TestLocalXyzPointer() {
 
     static const char *const encoding="ISO-8859-1";
     LocalUConverterSelectorPointer sel(
-        ucnvsel_open(&encoding, 1, NULL, UCNV_ROUNDTRIP_SET, errorCode));
+        ucnvsel_open(&encoding, 1, nullptr, UCNV_ROUNDTRIP_SET, errorCode));
     if(errorCode.errIfFailureAndReset("ucnvsel_open()")) {
         return;
     }
@@ -603,7 +605,7 @@ void LocalPointerTest::TestLocalXyzPointer() {
     }
 
 #if !UCONFIG_NO_FORMATTING
-    LocalUCalendarPointer cal(ucal_open(NULL, 0, "root", UCAL_GREGORIAN, errorCode));
+    LocalUCalendarPointer cal(ucal_open(nullptr, 0, "root", UCAL_GREGORIAN, errorCode));
     if(errorCode.errDataIfFailureAndReset("ucal_open()")) {
         return;
     }
@@ -632,7 +634,7 @@ void LocalPointerTest::TestLocalXyzPointer() {
 
     UnicodeString hello=UNICODE_STRING_SIMPLE("Hello {0}!");
     LocalUMessageFormatPointer msg(
-        umsg_open(hello.getBuffer(), hello.length(), "root", NULL, errorCode));
+        umsg_open(hello.getBuffer(), hello.length(), "root", nullptr, errorCode));
     if(errorCode.errIfFailureAndReset("umsg_open()")) {
         return;
     }
@@ -669,7 +671,7 @@ void LocalPointerTest::TestLocalXyzPointer() {
 #if !UCONFIG_NO_REGULAR_EXPRESSIONS
     UnicodeString pattern=UNICODE_STRING_SIMPLE("abc|xy+z");
     LocalURegularExpressionPointer regex(
-        uregex_open(pattern.getBuffer(), pattern.length(), 0, NULL, errorCode));
+        uregex_open(pattern.getBuffer(), pattern.length(), 0, nullptr, errorCode));
     if(errorCode.errIfFailureAndReset("uregex_open()")) {
         return;
     }
@@ -682,7 +684,7 @@ void LocalPointerTest::TestLocalXyzPointer() {
 #if !UCONFIG_NO_TRANSLITERATION
     UnicodeString id=UNICODE_STRING_SIMPLE("Grek-Latn");
     LocalUTransliteratorPointer trans(
-        utrans_openU(id.getBuffer(), id.length(), UTRANS_FORWARD, NULL, 0, NULL, errorCode));
+        utrans_openU(id.getBuffer(), id.length(), UTRANS_FORWARD, nullptr, 0, nullptr, errorCode));
     if(errorCode.errIfFailureAndReset("utrans_open()")) {
         return;
     }
@@ -745,38 +747,38 @@ void LocalPointerTest::TestLocalXyzPointerMoveSwap() {
 #endif /* !UCONFIG_NO_NORMALIZATION */
 }
 
-// Try LocalXyzPointer types with NULL pointers.
+// Try LocalXyzPointer types with nullptr pointers.
 void LocalPointerTest::TestLocalXyzPointerNull() {
     {
         IcuTestErrorCode errorCode(*this, "TestLocalXyzPointerNull/LocalUConverterSelectorPointer");
         static const char *const encoding="ISO-8859-1";
         LocalUConverterSelectorPointer null;
         LocalUConverterSelectorPointer sel(
-            ucnvsel_open(&encoding, 1, NULL, UCNV_ROUNDTRIP_SET, errorCode));
-        sel.adoptInstead(NULL);
+            ucnvsel_open(&encoding, 1, nullptr, UCNV_ROUNDTRIP_SET, errorCode));
+        sel.adoptInstead(nullptr);
     }
 #if !UCONFIG_NO_FORMATTING
     {
         IcuTestErrorCode errorCode(*this, "TestLocalXyzPointerNull/LocalUCalendarPointer");
         LocalUCalendarPointer null;
-        LocalUCalendarPointer cal(ucal_open(NULL, 0, "root", UCAL_GREGORIAN, errorCode));
+        LocalUCalendarPointer cal(ucal_open(nullptr, 0, "root", UCAL_GREGORIAN, errorCode));
         if(!errorCode.errDataIfFailureAndReset("ucal_open()")) {
-            cal.adoptInstead(NULL);
+            cal.adoptInstead(nullptr);
         }
     }
     {
         IcuTestErrorCode errorCode(*this, "TestLocalXyzPointerNull/LocalUDateTimePatternGeneratorPointer");
         LocalUDateTimePatternGeneratorPointer null;
         LocalUDateTimePatternGeneratorPointer patgen(udatpg_open("root", errorCode));
-        patgen.adoptInstead(NULL);
+        patgen.adoptInstead(nullptr);
     }
     {
         IcuTestErrorCode errorCode(*this, "TestLocalXyzPointerNull/LocalUMessageFormatPointer");
         UnicodeString hello=UNICODE_STRING_SIMPLE("Hello {0}!");
         LocalUMessageFormatPointer null;
         LocalUMessageFormatPointer msg(
-            umsg_open(hello.getBuffer(), hello.length(), "root", NULL, errorCode));
-        msg.adoptInstead(NULL);
+            umsg_open(hello.getBuffer(), hello.length(), "root", nullptr, errorCode));
+        msg.adoptInstead(nullptr);
     }
 #endif /* !UCONFIG_NO_FORMATTING */
 
@@ -786,9 +788,9 @@ void LocalPointerTest::TestLocalXyzPointerNull() {
         UnicodeString pattern=UNICODE_STRING_SIMPLE("abc|xy+z");
         LocalURegularExpressionPointer null;
         LocalURegularExpressionPointer regex(
-            uregex_open(pattern.getBuffer(), pattern.length(), 0, NULL, errorCode));
+            uregex_open(pattern.getBuffer(), pattern.length(), 0, nullptr, errorCode));
         if(!errorCode.errDataIfFailureAndReset("urege_open()")) {
-            regex.adoptInstead(NULL);
+            regex.adoptInstead(nullptr);
         }
     }
 #endif /* !UCONFIG_NO_REGULAR_EXPRESSIONS */
@@ -799,9 +801,9 @@ void LocalPointerTest::TestLocalXyzPointerNull() {
         UnicodeString id=UNICODE_STRING_SIMPLE("Grek-Latn");
         LocalUTransliteratorPointer null;
         LocalUTransliteratorPointer trans(
-            utrans_openU(id.getBuffer(), id.length(), UTRANS_FORWARD, NULL, 0, NULL, errorCode));
+            utrans_openU(id.getBuffer(), id.length(), UTRANS_FORWARD, nullptr, 0, nullptr, errorCode));
         if(!errorCode.errDataIfFailureAndReset("utrans_openU()")) {
-            trans.adoptInstead(NULL);
+            trans.adoptInstead(nullptr);
         }
     }
 #endif /* !UCONFIG_NO_TRANSLITERATION */
@@ -831,7 +833,7 @@ void LocalPointerTest::TestLocalXyzStdUniquePtr() {
 class EnumSetTest : public IntlTest {
 public:
   EnumSetTest() {}
-  virtual void runIndexedTest(int32_t index, UBool exec, const char *&name, char *par=NULL) override;
+  virtual void runIndexedTest(int32_t index, UBool exec, const char *&name, char *par=nullptr) override;
   void TestEnumSet();
 };
 
@@ -905,3 +907,45 @@ void EnumSetTest::TestEnumSet() {
     assertFalse(WHERE, flags.get(THING2));
     assertFalse(WHERE, flags.get(THING3));
 }
+
+/** UHashTest **/
+#include "uhash.h"
+#include <string_view>
+
+class UHashTest : public IntlTest {
+public:
+    UHashTest() = default;
+    virtual void runIndexedTest(int32_t index, UBool exec, const char*& name, char* par = nullptr) override;
+    void TestStringView();
+};
+
+static IntlTest* createUHashTest() {
+    return new UHashTest();
+}
+
+void UHashTest::runIndexedTest(int32_t index, UBool exec, const char*& name, char*  /*par*/) {
+    TESTCASE_AUTO_BEGIN;
+    TESTCASE_AUTO(TestStringView);
+    TESTCASE_AUTO_END;
+}
+
+void UHashTest::TestStringView() {
+    IcuTestErrorCode status(*this, "TestStringView");
+    LocalUHashtablePointer table(
+        uhash_open(uhash_hashIStringView, uhash_compareIStringView, nullptr, status));
+    if (status.errIfFailureAndReset("uhash_open") ||
+        !assertTrue("uhash_open", table.isValid())) {
+        return;
+    }
+
+    std::string_view key("aaa");
+    std::string_view value("bbb");
+
+    uhash_put(table.getAlias(), &key, &value, status);
+    if (status.errIfFailureAndReset("uhash_put")) return;
+
+    std::string_view lookup("AAA");
+
+    auto* result = static_cast<std::string_view*>(uhash_get(table.getAlias(), &lookup));
+    assertTrue("uhash_get", result == &value);
+}
