@@ -17,7 +17,7 @@
 
 class PluralMapForPluralMapTest : public PluralMap<UnicodeString> {
 public:
-    bool operator==(const PluralMapForPluralMapTest &other) {
+    bool operator==(const PluralMapForPluralMapTest &other) const {
         return equals(other, strEqual);
     }
 private:
@@ -36,7 +36,7 @@ public:
     void TestIterate();
     void TestEqual();
     void TestCopyAndAssign();
-    void runIndexedTest(int32_t index, UBool exec, const char *&name, char *par=0) override;
+    void runIndexedTest(int32_t index, UBool exec, const char*& name, char* par = nullptr) override;
     void addVariant(
             PluralMapBase::Category v,
             const UnicodeString &value,
@@ -56,24 +56,24 @@ void PluralMapTest::runIndexedTest(int32_t index, UBool exec, const char* &name,
 }
 
 void PluralMapTest::TestToCategory() {
-    assertEquals("", (int32_t)PluralMapBase::OTHER, PluralMapBase::toCategory("other"));
-    assertEquals("", (int32_t)PluralMapBase::ZERO, PluralMapBase::toCategory("zero"));
-    assertEquals("", (int32_t)PluralMapBase::ONE, PluralMapBase::toCategory("one"));
-    assertEquals("", (int32_t)PluralMapBase::TWO, PluralMapBase::toCategory("two"));
-    assertEquals("", (int32_t)PluralMapBase::FEW, PluralMapBase::toCategory("few"));
-    assertEquals("", (int32_t)PluralMapBase::MANY, PluralMapBase::toCategory("many"));
-    assertEquals("", (int32_t)PluralMapBase::NONE, PluralMapBase::toCategory("Many"));
+    assertEquals("", static_cast<int32_t>(PluralMapBase::OTHER), PluralMapBase::toCategory("other"));
+    assertEquals("", static_cast<int32_t>(PluralMapBase::ZERO), PluralMapBase::toCategory("zero"));
+    assertEquals("", static_cast<int32_t>(PluralMapBase::ONE), PluralMapBase::toCategory("one"));
+    assertEquals("", static_cast<int32_t>(PluralMapBase::TWO), PluralMapBase::toCategory("two"));
+    assertEquals("", static_cast<int32_t>(PluralMapBase::FEW), PluralMapBase::toCategory("few"));
+    assertEquals("", static_cast<int32_t>(PluralMapBase::MANY), PluralMapBase::toCategory("many"));
+    assertEquals("", static_cast<int32_t>(PluralMapBase::NONE), PluralMapBase::toCategory("Many"));
     assertEquals(
             "",
-            (int32_t)PluralMapBase::FEW,
+            static_cast<int32_t>(PluralMapBase::FEW),
             PluralMapBase::toCategory(UnicodeString("few")));
     assertEquals(
             "",
-            (int32_t)PluralMapBase::MANY,
+            static_cast<int32_t>(PluralMapBase::MANY),
             PluralMapBase::toCategory(UnicodeString("many")));
     assertEquals(
             "",
-            (int32_t)PluralMapBase::NONE,
+            static_cast<int32_t>(PluralMapBase::NONE),
             PluralMapBase::toCategory(UnicodeString("Many")));
 }
 
@@ -117,24 +117,24 @@ void PluralMapTest::TestIterate() {
     PluralMapBase::Category index = PluralMapBase::NONE;
     const UnicodeString *current = map.next(index);
     assertEquals("", "pickles", *current);
-    assertEquals("", (int32_t)PluralMapBase::OTHER, index);
+    assertEquals("", static_cast<int32_t>(PluralMapBase::OTHER), index);
     current = map.next(index);
     assertEquals("", "pickle", *current);
-    assertEquals("", (int32_t)PluralMapBase::ONE, index);
+    assertEquals("", static_cast<int32_t>(PluralMapBase::ONE), index);
     current = map.next(index);
     assertEquals("", "picklefew", *current);
-    assertEquals("", (int32_t)PluralMapBase::FEW, index);
+    assertEquals("", static_cast<int32_t>(PluralMapBase::FEW), index);
     current = map.next(index);
-    assertEquals("", (int32_t)PluralMapBase::CATEGORY_COUNT, index);
+    assertEquals("", static_cast<int32_t>(PluralMapBase::CATEGORY_COUNT), index);
     assertTrue("", current == nullptr);
 
     PluralMapForPluralMapTest map2;
     index = PluralMapBase::NONE;
     current = map2.next(index);
     assertEquals("", "", *current);
-    assertEquals("", (int32_t)PluralMapBase::OTHER, index);
+    assertEquals("", static_cast<int32_t>(PluralMapBase::OTHER), index);
     current = map2.next(index);
-    assertEquals("", (int32_t)PluralMapBase::CATEGORY_COUNT, index);
+    assertEquals("", static_cast<int32_t>(PluralMapBase::CATEGORY_COUNT), index);
     assertTrue("", current == nullptr);
 }
 
