@@ -36,7 +36,8 @@ public:
     void TestFormatReplaceOptimizationNoOffsets();
     void TestFormatReplaceNoOptimizationNoOffsets();
     void TestQuotingLikeMessageFormat();
-    void runIndexedTest(int32_t index, UBool exec, const char *&name, char *par=0);
+    void runIndexedTest(int32_t index, UBool exec, const char*& name, char* par = nullptr) override;
+
 private:
     void verifyOffsets(
             const int32_t *expected,
@@ -78,22 +79,22 @@ void SimpleFormatterTest::TestNoArguments() {
     assertEquals(
             "formatAndAppend",
             "This doesn't have templates {0}", 
-            fmt.formatAndAppend(NULL, 0, appendTo, offsets, 1, status));
+            fmt.formatAndAppend(nullptr, 0, appendTo, offsets, 1, status));
     assertEquals("formatAndAppend offsets[0]", -1, offsets[0]);
     assertEquals(
             "formatAndReplace",
             "This doesn't have templates {0}", 
-            fmt.formatAndReplace(NULL, 0, appendTo, NULL, 0, status));
+            fmt.formatAndReplace(nullptr, 0, appendTo, nullptr, 0, status));
     assertSuccess("Status", status);
 }
 
 void SimpleFormatterTest::TestSyntaxErrors() {
     UErrorCode status = U_ZERO_ERROR;
     SimpleFormatter fmt("{}", status);
-    assertEquals("syntax error {}", (int32_t)U_ILLEGAL_ARGUMENT_ERROR, status);
+    assertEquals("syntax error {}", static_cast<int32_t>(U_ILLEGAL_ARGUMENT_ERROR), status);
     status = U_ZERO_ERROR;
     fmt.applyPattern("{12d", status);
-    assertEquals("syntax error {12d", (int32_t)U_ILLEGAL_ARGUMENT_ERROR, status);
+    assertEquals("syntax error {12d", static_cast<int32_t>(U_ILLEGAL_ARGUMENT_ERROR), status);
 }
 
 void SimpleFormatterTest::TestOneArgument() {
@@ -138,12 +139,12 @@ void SimpleFormatterTest::TestBigArgument() {
     assertEquals("{20} count", 21, fmt.getArgumentLimit());
     UnicodeString b("b");
     UnicodeString *values[] = {
-        NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
-        NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
+        nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr,
+        nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr,
         &b
     };
     UnicodeString result;
-    assertEquals("{20}=b", "abc", fmt.formatAndAppend(values, 21, result, NULL, 0, status));
+    assertEquals("{20}=b", "abc", fmt.formatAndAppend(values, 21, result, nullptr, 0, status));
     assertSuccess("Status", status);
 }
 
@@ -203,7 +204,7 @@ void SimpleFormatterTest::TestManyArguments() {
                     params,
                     UPRV_LENGTHOF(params),
                     appendTo,
-                    NULL,
+                    nullptr,
                     0,
                     status));
 
@@ -217,7 +218,7 @@ void SimpleFormatterTest::TestManyArguments() {
                     params,
                     UPRV_LENGTHOF(params),
                     appendTo,
-                    NULL,
+                    nullptr,
                     0,
                     status));
     r.applyPattern("{0} meter", status);
@@ -259,14 +260,14 @@ void SimpleFormatterTest::TestTooFewArgumentValues() {
 
     status = U_ZERO_ERROR;
     fmt.formatAndAppend(
-            params, UPRV_LENGTHOF(params), appendTo, NULL, 0, status);
+            params, UPRV_LENGTHOF(params), appendTo, nullptr, 0, status);
     if (status != U_ILLEGAL_ARGUMENT_ERROR) {
         errln("Expected U_ILLEGAL_ARGUMENT_ERROR");
     }
 
     status = U_ZERO_ERROR;
     fmt.formatAndReplace(
-            params, UPRV_LENGTHOF(params), appendTo, NULL, 0, status);
+            params, UPRV_LENGTHOF(params), appendTo, nullptr, 0, status);
     if (status != U_ILLEGAL_ARGUMENT_ERROR) {
         errln("Expected U_ILLEGAL_ARGUMENT_ERROR");
     }
@@ -279,25 +280,25 @@ void SimpleFormatterTest::TestBadArguments() {
 
     // These succeed
     fmt.formatAndAppend(
-            NULL, 0, appendTo, NULL, 0, status);
+            nullptr, 0, appendTo, nullptr, 0, status);
     fmt.formatAndReplace(
-            NULL, 0, appendTo, NULL, 0, status);
+            nullptr, 0, appendTo, nullptr, 0, status);
     assertSuccess("", status);
     status = U_ZERO_ERROR;
 
     // fails
     fmt.formatAndAppend(
-            NULL, 1, appendTo, NULL, 0, status);
+            nullptr, 1, appendTo, nullptr, 0, status);
     if (status != U_ILLEGAL_ARGUMENT_ERROR) {
-        errln("Expected U_ILLEGAL_ARGUMENT_ERROR: formatAndAppend() values=NULL but length=1");
+        errln("Expected U_ILLEGAL_ARGUMENT_ERROR: formatAndAppend() values=nullptr but length=1");
     }
     status = U_ZERO_ERROR;
    
     // fails
     fmt.formatAndAppend(
-            NULL, 0, appendTo, NULL, 1, status);
+            nullptr, 0, appendTo, nullptr, 1, status);
     if (status != U_ILLEGAL_ARGUMENT_ERROR) {
-        errln("Expected U_ILLEGAL_ARGUMENT_ERROR: formatAndAppend() offsets=NULL but length=1");
+        errln("Expected U_ILLEGAL_ARGUMENT_ERROR: formatAndAppend() offsets=nullptr but length=1");
     }
     status = U_ZERO_ERROR;
 
@@ -305,7 +306,7 @@ void SimpleFormatterTest::TestBadArguments() {
     SimpleFormatter fmt2("Arguments {0} and {1}", status);
     UnicodeString frog("frog");
     const UnicodeString *params[] = { &appendTo, &frog };
-    fmt2.formatAndAppend(params, 2, appendTo, NULL, 0, status);
+    fmt2.formatAndAppend(params, 2, appendTo, nullptr, 0, status);
     if (status != U_ILLEGAL_ARGUMENT_ERROR) {
         errln("Expected U_ILLEGAL_ARGUMENT_ERROR: formatAndAppend() value=appendTo");
     }
@@ -314,17 +315,17 @@ void SimpleFormatterTest::TestBadArguments() {
    
     // fails
     fmt.formatAndReplace(
-            NULL, 1, appendTo, NULL, 0, status);
+            nullptr, 1, appendTo, nullptr, 0, status);
     if (status != U_ILLEGAL_ARGUMENT_ERROR) {
-        errln("Expected U_ILLEGAL_ARGUMENT_ERROR: formatAndReplace() values=NULL but length=1");
+        errln("Expected U_ILLEGAL_ARGUMENT_ERROR: formatAndReplace() values=nullptr but length=1");
     }
     status = U_ZERO_ERROR;
    
     // fails
     fmt.formatAndReplace(
-            NULL, 0, appendTo, NULL, 1, status);
+            nullptr, 0, appendTo, nullptr, 1, status);
     if (status != U_ILLEGAL_ARGUMENT_ERROR) {
-        errln("Expected U_ILLEGAL_ARGUMENT_ERROR: formatAndReplace() offsets=NULL but length=1");
+        errln("Expected U_ILLEGAL_ARGUMENT_ERROR: formatAndReplace() offsets=nullptr but length=1");
     }
 }
 
@@ -485,7 +486,7 @@ void SimpleFormatterTest::TestFormatReplaceOptimizationNoOffsets() {
                     params,
                     UPRV_LENGTHOF(params),
                     result,
-                    NULL,
+                    nullptr,
                     0,
                     status));
     assertSuccess("Status", status);
@@ -504,7 +505,7 @@ void SimpleFormatterTest::TestFormatReplaceNoOptimizationNoOffsets() {
                     params,
                     UPRV_LENGTHOF(params),
                     result,
-                    NULL,
+                    nullptr,
                     0,
                     status));
     assertSuccess("Status", status);
