@@ -278,10 +278,10 @@ void TestUtility::checkEditsIter(
     std::reverse(destIndexes.begin(), destIndexes.end());
     // Zig-zag across the indexes to stress next() <-> previous().
     static const int32_t ZIG_ZAG[] = { 0, 1, 2, 3, 2, 1 };
-    for (auto i = 0; i < (int32_t)srcIndexes.size(); ++i) {
+    for (auto i = 0; i < static_cast<int32_t>(srcIndexes.size()); ++i) {
         for (int32_t ij = 0; ij < UPRV_LENGTHOF(ZIG_ZAG); ++ij) {
             int32_t j = ZIG_ZAG[ij];
-            if ((i + j) < (int32_t)srcIndexes.size()) {
+            if ((i + j) < static_cast<int32_t>(srcIndexes.size())) {
                 int32_t si = srcIndexes[i + j];
                 test.assertEquals(name + u" destIndexFromSrc(" + si + u"):" + __LINE__,
                                   destIndexFromSrc(expected, expLength, srcLength, destLength, si),
@@ -289,10 +289,10 @@ void TestUtility::checkEditsIter(
             }
         }
     }
-    for (auto i = 0; i < (int32_t)destIndexes.size(); ++i) {
+    for (auto i = 0; i < static_cast<int32_t>(destIndexes.size()); ++i) {
         for (int32_t ij = 0; ij < UPRV_LENGTHOF(ZIG_ZAG); ++ij) {
             int32_t j = ZIG_ZAG[ij];
-            if ((i + j) < (int32_t)destIndexes.size()) {
+            if ((i + j) < static_cast<int32_t>(destIndexes.size())) {
                 int32_t di = destIndexes[i + j];
                 test.assertEquals(name + u" srcIndexFromDest(" + di + u"):" + __LINE__,
                                   srcIndexFromDest(expected, expLength, srcLength, destLength, di),
