@@ -57,8 +57,8 @@ CMAPMapper *CMAPMapper::createUnicodeMapper(const CMAPTable *cmap)
 {
     le_uint16 i;
     le_uint16 nSubtables = SWAPW(cmap->numberSubtables);
-    const CMAPEncodingSubtable *subtable = NULL;
-    le_bool found = FALSE;
+    const CMAPEncodingSubtable *subtable = nullptr;
+    le_bool found = false;
     le_uint16 foundPlatformID = 0xFFFF;
     le_uint16 foundPlatformSpecificID = 0xFFFF;
     le_uint32 foundOffset = 0;
@@ -76,7 +76,7 @@ CMAPMapper *CMAPMapper::createUnicodeMapper(const CMAPTable *cmap)
                 foundOffset = SWAPL(esh->encodingOffset);
                 foundPlatformID = platformID;
                 foundPlatformSpecificID = platformSpecificID;
-                found = TRUE;
+                found = true;
                 foundTable = i;
                 break;
 
@@ -106,7 +106,7 @@ CMAPMapper *CMAPMapper::createUnicodeMapper(const CMAPTable *cmap)
             foundPlatformID = platformID;
             foundPlatformSpecificID = platformSpecificID;
             foundTable = i;
-            found = TRUE;
+            found = true;
             break;
 
           default: printf("Error: table %d (psid %d) is unknown. Skipping.\n", i, platformSpecificID); break;
@@ -121,14 +121,14 @@ CMAPMapper *CMAPMapper::createUnicodeMapper(const CMAPTable *cmap)
 
     if (found)
     {
-      subtable = (const CMAPEncodingSubtable *) ((const char *) cmap + foundOffset);
+      subtable = reinterpret_cast<const CMAPEncodingSubtable*>(reinterpret_cast<const char*>(cmap) + foundOffset);
       //printf("%s:%d: using subtable #%d/%d type %d:%d\n", __FILE__, __LINE__, foundTable, nSubtables, foundPlatformID, foundPlatformSpecificID);
       (void)foundPlatformID;   // Suppress unused variable compiler warnings.
       (void)foundTable;
       (void)foundPlatformSpecificID;
     } else {
       printf("%s:%d: could not find subtable.\n", __FILE__, __LINE__);
-      return NULL;
+      return nullptr;
     }
 
     le_uint16 tableFormat = SWAPW(subtable->format);
@@ -140,7 +140,7 @@ CMAPMapper *CMAPMapper::createUnicodeMapper(const CMAPTable *cmap)
 
     case 12:
     {
-        const CMAPFormat12Encoding *encoding = (const CMAPFormat12Encoding *) subtable;
+        const CMAPFormat12Encoding* encoding = reinterpret_cast<const CMAPFormat12Encoding*>(subtable);
 
         return new CMAPGroupMapper(cmap, encoding->groups, SWAPL(encoding->nGroups));
     }
@@ -150,7 +150,7 @@ CMAPMapper *CMAPMapper::createUnicodeMapper(const CMAPTable *cmap)
     }
 
     printf("%s:%d: Unknown format %x.\n", __FILE__, __LINE__, (SWAPW(subtable->format)));
-    return NULL;
+    return nullptr;
 }
 
 CMAPFormat4Mapper::CMAPFormat4Mapper(const CMAPTable *cmap, const CMAPFormat4Encoding *header)
@@ -172,7 +172,7 @@ LEGlyphID CMAPFormat4Mapper::unicodeToGlyph(LEUnicode32 unicode32) const
         return 0;
     }
 
-    LEUnicode16 unicode = (LEUnicode16) unicode32;
+    LEUnicode16 unicode = static_cast<LEUnicode16>(unicode32);
     le_uint16 index = 0;
     le_uint16 probe = 1 << fEntrySelector;
     TTGlyphID result = 0;
@@ -191,11 +191,11 @@ LEGlyphID CMAPFormat4Mapper::unicodeToGlyph(LEUnicode32 unicode32) const
 
     if (unicode >= SWAPU16(fStartCodes[index]) && unicode <= SWAPU16(fEndCodes[index])) {
         if (fIdRangeOffset[index] == 0) {
-            result = (TTGlyphID) unicode;
+            result = static_cast<TTGlyphID>(unicode);
         } else {
             le_uint16 offset = unicode - SWAPU16(fStartCodes[index]);
             le_uint16 rangeOffset = SWAPW(fIdRangeOffset[index]);
-            le_uint16 *glyphIndexTable = (le_uint16 *) ((char *) &fIdRangeOffset[index] + rangeOffset);
+            const le_uint16* glyphIndexTable = reinterpret_cast<const le_uint16*>(reinterpret_cast<const char*>(&fIdRangeOffset[index]) + rangeOffset);
 
             result = SWAPW(glyphIndexTable[offset]);
         }
@@ -239,7 +239,7 @@ LEGlyphID CMAPGroupMapper::unicodeToGlyph(LEUnicode32 unicode32) const
     }
 
     if (SWAPU32(fGroups[range].startCharCode) <= unicode32 && SWAPU32(fGroups[range].endCharCode) >= unicode32) {
-        return (LEGlyphID) (SWAPU32(fGroups[range].startGlyphCode) + unicode32 - SWAPU32(fGroups[range].startCharCode));
+        return static_cast<LEGlyphID>(SWAPU32(fGroups[range].startGlyphCode) + unicode32 - SWAPU32(fGroups[range].startCharCode));
     }
 
     return 0;
