@@ -1,29 +1,18 @@
-/*
- *  %W% %E%
- *
- * (C) Copyright IBM Corp. 1998 - All Rights Reserved
- *
- * Portions Copyright 1998 by Sun Microsystems, Inc.,
- * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
- * All rights reserved.
- *
- * This software is the confidential and proprietary information
- * of Sun Microsystems, Inc. ("Confidential Information").  You
- * shall not disclose such Confidential Information and shall use
- * it only in accordance with the terms of the license agreement
- * you entered into with Sun.
- *
- * The original version of this source code and documentation is
- * copyrighted and owned by IBM. These materials are provided
- * under terms of a License Agreement between IBM and Sun.
- * This technology is protected by multiple US and International
- * patents. This notice and attribution to IBM may not be removed.
- */
+// © 2016 and later: Unicode, Inc. and others.
+// License & terms of use: http://www.unicode.org/copyright.html
+/***************************************************************************
+*
+*   Copyright (C) 1998-2013, International Business Machines
+*   Corporation and others.  All Rights Reserved.
+*
+************************************************************************/
+
 
 #ifndef __CMAPS_H
 #define __CMAPS_H
 
-#include "LETypes.h"
+#include "layout/LETypes.h"
+//#include "letest.h"
 #include "sfnt.h"
 
 class CMAPMapper
@@ -38,7 +27,7 @@ public:
 protected:
     CMAPMapper(const CMAPTable *cmap);
 
-    CMAPMapper() {};
+    CMAPMapper() {}
 
 private:
     const CMAPTable *fcmap;
@@ -51,10 +40,10 @@ public:
 
     virtual ~CMAPFormat4Mapper();
 
-    virtual LEGlyphID unicodeToGlyph(LEUnicode32 unicode32) const;
+    LEGlyphID unicodeToGlyph(LEUnicode32 unicode32) const override;
 
 protected:
-    CMAPFormat4Mapper() {};
+    CMAPFormat4Mapper() {}
 
 private:
     le_uint16       fEntrySelector;
@@ -72,10 +61,10 @@ public:
 
     virtual ~CMAPGroupMapper();
 
-    virtual LEGlyphID unicodeToGlyph(LEUnicode32 unicode32) const;
+    LEGlyphID unicodeToGlyph(LEUnicode32 unicode32) const override;
 
 protected:
-    CMAPGroupMapper() {};
+    CMAPGroupMapper() {}
 
 private:
     le_int32 fPower;
@@ -91,7 +80,7 @@ inline CMAPMapper::CMAPMapper(const CMAPTable *cmap)
 
 inline CMAPMapper::~CMAPMapper()
 {
-    delete[] (char *) fcmap;
+    LE_DELETE_ARRAY(fcmap);
 }
 
 #endif
