@@ -20,6 +20,12 @@
 #include "unicode/uniset.h"
 #include "unicode/ustring.h"
 
+using icu::BreakIterator;
+using icu::Locale;
+using icu::UCharCharacterIterator;
+using icu::UnicodeSet;
+using icu::UnicodeString;
+
 /*
  * This program takes a Unicode text file containing Thai text with
  * spaces inserted where the word breaks are. It computes a copy of
@@ -40,7 +46,7 @@ public:
     // The constructor:
     // text  - pointer to an array of UChars to iterate over
     // count - the number of UChars in text
-    SpaceBreakIterator(const UChar *text, int32_t count);
+    SpaceBreakIterator(const char16_t *text, int32_t count);
 
     // the destructor
     ~SpaceBreakIterator();
@@ -61,8 +67,8 @@ private:
     // The underlying BreakIterator
     BreakIterator *fBreakIter;
 
-    // address of the UChar array
-    const UChar *fText;
+    // address of the char16_t array
+    const char16_t *fText;
 
     // number of UChars in fText
     int32_t fTextCount;
@@ -87,12 +93,12 @@ class ThaiWordbreakTest
 {
 public:
     // The main constructor:
-    // spaces       - pointer to a UChar array for the text with spaces
+    // spaces       - pointer to a char16_t array for the text with spaces
     // spaceCount   - the number of characters in the spaces array
-    // noSpaces     - pointer to a UChar array for the text without spaces
+    // noSpaces     - pointer to a char16_t array for the text without spaces
     // noSpaceCount - the number of characters in the noSpaces array
     // verbose      - report all breaks if true, otherwise just report differences
-    ThaiWordbreakTest(const UChar *spaces, int32_t spaceCount, const UChar *noSpaces, int32_t noSpaceCount, UBool verbose);
+    ThaiWordbreakTest(const char16_t *spaces, int32_t spaceCount, const char16_t *noSpaces, int32_t noSpaceCount, UBool verbose);
     ~ThaiWordbreakTest();
 
     // returns the number of breaks that are in the spaces array
@@ -109,28 +115,28 @@ public:
     // reads the input Unicode text file:
     // fileName  - the path name of the file
     // charCount - set to the number of UChars read from the file
-    // returns   - the address of the UChar array containing the characters
-    static const UChar *readFile(char *fileName, int32_t &charCount);
+    // returns   - the address of the char16_t array containing the characters
+    static const char16_t *readFile(const char *fileName, int32_t &charCount);
 
-    // removes spaces form the input UChar array:
-    // spaces        - pointer to the input UChar array
+    // removes spaces form the input char16_t array:
+    // spaces        - pointer to the input char16_t array
     // count         - number of UChars in the spaces array
     // nonSpaceCount - the number of UChars in the result array
-    // returns       - the address of the UChar array with spaces removed
-    static const UChar *crunchSpaces(const UChar *spaces, int32_t count, int32_t &nonSpaceCount);
+    // returns       - the address of the char16_t array with spaces removed
+    static const char16_t *crunchSpaces(const char16_t *spaces, int32_t count, int32_t &nonSpaceCount);
 
 private:
     // The no arg constructor - private so clients can't call it
     ThaiWordbreakTest();
 
     // This does the actual comparison:
-    // spaces - the address of the UChar array for the text with spaces
+    // spaces - the address of the char16_t array for the text with spaces
     // spaceCount - the number of UChars in the spaces array
-    // noSpaces   - the address of the UChar array for the text without spaces
+    // noSpaces   - the address of the char16_t array for the text without spaces
     // noSpaceCount - the number of UChars in the noSpaces array
     // returns      - true if all breaks match, false otherwise
-    UBool compareWordBreaks(const UChar *spaces, int32_t spaceCount,
-                            const UChar *noSpaces, int32_t noSpaceCount);
+    UBool compareWordBreaks(const char16_t *spaces, int32_t spaceCount,
+                            const char16_t *noSpaces, int32_t noSpaceCount);
 
     // helper method to report a break in the spaces
     // array that's not found in the noSpaces array
@@ -158,8 +164,8 @@ private:
 /*
  * The main constructor: it calls compareWordBreaks and reports any differences
  */
-ThaiWordbreakTest::ThaiWordbreakTest(const UChar *spaces, int32_t spaceCount,
-                                     const UChar *noSpaces, int32_t noSpaceCount, UBool verbose)
+ThaiWordbreakTest::ThaiWordbreakTest(const char16_t *spaces, int32_t spaceCount,
+                                     const char16_t *noSpaces, int32_t noSpaceCount, UBool verbose)
 : fBreaksNotFound(0), fInvalidBreaks(0), fWordCount(0), fVerbose(verbose)
 {
     compareWordBreaks(spaces, spaceCount, noSpaces, noSpaceCount);
@@ -213,8 +219,8 @@ inline int32_t ThaiWordbreakTest::getWordCount()
  * and a word instance of a Thai BreakIterator to iterate over the text
  * without spaces.
  */
-UBool ThaiWordbreakTest::compareWordBreaks(const UChar *spaces, int32_t spaceCount,
-                                           const UChar *noSpaces, int32_t noSpaceCount)
+UBool ThaiWordbreakTest::compareWordBreaks(const char16_t *spaces, int32_t spaceCount,
+                                           const char16_t *noSpaces, int32_t noSpaceCount)
 {
     UBool result = true;
     Locale thai("th");
@@ -304,19 +310,19 @@ void ThaiWordbreakTest::foundInvalidBreak(int32_t br)
  * Read the text from a file. The text must start with a Unicode Byte
  * Order Mark (BOM) so that we know what order to read the bytes in.
  */
-const UChar *ThaiWordbreakTest::readFile(char *fileName, int32_t &charCount)
+const char16_t *ThaiWordbreakTest::readFile(const char *fileName, int32_t &charCount)
 {
     FILE *f;
     int32_t fileSize;
     
-    UChar *buffer;
+    char16_t *buffer;
     char *bufferChars;
     
     f = fopen(fileName, "rb");
     
-    if( f == NULL ) {
+    if( f == nullptr ) {
         fprintf(stderr,"Couldn't open %s reason: %s \n", fileName, strerror(errno));
-        return 0;
+        return nullptr;
     }
     
     fseek(f, 0, SEEK_END);
@@ -324,11 +330,11 @@ const UChar *ThaiWordbreakTest::readFile(char *fileName, int32_t &charCount)
     
     fseek(f, 0, SEEK_SET);
     bufferChars = new char[fileSize];
-    
-    if(bufferChars == 0) {
+
+    if (bufferChars == nullptr) {
         fprintf(stderr,"Couldn't get memory for reading %s reason: %s \n", fileName, strerror(errno));
         fclose(f);
-        return 0;
+        return nullptr;
     }
     
     fread(bufferChars, sizeof(char), fileSize, f);
@@ -336,7 +342,7 @@ const UChar *ThaiWordbreakTest::readFile(char *fileName, int32_t &charCount)
         fprintf(stderr,"Couldn't read %s reason: %s \n", fileName, strerror(errno));
         fclose(f);
         delete[] bufferChars;
-        return 0;
+        return nullptr;
     }
     fclose(f);
     
@@ -345,27 +351,27 @@ const UChar *ThaiWordbreakTest::readFile(char *fileName, int32_t &charCount)
     delete[] bufferChars;
     
     charCount = myText.length();
-    buffer = new UChar[charCount];
-    if(buffer == 0) {
+    buffer = new char16_t[charCount];
+    if (buffer == nullptr) {
         fprintf(stderr,"Couldn't get memory for reading %s reason: %s \n", fileName, strerror(errno));
-        return 0;
+        return nullptr;
     }
     
     myText.extract(1, myText.length(), buffer);
     charCount--;  // skip the BOM
-    buffer[charCount] = 0;    // NULL terminate for easier reading in the debugger
+    buffer[charCount] = 0;    // NUL terminate for easier reading in the debugger
     
     return buffer;
 }
 
 /*
- * Remove spaces from the input UChar array.
+ * Remove spaces from the input char16_t array.
  *
  * We check explicitly for a Unicode code value of 0x0020
  * because Unicode::isSpaceChar returns true for CR, LF, etc.
  *
  */
-const UChar *ThaiWordbreakTest::crunchSpaces(const UChar *spaces, int32_t count, int32_t &nonSpaceCount)
+const char16_t *ThaiWordbreakTest::crunchSpaces(const char16_t *spaces, int32_t count, int32_t &nonSpaceCount)
 {
     int32_t i, out, spaceCount;
 
@@ -377,11 +383,11 @@ const UChar *ThaiWordbreakTest::crunchSpaces(const UChar *spaces, int32_t count,
     }
 
     nonSpaceCount = count - spaceCount;
-    UChar *noSpaces = new UChar[nonSpaceCount];
+    char16_t *noSpaces = new char16_t[nonSpaceCount];
 
-    if (noSpaces == 0) {
+    if (noSpaces == nullptr) {
         fprintf(stderr, "Couldn't allocate memory for the space stripped text.\n");
-        return 0;
+        return nullptr;
     }
 
     for (out = 0, i = 0; i < count; i += 1) {
@@ -396,7 +402,7 @@ const UChar *ThaiWordbreakTest::crunchSpaces(const UChar *spaces, int32_t count,
 /*
  * Generate a text file with spaces in it from a file without.
  */
-int generateFile(const UChar *chars, int32_t length) {
+int generateFile(const char16_t *chars, int32_t length) {
     Locale root("");
     UCharCharacterIterator *noSpaceIter = new UCharCharacterIterator(chars, length);
     UErrorCode status = U_ZERO_ERROR;
@@ -406,7 +412,7 @@ int generateFile(const UChar *chars, int32_t length) {
     breakIter->adoptText(noSpaceIter);
     char outbuf[1024];
     int32_t strlength;
-    UChar bom = 0xFEFF;
+    char16_t bom = 0xFEFF;
     
     printf("%s", u_strToUTF8(outbuf, sizeof(outbuf), &strlength, &bom, 1, &status));
     int32_t prevbreak = 0;
@@ -439,7 +445,7 @@ int generateFile(const UChar *chars, int32_t length) {
  */
 int main(int argc, char **argv)
 {
-    char *fileName = "space.txt";
+    const char *fileName = "space.txt";
     int arg = 1;
     UBool verbose = false;
     UBool generate = false;
@@ -464,11 +470,11 @@ int main(int argc, char **argv)
     }
 
     int32_t spaceCount, nonSpaceCount;
-    const UChar *spaces, *noSpaces;
+    const char16_t *spaces, *noSpaces;
 
     spaces = ThaiWordbreakTest::readFile(fileName, spaceCount);
 
-    if (spaces == 0) {
+    if (spaces == nullptr) {
         return 1;
     }
     
@@ -478,7 +484,7 @@ int main(int argc, char **argv)
 
     noSpaces = ThaiWordbreakTest::crunchSpaces(spaces, spaceCount, nonSpaceCount);
 
-    if (noSpaces == 0) {
+    if (noSpaces == nullptr) {
         return 1;
     }
 
@@ -495,8 +501,8 @@ int main(int argc, char **argv)
  * The main constructor. Clear all the counts and construct a default
  * word instance of a BreakIterator.
  */
-SpaceBreakIterator::SpaceBreakIterator(const UChar *text, int32_t count)
-  : fBreakIter(0), fText(text), fTextCount(count), fWordCount(0), fSpaceCount(0), fDone(false)
+SpaceBreakIterator::SpaceBreakIterator(const char16_t *text, int32_t count)
+  : fBreakIter(nullptr), fText(text), fTextCount(count), fWordCount(0), fSpaceCount(0), fDone(false)
 {
     UCharCharacterIterator *iter = new UCharCharacterIterator(text, count);
     UErrorCode status = U_ZERO_ERROR;
