@@ -11,8 +11,8 @@
 #define UCLN_TYPE UCLN_TOOLUTIL
 #include "ucln_imp.h"
 
-int uprv_dummyFunction_TU(void);
-int uprv_dummyFunction_TU(void)
+int uprv_dummyFunction_TU();
+int uprv_dummyFunction_TU()
 {
   /* this is here to prevent the compiler from complaining about an empty file */
   return 0;
