@@ -146,14 +146,14 @@ UXMLParser::UXMLParser(UErrorCode &status) :
 
       fNames(status),
       fElementStack(status),
-      fOneLF((UChar)0x0a)        // Plain new-line string, used in new line normalization.
+      fOneLF(static_cast<char16_t>(0x0a)) // Plain new-line string, used in new line normalization.
       {
       }
 
 UXMLParser *
 UXMLParser::createParser(UErrorCode &errorCode) {
     if (U_FAILURE(errorCode)) {
-        return NULL;
+        return nullptr;
     } else {
         return new UXMLParser(errorCode);
     }
@@ -168,22 +168,22 @@ UXMLParser::parseFile(const char *filename, UErrorCode &errorCode) {
     const char *charset, *pb;
     UnicodeString src;
     UConverter *cnv;
-    UChar *buffer, *pu;
+    char16_t *buffer, *pu;
     int32_t fileLength, bytesLength, length, capacity;
     UBool flush;
 
     if(U_FAILURE(errorCode)) {
-        return NULL;
+        return nullptr;
     }
 
     f=T_FileStream_open(filename, "rb");
-    if(f==NULL) {
+    if(f==nullptr) {
         errorCode=U_FILE_ACCESS_ERROR;
-        return NULL;
+        return nullptr;
     }
 
-    bytesLength=T_FileStream_read(f, bytes, (int32_t)sizeof(bytes));
-    if(bytesLength<(int32_t)sizeof(bytes)) {
+    bytesLength = T_FileStream_read(f, bytes, static_cast<int32_t>(sizeof(bytes)));
+    if (bytesLength < static_cast<int32_t>(sizeof(bytes))) {
         // we have already read the entire file
         fileLength=bytesLength;
     } else {
@@ -197,8 +197,8 @@ UXMLParser::parseFile(const char *filename, UErrorCode &errorCode) {
      * 2. treat as ISO-8859-1 and read XML encoding="charser"
      * 3. default to UTF-8
      */
-    charset=ucnv_detectUnicodeSignature(bytes, bytesLength, NULL, &errorCode);
-    if(U_SUCCESS(errorCode) && charset!=NULL) {
+    charset=ucnv_detectUnicodeSignature(bytes, bytesLength, nullptr, &errorCode);
+    if(U_SUCCESS(errorCode) && charset!=nullptr) {
         // open converter according to Unicode signature
         cnv=ucnv_open(charset, &errorCode);
     } else {
@@ -210,7 +210,7 @@ UXMLParser::parseFile(const char *filename, UErrorCode &errorCode) {
         }
 
         buffer=toUCharPtr(src.getBuffer(bytesLength));
-        if(buffer==NULL) {
+        if(buffer==nullptr) {
             // unexpected failure to reserve some string capacity
             errorCode=U_MEMORY_ALLOCATION_ERROR;
             goto exit;
@@ -221,10 +221,10 @@ UXMLParser::parseFile(const char *filename, UErrorCode &errorCode) {
             cnv,
             &pu, buffer+src.getCapacity(),
             &pb, bytes+bytesLength,
-            NULL, true, &errorCode);
-        src.releaseBuffer(U_SUCCESS(errorCode) ? (int32_t)(pu-buffer) : 0);
+            nullptr, true, &errorCode);
+        src.releaseBuffer(U_SUCCESS(errorCode) ? static_cast<int32_t>(pu - buffer) : 0);
         ucnv_close(cnv);
-        cnv=NULL;
+        cnv=nullptr;
         if(U_FAILURE(errorCode)) {
             // unexpected error in conversion from Latin-1
             src.remove();
@@ -235,7 +235,7 @@ UXMLParser::parseFile(const char *filename, UErrorCode &errorCode) {
         if(mXMLDecl.reset(src).lookingAt(0, errorCode)) {
             int32_t declEnd=mXMLDecl.end(errorCode);
             // go beyond <?xml
-            int32_t pos=src.indexOf((UChar)x_l)+1;
+            int32_t pos = src.indexOf(static_cast<char16_t>(x_l)) + 1;
 
             mAttrValue.reset(src);
             while(pos<declEnd && mAttrValue.lookingAt(pos, errorCode)) {  // loop runs once per attribute on this element.
@@ -248,14 +248,14 @@ UXMLParser::parseFile(const char *filename, UErrorCode &errorCode) {
                 attValue.truncate(attValue.length()-1);  // and one from the end.
 
                 if(attName==UNICODE_STRING("encoding", 8)) {
-                    length=attValue.extract(0, 0x7fffffff, charsetBuffer, (int32_t)sizeof(charsetBuffer));
+                    length = attValue.extract(0, 0x7fffffff, charsetBuffer, static_cast<int32_t>(sizeof(charsetBuffer)));
                     charset=charsetBuffer;
                     break;
                 }
                 pos = mAttrValue.end(2, errorCode);
             }
 
-            if(charset==NULL) {
+            if(charset==nullptr) {
                 // default to UTF-8
                 charset="UTF-8";
             }
@@ -279,7 +279,7 @@ UXMLParser::parseFile(const char *filename, UErrorCode &errorCode) {
         for(;;) {
             length=src.length();
             buffer=toUCharPtr(src.getBuffer(capacity));
-            if(buffer==NULL) {
+            if(buffer==nullptr) {
                 // unexpected failure to reserve some string capacity
                 errorCode=U_MEMORY_ALLOCATION_ERROR;
                 goto exit;
@@ -289,8 +289,8 @@ UXMLParser::parseFile(const char *filename, UErrorCode &errorCode) {
             ucnv_toUnicode(
                 cnv, &pu, buffer+src.getCapacity(),
                 &pb, bytes+bytesLength,
-                NULL, false, &errorCode);
-            src.releaseBuffer(U_SUCCESS(errorCode) ? (int32_t)(pu-buffer) : 0);
+                nullptr, false, &errorCode);
+            src.releaseBuffer(U_SUCCESS(errorCode) ? static_cast<int32_t>(pu - buffer) : 0);
             if(errorCode==U_BUFFER_OVERFLOW_ERROR) {
                 errorCode=U_ZERO_ERROR;
                 capacity=(3*src.getCapacity())/2; // increase capacity by 50%
@@ -308,7 +308,7 @@ UXMLParser::parseFile(const char *filename, UErrorCode &errorCode) {
         }
 
         // read next block
-        bytesLength=T_FileStream_read(f, bytes, (int32_t)sizeof(bytes));
+        bytesLength = T_FileStream_read(f, bytes, static_cast<int32_t>(sizeof(bytes)));
         if(bytesLength==0) {
             // reached end of file, convert once more to flush the converter
             flush=true;
@@ -322,17 +322,17 @@ exit:
     if(U_SUCCESS(errorCode)) {
         return parse(src, errorCode);
     } else {
-        return NULL;
+        return nullptr;
     }
 }
 
 UXMLElement *
 UXMLParser::parse(const UnicodeString &src, UErrorCode &status) {
     if(U_FAILURE(status)) {
-        return NULL;
+        return nullptr;
     }
 
-    UXMLElement   *root = NULL;
+    UXMLElement   *root = nullptr;
     fPos = 0; // TODO use just a local pos variable and pass it into functions
               // where necessary?
 
@@ -435,10 +435,10 @@ UXMLParser::parse(const UnicodeString &src, UErrorCode &status) {
                 }
                 if (fElementStack.empty()) {
                     // Close of the root element.  We're done with the doc.
-                    el = NULL;
+                    el = nullptr;
                     break;
                 }
-                el = (UXMLElement *)fElementStack.pop();
+                el = static_cast<UXMLElement*>(fElementStack.pop());
                 continue;
             }
 
@@ -455,7 +455,7 @@ UXMLParser::parse(const UnicodeString &src, UErrorCode &status) {
             break;
         }
 
-        if (el != NULL || !fElementStack.empty()) {
+        if (el != nullptr || !fElementStack.empty()) {
             // We bailed out early, for some reason.
             error("Root element not closed.", status);
             goto errorExit;
@@ -477,7 +477,7 @@ UXMLParser::parse(const UnicodeString &src, UErrorCode &status) {
 
 errorExit:
     delete root;
-    return NULL;
+    return nullptr;
 }
 
 //
@@ -514,7 +514,7 @@ UXMLParser::createElement(RegexMatcher  &mEl, UErrorCode &status) {
 
         // Next change all xml white space chars to plain \u0020 spaces.
         mAttrNormalizer.reset(attValue);
-        UnicodeString oneSpace((UChar)0x0020);
+        UnicodeString oneSpace(static_cast<char16_t>(0x0020));
         attValue = mAttrNormalizer.replaceAll(oneSpace, status);
 
         // Replace character entities.
@@ -565,7 +565,7 @@ UnicodeString
 UXMLParser::scanContent(UErrorCode &status) {
     UnicodeString  result;
     if (mXMLCharData.lookingAt(fPos, status)) {
-        result = mXMLCharData.group((int32_t)0, status);
+        result = mXMLCharData.group(static_cast<int32_t>(0), status);
         // Normalize the new-lines.  (Before char ref substitution)
         mNewLineNormalizer.reset(result);
         result = mNewLineNormalizer.replaceAll(fOneLF, status);
@@ -595,15 +595,15 @@ UXMLParser::replaceCharRefs(UnicodeString &s, UErrorCode &status) {
     //      which is flagged by start() of that group not being -1.
     while (mAmps.find()) {
         if (mAmps.start(1, status) != -1) {
-            replacement.setTo((UChar)x_AMP);
+            replacement.setTo(static_cast<char16_t>(x_AMP));
         } else if (mAmps.start(2, status) != -1) {
-            replacement.setTo((UChar)x_LT);
+            replacement.setTo(static_cast<char16_t>(x_LT));
         } else if (mAmps.start(3, status) != -1) {
-            replacement.setTo((UChar)x_GT);
+            replacement.setTo(static_cast<char16_t>(x_GT));
         } else if (mAmps.start(4, status) != -1) {
-            replacement.setTo((UChar)x_APOS);
+            replacement.setTo(static_cast<char16_t>(x_APOS));
         } else if (mAmps.start(5, status) != -1) {
-            replacement.setTo((UChar)x_QUOT);
+            replacement.setTo(static_cast<char16_t>(x_QUOT));
         } else if (mAmps.start(6, status) != -1) {
             UnicodeString hexString = mAmps.group(6, status);
             UChar32 val = 0;
@@ -624,7 +624,7 @@ UXMLParser::replaceCharRefs(UnicodeString &s, UErrorCode &status) {
             // An unrecognized &entity;  Leave it alone.
             //  TODO:  check that it really looks like an entity, and is not some
             //         random & in the text.
-            replacement = mAmps.group((int32_t)0, status);
+            replacement = mAmps.group(static_cast<int32_t>(0), status);
         }
         mAmps.appendReplacement(result, replacement, status);
     }
@@ -639,7 +639,7 @@ UXMLParser::error(const char *message, UErrorCode &status) {
     int  line = 0;
     int  ci = 0;
     while (ci < fPos && ci>=0) {
-        ci = src.indexOf((UChar)0x0a, ci+1);
+        ci = src.indexOf(static_cast<char16_t>(0x0a), ci + 1);
         line++;
     }
     fprintf(stderr, "Error: %s at line %d\n", message, line);
@@ -653,26 +653,26 @@ UXMLParser::error(const char *message, UErrorCode &status) {
 const UnicodeString *
 UXMLParser::intern(const UnicodeString &s, UErrorCode &errorCode) {
     const UHashElement *he=fNames.find(s);
-    if(he!=NULL) {
+    if(he!=nullptr) {
         // already a known name, return its hashed key pointer
-        return (const UnicodeString *)he->key.pointer;
+        return static_cast<const UnicodeString*>(he->key.pointer);
     } else {
         // add this new name and return its hashed key pointer
         fNames.puti(s, 1, errorCode);
         he=fNames.find(s);
-        return (const UnicodeString *)he->key.pointer;
+        return static_cast<const UnicodeString*>(he->key.pointer);
     }
 }
 
 const UnicodeString *
 UXMLParser::findName(const UnicodeString &s) const {
     const UHashElement *he=fNames.find(s);
-    if(he!=NULL) {
+    if(he!=nullptr) {
         // a known name, return its hashed key pointer
-        return (const UnicodeString *)he->key.pointer;
+        return static_cast<const UnicodeString*>(he->key.pointer);
     } else {
         // unknown name
-        return NULL;
+        return nullptr;
     }
 }
 
@@ -684,7 +684,7 @@ UXMLElement::UXMLElement(const UXMLParser *parser, const UnicodeString *name, UE
    fAttNames(errorCode),
    fAttValues(errorCode),
    fChildren(errorCode),
-   fParent(NULL)
+   fParent(nullptr)
 {
 }
 
@@ -692,10 +692,10 @@ UXMLElement::~UXMLElement() {
     int   i;
     // attribute names are owned by the UXMLParser, don't delete them here
     for (i=fAttValues.size()-1; i>=0; i--) {
-        delete (UObject *)fAttValues.elementAt(i);
+        delete static_cast<UObject*>(fAttValues.elementAt(i));
     }
     for (i=fChildren.size()-1; i>=0; i--) {
-        delete (UObject *)fChildren.elementAt(i);
+        delete static_cast<UObject*>(fChildren.elementAt(i));
     }
 }
 
@@ -716,9 +716,9 @@ UXMLElement::appendText(UnicodeString &text, UBool recurse) const {
     const UObject *node;
     int32_t i, count=fChildren.size();
     for(i=0; i<count; ++i) {
-        node=(const UObject *)fChildren.elementAt(i);
+        node = static_cast<const UObject*>(fChildren.elementAt(i));
         const UnicodeString *s=dynamic_cast<const UnicodeString *>(node);
-        if(s!=NULL) {
+        if(s!=nullptr) {
             text.append(*s);
         } else if(recurse) /* must be a UXMLElement */ {
             ((const UXMLElement *)node)->appendText(text, recurse);
@@ -734,11 +734,11 @@ UXMLElement::countAttributes() const {
 const UnicodeString *
 UXMLElement::getAttribute(int32_t i, UnicodeString &name, UnicodeString &value) const {
     if(0<=i && i<fAttNames.size()) {
-        name.setTo(*(const UnicodeString *)fAttNames.elementAt(i));
-        value.setTo(*(const UnicodeString *)fAttValues.elementAt(i));
+        name.setTo(*static_cast<const UnicodeString*>(fAttNames.elementAt(i)));
+        value.setTo(*static_cast<const UnicodeString*>(fAttValues.elementAt(i)));
         return &value; // or return (UnicodeString *)fAttValues.elementAt(i);
     } else {
-        return NULL;
+        return nullptr;
     }
 }
 
@@ -747,17 +747,17 @@ UXMLElement::getAttribute(const UnicodeString &name) const {
     // search for the attribute name by comparing the interned pointer,
     // not the string contents
     const UnicodeString *p=fParser->findName(name);
-    if(p==NULL) {
-        return NULL; // no such attribute seen by the parser at all
+    if(p==nullptr) {
+        return nullptr; // no such attribute seen by the parser at all
     }
 
     int32_t i, count=fAttNames.size();
     for(i=0; i<count; ++i) {
-        if(p==(const UnicodeString *)fAttNames.elementAt(i)) {
-            return (const UnicodeString *)fAttValues.elementAt(i);
+        if (p == static_cast<const UnicodeString*>(fAttNames.elementAt(i))) {
+            return static_cast<const UnicodeString*>(fAttValues.elementAt(i));
         }
     }
-    return NULL;
+    return nullptr;
 }
 
 int32_t
@@ -768,34 +768,34 @@ UXMLElement::countChildren() const {
 const UObject *
 UXMLElement::getChild(int32_t i, UXMLNodeType &type) const {
     if(0<=i && i<fChildren.size()) {
-        const UObject *node=(const UObject *)fChildren.elementAt(i);
-        if(dynamic_cast<const UXMLElement *>(node)!=NULL) {
+        const UObject* node = static_cast<const UObject*>(fChildren.elementAt(i));
+        if(dynamic_cast<const UXMLElement *>(node)!=nullptr) {
             type=UXML_NODE_TYPE_ELEMENT;
         } else {
             type=UXML_NODE_TYPE_STRING;
         }
         return node;
     } else {
-        return NULL;
+        return nullptr;
     }
 }
 
 const UXMLElement *
 UXMLElement::nextChildElement(int32_t &i) const {
     if(i<0) {
-        return NULL;
+        return nullptr;
     }
 
     const UObject *node;
     int32_t count=fChildren.size();
     while(i<count) {
-        node=(const UObject *)fChildren.elementAt(i++);
+        node = static_cast<const UObject*>(fChildren.elementAt(i++));
         const UXMLElement *elem=dynamic_cast<const UXMLElement *>(node);
-        if(elem!=NULL) {
+        if(elem!=nullptr) {
             return elem;
         }
     }
-    return NULL;
+    return nullptr;
 }
 
 const UXMLElement *
@@ -803,22 +803,22 @@ UXMLElement::getChildElement(const UnicodeString &name) const {
     // search for the element name by comparing the interned pointer,
     // not the string contents
     const UnicodeString *p=fParser->findName(name);
-    if(p==NULL) {
-        return NULL; // no such element seen by the parser at all
+    if(p==nullptr) {
+        return nullptr; // no such element seen by the parser at all
     }
 
     const UObject *node;
     int32_t i, count=fChildren.size();
     for(i=0; i<count; ++i) {
-        node=(const UObject *)fChildren.elementAt(i);
+        node = static_cast<const UObject*>(fChildren.elementAt(i));
         const UXMLElement *elem=dynamic_cast<const UXMLElement *>(node);
-        if(elem!=NULL) {
+        if(elem!=nullptr) {
             if(p==elem->fName) {
                 return elem;
             }
         }
     }
-    return NULL;
+    return nullptr;
 }
 
 U_NAMESPACE_END
